/*+
 * Name:	funct_type.c
 *
 * Purpose:	Transforms gcc function_type and mthod_type to datrix cfcttype.
 *              -- Turns tree list for parameters into arcSons that give
 *                 the parameter list
 *              -- turns retn link into an cInstance edge.
 *              Note: gcc uses _type nodes for declarations as well. This
 *              transfrom produces c_fct_type nodes for these as well.
 *              They will be cleaned up later when function declarations
 *              are handled.
 *
 * Author:	Thomas R. Dean
 *
 * Date:	March 7, 2001
 *
-*/

#include <stdio.h>
#include <stdlib.h>

#include "gd_graph.h"
#include "gd_utils.h"

char * progname;

void do_function_type(gd_graph *g, node_ptr i);
void do_method_type(gd_graph *g, node_ptr i);

void usage(){
  fprintf(stderr,"usage: %s infile outfile\n",progname);
  exit(1);
}

str_ptr void_str = -1;

int main(int argc, char * argv[]){
  gd_graph g;
  char * filename, *outfilename;
  int nodenum,i, count;
  progname = argv[0];

  if (argc != 3) {
    usage();
  }

  filename = argv[1];
  outfilename = argv[2];

  g = gd_graph_read_binary_file(filename,1.0,1.0,1.0);
  if (g.curStringPos == 0) {
    fprintf(stderr,"could not get graph\n");
    exit(1);
  }

  void_str = gd_lookup_string(&g,"void");

  for(i = 0; i < g.curNodePos;i++){
    if (g.nodeBuffer[i].common.type == gFUNCTION_TYPE){
      do_function_type(&g,i);
    }
    if (g.nodeBuffer[i].common.type == gMETHOD_TYPE){
      do_method_type(&g,i);
    }
  }
  /* all done, write the file */
  if (gd_graph_write_binary_file(outfilename,g) == 0){
    fprintf(stderr,"problem writing file\n");
    exit(1);
  }
  return 0;
}

void do_function_type(gd_graph *g, node_ptr i){

  /*
   * fields to save
   */

  /* from type decl */
  node_ptr decl;	/* points to decl if it exists   */
  str_ptr srcf;		/*  -- src file from decl        */
  int srcl;		/*  -- src line from decl        */
  node_ptr name;	/*  -- identifier node from decl */
  str_ptr  namestr;	/*     -- str_ptr for name       */

  /* from function type */
  node_ptr parms;	/* pointer to parameters */
  node_ptr retn;	/* pointer to return type */
  			/* size field is ignored */

  /* other locals */
  node_ptr new_node, 	/* two variables for node */
  	new_node2,	/*  allocation */
	next_node,	/* used when traversing tree_list */
	prev_node,
	value_node;	/* used when traversing tree_list */

  str_ptr param_name;
  char param_str[4]; /* pNN */

  /* go get decl information if it exists */
  decl = g->nodeBuffer[i].type.name;
  if (decl != -1) {
    if (g->nodeBuffer[decl].common.type != gTYPE_DECL) {
      fprintf(stderr,"type name that is not a decl!!(%d)\n",i);
      exit(1);
    }
    srcf = g->nodeBuffer[decl].decl.srcf;
    srcl = g->nodeBuffer[decl].decl.srcl;
    name = g->nodeBuffer[decl].decl.name;
    if (name != -1){
      if (g->nodeBuffer[name].common.type != gIDENTIFIER_NODE) {
	fprintf(stderr,"decl name that is not a identifier!!(%d)\n",i);
	exit(1);
      }
      namestr = g->nodeBuffer[name].identifier_node.strg;
    } else {
      namestr = -1;
    }
  } else {
    /* decl doesn't exist */
    srcf = -1;
    srcl = -1;
    name = -1;
    namestr = -1;
  }

  parms = g->nodeBuffer[i].function_type.prms;
  retn  = g->nodeBuffer[i].function_type.retn;

  /****************************
   * generate datrix nodes
   ****************************/

  /* switch node type */
  g->nodeBuffer[i].common.type = dC_FCT_TYPE;

  /* determin datrix visibility from common flags */
  /* gd_common doesn't change, no need to have saved flags */

  g->nodeBuffer[i].dc_identifier.visb = visb_unk;
  if (g->nodeBuffer[i].common.private_flag){
    g->nodeBuffer[i].dc_identifier.visb = visb_pri;
  }
  if (g->nodeBuffer[i].common.public_flag){
    g->nodeBuffer[i].dc_identifier.visb = visb_pub;
  }
  if (g->nodeBuffer[i].common.protected_flag){
    g->nodeBuffer[i].dc_identifier.visb = visb_pro;
  }

  /* other identifier and asg attributes */
  g->nodeBuffer[i].dc_asg_nd.line = srcl;
  g->nodeBuffer[i].dc_asg_nd.file = srcf;
  g->nodeBuffer[i].dc_identifier.name = namestr;

  /* unused fields */
  g->nodeBuffer[i].dc_identifier.defLoc = -1;
  g->nodeBuffer[i].dc_identifier.arcFriend = -1;
  g->nodeBuffer[i].dc_type.implements = -1;

  /* handle return type of function type */
  new_node = newNode(g);
  if (new_node == -1){
    fprintf(stderr,"out of nodes\n");
    exit(1);
  }
  g-> nodeBuffer[new_node].common.type = dC_INSTANCE;
  g-> nodeBuffer[new_node].dc_arc.ordinal = 0;
  g-> nodeBuffer[new_node].dc_arc.next = -1;
  g-> nodeBuffer[new_node].dc_arc.src = i;
  g-> nodeBuffer[new_node].dc_arc.dest = retn;
  g-> nodeBuffer[i].dc_identifier.instance = new_node;

  g-> nodeBuffer[i].dc_fct_type.formalParams = -1;

  /* parameters */
  /* since more than on funct_type may share a parameter list, check
   * to see if parameter list has been already converted */

  /* perhaps there's no parms list at all., ajm 2001 05 11 */
  if (0 <= parms && parms < g-> curNodePos)
      if (g->nodeBuffer[parms].common.type == gTREE_LIST) {
	/* gcc type parameters, need to be converted */
	if (parms != -1){
	  while(parms != -1){
	    next_node = g-> nodeBuffer[parms].tree_list.chan;
	    value_node = g -> nodeBuffer[parms].tree_list.value;
	    if (g->nodeBuffer[value_node].common.type != gVOID_TYPE &&
		!(g->nodeBuffer[value_node].common.type == dC_BUILT_IN_TYPE &&
		  g->nodeBuffer[value_node].dc_identifier.name == void_str)){
	      /* check for built-in-type and name == void ?? */
	      new_node = newNode(g);
	      if (new_node == -1){
		fprintf(stderr,"out of nodes\n");
		exit(1);
	      }
	      g->nodeBuffer[parms].common.type = dC_ARC_SON;
	      g->nodeBuffer[parms].dc_arc.ordinal = 0;
	      g->nodeBuffer[parms].dc_arc.src = i;
	      g->nodeBuffer[parms].dc_arc.dest = new_node;
	      g->nodeBuffer[parms].dc_arc.next = -1;
	      if(g->nodeBuffer[i].dc_fct_type.formalParams == -1){
		g->nodeBuffer[i].dc_fct_type.formalParams = parms;
	      } else {
		g->nodeBuffer[prev_node].dc_arc.next = parms;
	      }
	      prev_node = parms;
	      g-> nodeBuffer[new_node].common = g-> nodeBuffer[i].common;
	      g-> nodeBuffer[new_node].common.type = dC_FORMAL_FCT_PARAM;
	      g-> nodeBuffer[new_node].common.other_node = -1;
	      g-> nodeBuffer[new_node].dc_identifier.visb =
		    g-> nodeBuffer[i].dc_identifier.visb;
	      g-> nodeBuffer[new_node].dc_asg_nd.line =
		    g-> nodeBuffer[i].dc_asg_nd.line;
	      g-> nodeBuffer[new_node].dc_asg_nd.file =
		    g-> nodeBuffer[i].dc_asg_nd.file;
		g-> nodeBuffer[new_node].dc_identifier.name = -1; /* should generate name? */
	      g-> nodeBuffer[new_node].dc_identifier.defLoc = -1;
	      g-> nodeBuffer[new_node].dc_identifier.arcFriend = -1;
	      g-> nodeBuffer[new_node].dc_object.static_flag = 0;
	      g-> nodeBuffer[new_node].dc_object.extern_flag = 0;
	      g-> nodeBuffer[new_node].dc_object.const_flag = 0; /* wrong */
	      g-> nodeBuffer[new_node].dc_identifier.array_dim = -1;/* wrong */
	      g-> nodeBuffer[new_node].dc_object.bitfield_dim = -1;
	      g-> nodeBuffer[new_node].dc_object.initial_val = -1;

	      new_node2 = newNode(g);
	      if (new_node2 == -1){
		fprintf(stderr,"out of nodes\n");
		exit(1);
	      }
	      g-> nodeBuffer[new_node2].common.other_node = -1;
	      g-> nodeBuffer[new_node2].common.type = dC_INSTANCE;
	      g-> nodeBuffer[new_node2].dc_arc.ordinal = 0;
	      g-> nodeBuffer[new_node2].dc_arc.src = new_node;
	      g-> nodeBuffer[new_node2].dc_arc.dest = value_node;
	      g-> nodeBuffer[new_node2].dc_arc.next = -1;
	      g-> nodeBuffer[new_node].dc_identifier.instance = new_node2;

	      parms = next_node;
	    } else {
	      /* when we hit a void parameter, end of list */
	      parms = -1;
	    }
	  }
	}
      } else {
	/* parameters are already converted, copy the arc nodes */
	new_node = newNode(g);
	if (new_node == -1){
	  fprintf(stderr,"out of nodes\n");
	  exit(1);
	}
	g->nodeBuffer[new_node].common.type = dC_ARC_SON;
	g->nodeBuffer[new_node].dc_arc.ordinal = 0;
	g->nodeBuffer[new_node].dc_arc.src = i;
	g->nodeBuffer[new_node].dc_arc.dest = g->nodeBuffer[parms].dc_arc.dest;
	g->nodeBuffer[new_node].dc_arc.next = -1;
	g->nodeBuffer[i].dc_fct_type.formalParams = new_node;
	parms = g->nodeBuffer[parms].dc_arc.next;
	while(parms != -1){
	  prev_node = new_node;
	  new_node = newNode(g);
	  if (new_node == -1){
	    fprintf(stderr,"out of nodes\n");
	    exit(1);
	  }
	  g->nodeBuffer[new_node].common.type = dC_ARC_SON;
	  g->nodeBuffer[new_node].dc_arc.ordinal = 0;
	  g->nodeBuffer[new_node].dc_arc.src = i;
	  g->nodeBuffer[new_node].dc_arc.dest = g->nodeBuffer[parms].dc_arc.dest;
	  g->nodeBuffer[new_node].dc_arc.next = -1;
	  g->nodeBuffer[prev_node].dc_arc.next = new_node;
	  parms = g->nodeBuffer[parms].dc_arc.next;
	}
      }
}

void do_method_type(gd_graph *g, node_ptr i){

  /*
   * fields to save
   */

  /* from type decl */
  node_ptr decl;	/* points to decl if it exists   */
  str_ptr srcf;		/*  -- src file from decl        */
  int srcl;		/*  -- src line from decl        */
  node_ptr name;	/*  -- identifier node from decl */
  str_ptr  namestr;	/*     -- str_ptr for name       */

  /* from method type */
  node_ptr clas;	/* pointer to defining class */
  node_ptr parms;	/* pointer to parameters */
  node_ptr retn;	/* pointer to return type */
  			/* size field is ignored */

  /* other locals */
  node_ptr new_node, 	/* two variables for node */
  	new_node2,	/*  allocation */
	next_node,	/* used when traversing tree_list */
	prev_node,
	value_node;	/* used when traversing tree_list */

  str_ptr param_name;
  char param_str[4]; /* pNN */

  /* go get decl information if it exists */
  decl = g->nodeBuffer[i].type.name;
  if (decl != -1) {
    if (g->nodeBuffer[decl].common.type != gTYPE_DECL) {
      fprintf(stderr,"type name that is not a decl!!(%d)\n",i);
      exit(1);
    }
    srcf = g->nodeBuffer[decl].decl.srcf;
    srcl = g->nodeBuffer[decl].decl.srcl;
    name = g->nodeBuffer[decl].decl.name;
    if (name != -1){
      if (g->nodeBuffer[name].common.type != gIDENTIFIER_NODE) {
	fprintf(stderr,"decl name that is not a identifier!!(%d)\n",i);
	exit(1);
      }
      namestr = g->nodeBuffer[name].identifier_node.strg;
    } else {
      namestr = -1;
    }
  } else {
    /* decl doesn't exist */
    srcf = -1;
    srcl = -1;
    name = -1;
    namestr = -1;
  }

  parms = g->nodeBuffer[i].method_type.prms;
  retn  = g->nodeBuffer[i].method_type.retn;
  clas  = g->nodeBuffer[i].method_type.clas;

  /****************************
   * generate datrix nodes
   ****************************/

  /* switch node type */
  g->nodeBuffer[i].common.type = dC_FCT_TYPE;

  /* determin datrix visibility from common flags */
  /* gd_common doesn't change, no need to have saved flags */

  g->nodeBuffer[i].dc_identifier.visb = visb_unk;
  if (g->nodeBuffer[i].common.private_flag){
    g->nodeBuffer[i].dc_identifier.visb = visb_pri;
  }
  if (g->nodeBuffer[i].common.public_flag){
    g->nodeBuffer[i].dc_identifier.visb = visb_pub;
  }
  if (g->nodeBuffer[i].common.protected_flag){
    g->nodeBuffer[i].dc_identifier.visb = visb_pro;
  }

  /* other identifier and asg attributes */
  g->nodeBuffer[i].dc_asg_nd.line = srcl;
  g->nodeBuffer[i].dc_asg_nd.file = srcf;
  g->nodeBuffer[i].dc_identifier.name = namestr;

  /* unused fields */
  g->nodeBuffer[i].dc_identifier.arcFriend = -1;
  g->nodeBuffer[i].dc_type.implements = -1;

  /* for now TODO */
  g->nodeBuffer[i].dc_identifier.defLoc = clas;

  /* handle return type of function type */
  new_node = newNode(g);
  if (new_node == -1){
    fprintf(stderr,"out of nodes\n");
    exit(1);
  }
  g-> nodeBuffer[new_node].common.type = dC_INSTANCE;
  g-> nodeBuffer[new_node].dc_arc.ordinal = 0;
  g-> nodeBuffer[new_node].dc_arc.next = -1;
  g-> nodeBuffer[new_node].dc_arc.src = i;
  g-> nodeBuffer[new_node].dc_arc.dest = retn;
  g-> nodeBuffer[i].dc_identifier.instance = new_node;

  g-> nodeBuffer[i].dc_fct_type.formalParams = -1;

  /* parameters */
  /* since more than on funct_type may share a parameter list, check
   * to see if parameter list has been already converted */

  /* perhaps there's no parms list at all., ajm 2001 05 11 */
  if (0 <= parms && parms < g-> curNodePos)
      if (g->nodeBuffer[parms].common.type == gTREE_LIST) {
	/* gcc type parameters, need to be converted */
	if (parms != -1){
	  while(parms != -1){
	    next_node = g-> nodeBuffer[parms].tree_list.chan;
	    value_node = g -> nodeBuffer[parms].tree_list.value;
	    if (g->nodeBuffer[value_node].common.type != gVOID_TYPE &&
		!(g->nodeBuffer[value_node].common.type == dC_BUILT_IN_TYPE &&
		  g->nodeBuffer[value_node].dc_identifier.name == void_str)){
	      /* check for built-in-type and name == void ?? */
	      prev_node = parms; /* want to point to the node prior to the void */
	      new_node = newNode(g);
	      if (new_node == -1){
		fprintf(stderr,"out of nodes\n");
		exit(1);
	      }
	      g->nodeBuffer[parms].common.type = dC_ARC_SON;
	      g->nodeBuffer[parms].dc_arc.ordinal = 0;
	      g->nodeBuffer[parms].dc_arc.src = i;
	      g->nodeBuffer[parms].dc_arc.dest = new_node;
	      g->nodeBuffer[parms].dc_arc.next = -1;
	      if(g->nodeBuffer[i].dc_fct_type.formalParams == -1){
		g->nodeBuffer[i].dc_fct_type.formalParams = parms;
	      } else {
		g->nodeBuffer[prev_node].dc_arc.next = parms;
	      }
	      prev_node = parms;
	      g-> nodeBuffer[new_node].common = g-> nodeBuffer[i].common;
	      g-> nodeBuffer[new_node].common.type = dC_FORMAL_FCT_PARAM;
	      g-> nodeBuffer[new_node].common.other_node = -1;
	      g-> nodeBuffer[new_node].dc_identifier.visb =
		    g-> nodeBuffer[i].dc_identifier.visb;
	      g-> nodeBuffer[new_node].dc_asg_nd.line =
		    g-> nodeBuffer[i].dc_asg_nd.line;
	      g-> nodeBuffer[new_node].dc_asg_nd.file =
		    g-> nodeBuffer[i].dc_asg_nd.file;
		g-> nodeBuffer[new_node].dc_identifier.name = -1; /* should generate name? */
	      g-> nodeBuffer[new_node].dc_identifier.defLoc = -1;
	      g-> nodeBuffer[new_node].dc_identifier.arcFriend = -1;
	      g-> nodeBuffer[new_node].dc_object.static_flag = 0;
	      g-> nodeBuffer[new_node].dc_object.extern_flag = 0;
	      g-> nodeBuffer[new_node].dc_object.const_flag = 0; /* wrong */
	      g-> nodeBuffer[new_node].dc_identifier.array_dim = -1;/* wrong */
	      g-> nodeBuffer[new_node].dc_object.bitfield_dim = -1;
	      g-> nodeBuffer[new_node].dc_object.initial_val = -1;

	      new_node2 = newNode(g);
	      if (new_node2 == -1){
		fprintf(stderr,"out of nodes\n");
		exit(1);
	      }
	      g-> nodeBuffer[new_node2].common.other_node = -1;
	      g-> nodeBuffer[new_node2].common.type = dC_INSTANCE;
	      g-> nodeBuffer[new_node2].dc_arc.ordinal = 0;
	      g-> nodeBuffer[new_node2].dc_arc.src = new_node;
	      g-> nodeBuffer[new_node2].dc_arc.dest = value_node;
	      g-> nodeBuffer[new_node2].dc_arc.next = -1;
	      g-> nodeBuffer[new_node].dc_identifier.instance = new_node2;

	      parms = next_node;
	    } else {
	      /* when we hit a void parameter, end of list */
	      parms = -1;
	    }
	  }
	}
      } else {
	/* parameters are already converted, copy the arc nodes */
	new_node = newNode(g);
	if (new_node == -1){
	  fprintf(stderr,"out of nodes\n");
	  exit(1);
	}
	g->nodeBuffer[new_node].common.type = dC_ARC_SON;
	g->nodeBuffer[new_node].dc_arc.ordinal = 0;
	g->nodeBuffer[new_node].dc_arc.src = i;
	g->nodeBuffer[new_node].dc_arc.dest = g->nodeBuffer[parms].dc_arc.dest;
	g->nodeBuffer[new_node].dc_arc.next = -1;
	g->nodeBuffer[i].dc_fct_type.formalParams = i;
	parms = g->nodeBuffer[parms].dc_arc.next;
	while(parms != -1){
	  prev_node = new_node;
	  new_node = newNode(g);
	  if (new_node == -1){
	    fprintf(stderr,"out of nodes\n");
	    exit(1);
	  }
	  g->nodeBuffer[new_node].common.type = dC_ARC_SON;
	  g->nodeBuffer[new_node].dc_arc.ordinal = 0;
	  g->nodeBuffer[new_node].dc_arc.src = i;
	  g->nodeBuffer[new_node].dc_arc.dest = g->nodeBuffer[parms].dc_arc.dest;
	  g->nodeBuffer[new_node].dc_arc.next = -1;
	  g->nodeBuffer[i].dc_fct_type.formalParams = i;
	  g->nodeBuffer[prev_node].dc_arc.next = new_node;
	  parms = g->nodeBuffer[parms].dc_arc.next;
	}
      }
}
