#include <stdio.h>
#include <stdlib.h>

#include "gd_graph.h"
#include "gd_utils.h"

void attr_decl(gd_graph g, t_gcc_decl decl);
void attr_common(gd_graph_common comm);
void attr_type(t_gcc_type type);
void attr_cst(t_gcc_const cst);
void attr_expr(t_gcc_expr expr);

void edge_decl(gd_graph g, t_gcc_decl decl,int pos);
void edge_common(gd_graph_common comm,int pos);
void edge_type(t_gcc_type type,int pos);
void edge_cst(t_gcc_const cst,int pos);
void edge_expr(t_gcc_expr expr,int pos);

void write_edge(int from, int to, const char * type);
void write_boolean_attr(const char * attr, int val);
void write_number_attr(const char * attr, int val);
void write_string_attr(const char * attr, char * val);
void write_double_attr(const char * attr, char * val);

void write_gxl_type(const char * typename);

void datrix_arc_syntax(t_dc_arc_syntax  dc_arc_syntax);
void datrix_arc(t_dc_arc  dc_arc_syntax);

void datrix_scope(gd_graph g, t_dc_scope scope);
void datrix_identifier(gd_graph g, t_dc_identifier ident);
void datrix_asg_nd(gd_graph g, t_dc_asg_nd nd);
void datrix_type(gd_graph g, t_dc_type type);
void datrix_object(gd_graph g, t_dc_object object);
void datrix_label(gd_graph g, t_dc_label label);
void datrix_expression(gd_graph g, t_dc_expression expr);
void datrix_operator(gd_graph g, t_dc_operator operator);
void datrix_ctrl_stmt(gd_graph g, t_dc_ctrl_stmt cntrl);
void datrix_jump(gd_graph g, t_dc_jump jump);
void datrix_loop(gd_graph g, t_dc_loop loop);
void datrix_comment(gd_graph g, t_dc_comment comment);
void datrix_arc_reference(t_dc_arc_reference  dc_arc_reference);


char * progname;

void usage(){
  fprintf(stderr,"usage: %s dumpfile\n",progname);
  exit(1);
}

int main(int argc, char * argv[]){
  gd_graph g;
  char * filename;
  int nodenum;
  progname = argv[0];

  if (argc != 2) {
    usage();
  }

  filename = argv[1];

  g = gd_graph_read_binary_file(filename,1.0,1.0,1.0);
  if (g.curStringPos == 0) {
    fprintf(stderr,"could not get graph\n");
    exit(1);
  }

  printf("<?xml version=\"1.0\"?>\n");
  printf("<!DOCTYPE gxl SYSTEM \"gxl-1-0.dtd\">\n");
  printf("<gxl>\n");
  /* note, converte nested quotes in file name? */
  printf(" <graph id=\"%s\">\n", filename);

  for (nodenum = 0; nodenum < g.curNodePos; nodenum++) {
    if (g.nodeBuffer[nodenum].common.type != NODE_DELTED &&
	g.nodeBuffer[nodenum].common.type != NODE_NONE) {
      if (g.nodeBuffer[nodenum].common.type < dC_ARC||
	 g.nodeBuffer[nodenum].common.type > dC_ARC_ARRAY_DIM){
	printf("  <node id=\"n%d\">\n",nodenum);
      } else  {
	printf("  <edge from=\"n%d\" to=\"n%d\">\n",
	    g.nodeBuffer[nodenum].dc_arc.src,
	    g.nodeBuffer[nodenum].dc_arc.dest);
      }
      write_gxl_type(gd_node_type_ascii[g.nodeBuffer[nodenum].common.type]);
    }
    switch (g.nodeBuffer[nodenum].common.type) {
      case gNAMESPACE_DECL:
	attr_decl(g,g.nodeBuffer[nodenum].decl);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_decl(g,g.nodeBuffer[nodenum].decl,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].namespace_decl.dcls,"namespace_decl_dcls");
	write_edge(nodenum,g.nodeBuffer[nodenum].namespace_decl.alis,"namespace_decl_alis");
	break;
      case gFUNCTION_DECL:
	attr_decl(g,g.nodeBuffer[nodenum].decl);
	write_boolean_attr("undef",g.nodeBuffer[nodenum].function_decl.undef);
	write_number_attr("ext_static", g.nodeBuffer[nodenum].function_decl.ext_static);
	write_boolean_attr("member", g.nodeBuffer[nodenum].function_decl.member);
	write_boolean_attr("constr", g.nodeBuffer[nodenum].function_decl.constr);
	write_boolean_attr("decon", g.nodeBuffer[nodenum].function_decl.decon);
	write_boolean_attr("oper", g.nodeBuffer[nodenum].function_decl.oper);
	write_boolean_attr("conver",g.nodeBuffer[nodenum].function_decl.conver);
	write_boolean_attr("ginit",g.nodeBuffer[nodenum].function_decl.ginit);
	write_boolean_attr("gfini",g.nodeBuffer[nodenum].function_decl.gfini);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_decl(g,g.nodeBuffer[nodenum].decl,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].function_decl.args,"function_decl_args");
	write_edge(nodenum,g.nodeBuffer[nodenum].function_decl.body,"function_decl_body");
	break;
      case gVAR_DECL:
	attr_decl(g,g.nodeBuffer[nodenum].decl);
	write_number_attr("algn",g.nodeBuffer[nodenum].var_decl.algn);
	write_number_attr("used",g.nodeBuffer[nodenum].var_decl.used);
	write_number_attr("reg",g.nodeBuffer[nodenum].var_decl.reg);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_decl(g,g.nodeBuffer[nodenum].decl,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].var_decl.init,"var_decl_init");
	write_edge(nodenum,g.nodeBuffer[nodenum].var_decl.size,"var_decl_size");
	break;
      case gPARM_DECL:
	attr_decl(g,g.nodeBuffer[nodenum].decl);
	write_number_attr("algn",g.nodeBuffer[nodenum].parm_decl.algn);
	write_number_attr("used",g.nodeBuffer[nodenum].parm_decl.used);
	write_number_attr("reg",g.nodeBuffer[nodenum].parm_decl.reg);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_decl(g,g.nodeBuffer[nodenum].decl,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].parm_decl.argt,"parm_decl_argt");
	write_edge(nodenum,g.nodeBuffer[nodenum].parm_decl.size,"parm_decl_size");
	break;
      case gTYPE_DECL:
	attr_decl(g,g.nodeBuffer[nodenum].decl);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_decl(g,g.nodeBuffer[nodenum].decl,nodenum);
	break;
      case gLABEL_DECL:
	attr_decl(g,g.nodeBuffer[nodenum].decl);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_decl(g,g.nodeBuffer[nodenum].decl,nodenum);
	break;
      case gCONST_DECL:
	attr_decl(g,g.nodeBuffer[nodenum].decl);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_decl(g,g.nodeBuffer[nodenum].decl,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].const_decl.cnst,"const_decl_cnst");
	break;
      case gRESULT_DECL:
	attr_decl(g,g.nodeBuffer[nodenum].decl);
	write_number_attr("algn",g.nodeBuffer[nodenum].result_decl.algn);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_decl(g,g.nodeBuffer[nodenum].decl,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].result_decl.init,"result_decl_init");
	write_edge(nodenum,g.nodeBuffer[nodenum].result_decl.size,"result_decl_size");
	break;
      case gFIELD_DECL:
	attr_decl(g,g.nodeBuffer[nodenum].decl);
	write_number_attr("algn",g.nodeBuffer[nodenum].field_decl.algn);
	write_number_attr("bitfield",g.nodeBuffer[nodenum].field_decl.bitfield);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_decl(g,g.nodeBuffer[nodenum].decl,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].field_decl.init,"field_decl_init");
	write_edge(nodenum,g.nodeBuffer[nodenum].field_decl.size,"field_decl_size");
	write_edge(nodenum,g.nodeBuffer[nodenum].field_decl.bpos,"field_decl_bpos");
	break;
      case gTEMPLATE_DECL:
	attr_decl(g,g.nodeBuffer[nodenum].decl);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_decl(g,g.nodeBuffer[nodenum].decl,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].template_decl.rslt,"template_decl_rslt");
	write_edge(nodenum,g.nodeBuffer[nodenum].template_decl.inst,"template_decl_inst");
	write_edge(nodenum,g.nodeBuffer[nodenum].template_decl.spcs,"template_decl_spcs");
	break;
      case gUSING_DECL:
	attr_decl(g,g.nodeBuffer[nodenum].decl);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_decl(g,g.nodeBuffer[nodenum].decl,nodenum);
	break;
      case gFUNCTION_TYPE:
	attr_type(g.nodeBuffer[nodenum].type);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].function_type.retn,"function_type_retn");
	write_edge(nodenum,g.nodeBuffer[nodenum].function_type.prms,"function_type_prms");
	break;
      case gINTEGER_TYPE:
	attr_type(g.nodeBuffer[nodenum].type);
	write_boolean_attr("unsigned",g.nodeBuffer[nodenum].integer_type.unsgnd);
	write_number_attr("prec", g.nodeBuffer[nodenum].integer_type.prec);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].integer_type.min,"integer_type_min");
	write_edge(nodenum,g.nodeBuffer[nodenum].integer_type.max,"integer_type_max");
	break;
      case gARRAY_TYPE:
	attr_type(g.nodeBuffer[nodenum].type);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].array_type.elts,"array_type_elts");
	write_edge(nodenum,g.nodeBuffer[nodenum].array_type.domn,"array_type_domn");
	break;
      case gVOID_TYPE:
	attr_type(g.nodeBuffer[nodenum].type);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	break;
      case gREAL_TYPE:
	attr_type(g.nodeBuffer[nodenum].type);
	write_number_attr("prec", g.nodeBuffer[nodenum].real_type.prec);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	break;
      case gCOMPLEX_TYPE:
	attr_type(g.nodeBuffer[nodenum].type);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	break;
      case gVECTOR_TYPE:
	attr_type(g.nodeBuffer[nodenum].type);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	break;
      case gENUMERAL_TYPE:
	attr_type(g.nodeBuffer[nodenum].type);
	write_boolean_attr("unsigned",g.nodeBuffer[nodenum].enumeral_type.unsgnd);
	write_number_attr("prec", g.nodeBuffer[nodenum].enumeral_type.prec);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].enumeral_type.min,"enumeral_type_min");
	write_edge(nodenum,g.nodeBuffer[nodenum].enumeral_type.max,"enumeral_type_max");
	write_edge(nodenum,g.nodeBuffer[nodenum].enumeral_type.csts,"enumeral_type_csts");
	break;
      case gBOOLEAN_TYPE:
	attr_type(g.nodeBuffer[nodenum].type);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	break;
      case gCHAR_TYPE:
	attr_type(g.nodeBuffer[nodenum].type);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	break;
      case gPOINTER_TYPE:
	attr_type(g.nodeBuffer[nodenum].type);
	write_boolean_attr("ptrmem",g.nodeBuffer[nodenum].pointer_type.ptrmem);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].pointer_type.ptd,"pointer_type_ptd");
	write_edge(nodenum,g.nodeBuffer[nodenum].pointer_type.cls,"pointer_type_cls");
	break;
      case gOFFSET_TYPE:
	attr_type(g.nodeBuffer[nodenum].type);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	break;
      case gREFERENCE_TYPE:
	attr_type(g.nodeBuffer[nodenum].type);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].reference_type.refd,"reference_type_refd");
	break;
      case gMETHOD_TYPE:
	attr_type(g.nodeBuffer[nodenum].type);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].method_type.clas,"method_type_clas");
	write_edge(nodenum,g.nodeBuffer[nodenum].method_type.retn,"method_type_retn");
	write_edge(nodenum,g.nodeBuffer[nodenum].method_type.prms,"method_type_prms");
	break;
      case gFILE_TYPE:
	attr_type(g.nodeBuffer[nodenum].type);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	break;
      case gSET_TYPE:
	attr_type(g.nodeBuffer[nodenum].type);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	break;
      case gRECORD_TYPE:
	attr_type(g.nodeBuffer[nodenum].type);
	write_boolean_attr("ptrmem",g.nodeBuffer[nodenum].record_type.ptrmem);
	write_boolean_attr("isClass",g.nodeBuffer[nodenum].record_type.isClass);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].record_type.flds,"record_type_flds");
	write_edge(nodenum,g.nodeBuffer[nodenum].record_type.fncs,"record_type_fncs");
	write_edge(nodenum,g.nodeBuffer[nodenum].record_type.ptd,"record_type_ptd");
	write_edge(nodenum,g.nodeBuffer[nodenum].record_type.cls,"record_type_cls");
	write_edge(nodenum,g.nodeBuffer[nodenum].record_type.vfld,"record_type_vfld");
	write_edge(nodenum,g.nodeBuffer[nodenum].record_type.binf,"record_type_binf");
	break;
      case gUNION_TYPE:
	attr_type(g.nodeBuffer[nodenum].type);
	write_boolean_attr("ptrmem",g.nodeBuffer[nodenum].record_type.ptrmem);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].record_type.flds,"record_type_flds");
	write_edge(nodenum,g.nodeBuffer[nodenum].record_type.fncs,"record_type_fncs");
	write_edge(nodenum,g.nodeBuffer[nodenum].record_type.ptd,"record_type_ptd");
	write_edge(nodenum,g.nodeBuffer[nodenum].record_type.cls,"record_type_cls");
	write_edge(nodenum,g.nodeBuffer[nodenum].record_type.vfld,"record_type_vfld");
	write_edge(nodenum,g.nodeBuffer[nodenum].record_type.binf,"record_type_binf");
	break;
      case gQUAL_UNION_TYPE:
	attr_type(g.nodeBuffer[nodenum].type);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	break;
      case gLANG_TYPE:
	attr_type(g.nodeBuffer[nodenum].type);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	break;
      case gTEMPLATE_TYPE_PARM:
	attr_type(g.nodeBuffer[nodenum].type);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	break;
      case gTEMPLATE_PARM_INDEX:
	attr_common(g.nodeBuffer[nodenum].common);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	break;
      case gTEMPLATE_TEMPLATE_PARM:
	attr_type(g.nodeBuffer[nodenum].type);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	break;
      case gBOUND_TEMPLATE_TEMPLATE_PARM:
	attr_type(g.nodeBuffer[nodenum].type);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	break;
      case gTYPENAME_TYPE:
	attr_type(g.nodeBuffer[nodenum].type);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	break;
      case gTYPEOF_TYPE:
	attr_type(g.nodeBuffer[nodenum].type);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	break;
      case gBLOCK:
	attr_common(g.nodeBuffer[nodenum].common);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	break;
      case gCOMPOUND_STMT:
	attr_expr(g.nodeBuffer[nodenum].expr);
	write_number_attr("line", g.nodeBuffer[nodenum].compound_stmt.line);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].compound_stmt.body,"compound_stmt_body");
	write_edge(nodenum,g.nodeBuffer[nodenum].compound_stmt.next,"compound_stmt_next");
	break;
      case gEXPR_STMT:
	attr_expr(g.nodeBuffer[nodenum].expr);
	write_number_attr("line", g.nodeBuffer[nodenum].expr_stmt.line);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].expr_stmt.texpr,"expr_stmt_expr");
	write_edge(nodenum,g.nodeBuffer[nodenum].expr_stmt.next,"expr_stmt_next");
	break;
      case gDECL_STMT:
	attr_expr(g.nodeBuffer[nodenum].expr);
	write_number_attr("line", g.nodeBuffer[nodenum].decl_stmt.line);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].decl_stmt.decl,"decl_stmt_decl");
	write_edge(nodenum,g.nodeBuffer[nodenum].decl_stmt.next,"decl_stmt_next");
	break;
      case gSCOPE_STMT:
	attr_expr(g.nodeBuffer[nodenum].expr);
	write_number_attr("line", g.nodeBuffer[nodenum].scope_stmt.line);
	write_number_attr("null", g.nodeBuffer[nodenum].scope_stmt.null);
	write_number_attr("begin_end", g.nodeBuffer[nodenum].scope_stmt.begin_end);
	write_number_attr("clnp", g.nodeBuffer[nodenum].scope_stmt.clnp);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].scope_stmt.next,"scope_stmt_next");
	break;
      case gRETURN_STMT:
	attr_expr(g.nodeBuffer[nodenum].expr);
	write_number_attr("line", g.nodeBuffer[nodenum].return_stmt.line);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].return_stmt.ret_expr,"return_stmt_expr");
	write_edge(nodenum,g.nodeBuffer[nodenum].return_stmt.next,"return_stmt_next");
	break;
      case gSTMT_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].stmt_expr.stmt,"stmt_expr_stmt");
	break;
      case gBREAK_STMT:
	attr_expr(g.nodeBuffer[nodenum].expr);
	write_number_attr("line", g.nodeBuffer[nodenum].break_stmt.line);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].break_stmt.next,"break_stmt_next");
	break;
      case gCONTINUE_STMT:
	attr_expr(g.nodeBuffer[nodenum].expr);
	write_number_attr("line", g.nodeBuffer[nodenum].continue_stmt.line);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].continue_stmt.next,"continue_stmt_next");
	break;
      case gIF_STMT:
	attr_expr(g.nodeBuffer[nodenum].expr);
	write_number_attr("line", g.nodeBuffer[nodenum].if_stmt.line);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].if_stmt.cond,"if_stmt_cond");
	write_edge(nodenum,g.nodeBuffer[nodenum].if_stmt.thenc,"if_stmt_thenc");
	write_edge(nodenum,g.nodeBuffer[nodenum].if_stmt.elsec,"if_stmt_elsec");
	write_edge(nodenum,g.nodeBuffer[nodenum].if_stmt.next,"if_stmt_next");
	break;
      case gSWITCH_STMT:
	attr_expr(g.nodeBuffer[nodenum].expr);
	write_number_attr("line", g.nodeBuffer[nodenum].switch_stmt.line);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].switch_stmt.cond,"switch_stmt_cond");
	write_edge(nodenum,g.nodeBuffer[nodenum].switch_stmt.body,"switch_stmt_body");
	write_edge(nodenum,g.nodeBuffer[nodenum].switch_stmt.next,"switch_stmt_next");
	break;
      case gLABEL_STMT:
	attr_expr(g.nodeBuffer[nodenum].expr);
	write_number_attr("line", g.nodeBuffer[nodenum].label_stmt.line);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].label_stmt.label,"label_stmt_label");
	write_edge(nodenum,g.nodeBuffer[nodenum].label_stmt.next,"label_stmt_next");
	break;
      case gTRY_BLOCK:
	attr_expr(g.nodeBuffer[nodenum].expr);
	write_number_attr("line", g.nodeBuffer[nodenum].try_block.line);
	write_boolean_attr("cleanup", g.nodeBuffer[nodenum].try_block.cleanup);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].try_block.body,"try_block_body");
	write_edge(nodenum,g.nodeBuffer[nodenum].try_block.hdlr,"try_block_hdlr");
	write_edge(nodenum,g.nodeBuffer[nodenum].try_block.next,"try_block_next");
	break;
      case gHANDLER:
	attr_expr(g.nodeBuffer[nodenum].expr);
	write_number_attr("line", g.nodeBuffer[nodenum].handler.line);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].handler.body,"handler_body");
	write_edge(nodenum,g.nodeBuffer[nodenum].handler.next,"handler_next");
	break;
      case gSTART_CATCH_STMT:
	attr_expr(g.nodeBuffer[nodenum].expr);
	write_number_attr("line", g.nodeBuffer[nodenum].start_catch_stmt.line);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].start_catch_stmt.catch_type,"start_catch_stmt_catch_type");
	write_edge(nodenum,g.nodeBuffer[nodenum].start_catch_stmt.next,"start_catch_stmt_next");
	break;
      case gCLEANUP_STMT:
	attr_expr(g.nodeBuffer[nodenum].expr);
	write_number_attr("line", g.nodeBuffer[nodenum].cleanup_stmt.line);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].cleanup_stmt.decl,"cleanup_stmt_decl");
	write_edge(nodenum,g.nodeBuffer[nodenum].cleanup_stmt.clean_expr,"cleanup_stmt_expr");
	write_edge(nodenum,g.nodeBuffer[nodenum].cleanup_stmt.next,"cleanup_stmt_next");
	break;
      case gCASE_LABEL:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].case_label.low,"case_label_low");
	write_edge(nodenum,g.nodeBuffer[nodenum].case_label.high,"case_label_high");
	write_edge(nodenum,g.nodeBuffer[nodenum].case_label.next,"case_label_next");
	break;
      case gGOTO_STMT:
	attr_expr(g.nodeBuffer[nodenum].expr);
	write_number_attr("line", g.nodeBuffer[nodenum].goto_stmt.line);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].goto_stmt.dest,"goto_stmt_dest");
	write_edge(nodenum,g.nodeBuffer[nodenum].goto_stmt.next,"goto_stmt_next");
	break;
      case gFOR_STMT:
	attr_expr(g.nodeBuffer[nodenum].expr);
	write_number_attr("line", g.nodeBuffer[nodenum].for_stmt.line);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].for_stmt.init,"for_stmt_init");
	write_edge(nodenum,g.nodeBuffer[nodenum].for_stmt.cond,"for_stmt_cond");
	write_edge(nodenum,g.nodeBuffer[nodenum].for_stmt.inc_expr,"for_stmt_expr");
	write_edge(nodenum,g.nodeBuffer[nodenum].for_stmt.body,"for_stmt_body");
	write_edge(nodenum,g.nodeBuffer[nodenum].for_stmt.next,"for_stmt_next");
	break;
      case gWHILE_STMT:
	attr_expr(g.nodeBuffer[nodenum].expr);
	write_number_attr("line", g.nodeBuffer[nodenum].while_stmt.line);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].while_stmt.cond,"while_stmt_cond");
	write_edge(nodenum,g.nodeBuffer[nodenum].while_stmt.body,"while_stmt_body");
	write_edge(nodenum,g.nodeBuffer[nodenum].while_stmt.next,"while_stmt_next");
	break;
      case gDO_STMT:
	attr_expr(g.nodeBuffer[nodenum].expr);
	write_number_attr("line", g.nodeBuffer[nodenum].do_stmt.line);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].do_stmt.cond,"do_stmt_cond");
	write_edge(nodenum,g.nodeBuffer[nodenum].do_stmt.body,"do_stmt_body");
	write_edge(nodenum,g.nodeBuffer[nodenum].do_stmt.next,"do_stmt_next");
	break;
      case gASM_STMT:
	attr_expr(g.nodeBuffer[nodenum].expr);
	write_number_attr("line", g.nodeBuffer[nodenum].asm_stmt.line);
	write_number_attr("vol", g.nodeBuffer[nodenum].asm_stmt.vol);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].asm_stmt.strg,"asm_stmt_strg");
	write_edge(nodenum,g.nodeBuffer[nodenum].asm_stmt.outs,"asm_stmt_outs");
	write_edge(nodenum,g.nodeBuffer[nodenum].asm_stmt.ins,"asm_stmt_ins");
	write_edge(nodenum,g.nodeBuffer[nodenum].asm_stmt.clbr,"asm_stmt_clbr");
	write_edge(nodenum,g.nodeBuffer[nodenum].asm_stmt.next,"asm_stmt_next");
	break;
      case gCOMPONENT_REF:
	attr_common(g.nodeBuffer[nodenum].common);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].component_ref.op0,"component_ref_op0");
	break;
      case gBIT_FIELD_REF:
	attr_common(g.nodeBuffer[nodenum].common);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	break;
      case gINDIRECT_REF:
	attr_common(g.nodeBuffer[nodenum].common);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].indirect_ref.op0,"indirect_ref_op0");
	break;
      case gBUFFER_REF:
	attr_common(g.nodeBuffer[nodenum].common);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	break;
      case gARRAY_REF:
	attr_common(g.nodeBuffer[nodenum].common);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].array_ref.op0,"array_ref_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].array_ref.op1,"array_ref_op1");
	break;
      case gOFFSET_REF:
	attr_common(g.nodeBuffer[nodenum].common);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].offset_ref.op0,"offset_ref_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].offset_ref.op1,"offset_ref_op1");
	break;
      case gSCOPE_REF:
	attr_common(g.nodeBuffer[nodenum].common);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].scope_ref.op0,"scope_ref_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].scope_ref.op1,"scope_ref_op1");
	break;
      case gMEMBER_REF:
	attr_common(g.nodeBuffer[nodenum].common);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].member_ref.op0,"member_ref_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].member_ref.op1,"member_ref_op1");
	break;
      case gCONSTRUCTOR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].constructor.elts,"constructor_elts");
	break;
      case gCOMPOUND_EXPR:
	attr_common(g.nodeBuffer[nodenum].common);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].compound_expr.op0,"compound_ref_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].compound_expr.op1,"compound_ref_op1");
	break;
      case gMODIFY_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].modify_expr.op0,"modify_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].modify_expr.op1,"modify_expr_op1");
	break;
      case gINIT_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].init_expr.op0,"init_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].init_expr.op1,"init_expr_op1");
	break;
      case gTHROW_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].throw_expr.op0,"throw_expr_op0");
	break;
      case gTARGET_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].target_expr.decl,"target_expr_decl");
	write_edge(nodenum,g.nodeBuffer[nodenum].target_expr.init,"target_expr_init");
	write_edge(nodenum,g.nodeBuffer[nodenum].target_expr.clnp,"target_expr_clnp");
	break;
      case gCOND_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].cond_expr.op0,"cond_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].cond_expr.op1,"cond_expr_op1");
	write_edge(nodenum,g.nodeBuffer[nodenum].cond_expr.op2,"cond_expr_op2");
	break;
      case gBIND_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].bind_expr.vars,"bind_expr_vars");
	write_edge(nodenum,g.nodeBuffer[nodenum].bind_expr.body,"bind_expr_body");
	break;
      case gCALL_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].call_expr.fn,"call_expr_fn");
	write_edge(nodenum,g.nodeBuffer[nodenum].call_expr.args,"call_expr_args");
	break;
      case gMETHOD_CALL_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	break;
      case gWITH_CLEANUP_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	break;
      case gCLEANUP_POINT_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].cleanup_point_expr.op0,"cleanup_point_expr_op0");
	break;
      case gPLACEHOLDER_EXPR:
	attr_common(g.nodeBuffer[nodenum].common);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	break;
      case gWITH_RECORD_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	break;
      case gPLUS_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].plus_expr.expr2.op0,"plus_expr_expr0");
	write_edge(nodenum,g.nodeBuffer[nodenum].plus_expr.expr2.op1,"plus_expr_expr1");
	break;
      case gMINUS_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].minus_expr.expr2.op0,"minus_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].minus_expr.expr2.op1,"minus_expr_op1");
	break;
      case gMULT_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].mult_expr.expr2.op0,"mult_expr_expr0");
	write_edge(nodenum,g.nodeBuffer[nodenum].mult_expr.expr2.op1,"mult_expr_expr1");
	break;
      case gTRUNC_DIV_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].trunc_div_expr.expr2.op0,"trunc_div_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].trunc_div_expr.expr2.op1,"trunc_div_expr_op1");
	break;
      case gCEIL_DIV_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].ceil_div_expr.expr2.op0,"ceil_div_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].ceil_div_expr.expr2.op1,"ceil_div_expr_op1");
	break;
      case gFLOOR_DIV_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].floor_div_expr.expr2.op0,"floor_div_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].floor_div_expr.expr2.op1,"floor_div_expr_op1");
	break;
      case gROUND_DIV_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].round_div_expr.expr2.op0,"round_div_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].round_div_expr.expr2.op1,"round_div_expr_op1");
	break;
      case gTRUNC_MOD_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].trunc_mod_expr.expr2.op0,"trunc_mod_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].trunc_mod_expr.expr2.op1,"trunc_mod_expr_op1");
	break;
      case gCEIL_MOD_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].ceil_mod_expr.expr2.op0,"ceil_mod_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].ceil_mod_expr.expr2.op1,"ceil_mod_expr_op1");
	break;
      case gFLOOR_MOD_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].floor_mod_expr.expr2.op0,"floor_mod_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].floor_mod_expr.expr2.op1,"floor_mod_expr_op1");
	break;
      case gROUND_MOD_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].round_mod_expr.expr2.op0,"round_mod_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].round_mod_expr.expr2.op1,"round_mod_expr_op1");
	break;
      case gRDIV_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].rdiv_expr.expr2.op0,"rdiv_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].rdiv_expr.expr2.op1,"rdiv_expr_op1");
	break;
      case gEXACT_DIV_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].exact_div_expr.expr2.op0,"exact_div_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].exact_div_expr.expr2.op1,"exact_div_expr_op1");
	break;
      case gFIX_TRUNC_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].fix_trunc_expr.expr1.op0,"fix_trunc_expr_op0");
	break;
      case gFIX_CEIL_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].fix_ceil_expr.expr1.op0,"fix_ceil_expr_op0");
	break;
      case gFIX_FLOOR_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].fix_floor_expr.expr1.op0,"fix_floor_expr_op0");
	break;
      case gFIX_ROUND_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].fix_round_expr.expr1.op0,"fix_round_expr_op0");
	break;
      case gFLOAT_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].float_expr.expr1.op0,"float_expr_op0");
	break;
      case gEXPON_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].expon_expr.expr2.op0,"expon_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].expon_expr.expr2.op1,"expon_expr_op1");
	break;
      case gNEGATE_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].negate_expr.expr1.op0,"negate_expr_op0");
	break;
      case gMIN_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].min_expr.expr2.op0,"min_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].min_expr.expr2.op1,"min_expr_op1");
	break;
      case gMAX_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].max_expr.expr2.op0,"max_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].max_expr.expr2.op1,"max_expr_op1");
	break;
      case gABS_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].abs_expr.expr1.op0,"abs_expr_op0");
	break;
      case gFFS_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].ffs_expr.expr1.op0,"ffs_expr_op0");
	break;
      case gLSHIFT_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].lshift_expr.expr2.op0,"lshift_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].lshift_expr.expr2.op1,"lshift_expr_op1");
	break;
      case gRSHIFT_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].rshift_expr.expr2.op0,"rshift_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].rshift_expr.expr2.op1,"rshift_expr_op1");
	break;
      case gLROTATE_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].lrotate_expr.expr2.op0,"lrotate_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].lrotate_expr.expr2.op1,"lrotate_expr_op1");
	break;
      case gRROTATE_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].rrotate_expr.expr2.op0,"rrotate_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].rrotate_expr.expr2.op1,"rrotate_expr_op1");
	break;
      case gBIT_IOR_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].bit_ior_expr.expr2.op0,"bit_ior_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].bit_ior_expr.expr2.op1,"bit_ior_expr_op1");
	break;
      case gBIT_XOR_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].bit_xor_expr.expr2.op0,"bit_xor_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].bit_xor_expr.expr2.op1,"bit_xor_expr_op1");
	break;
      case gBIT_AND_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].bit_and_expr.expr2.op0,"bit_and_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].bit_and_expr.expr2.op1,"bit_and_expr_op1");
	break;
      case gBIT_ANDTC_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].bit_andtc_expr.expr2.op0,"bit_andtc_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].bit_andtc_expr.expr2.op1,"bit_andtc_expr_op1");
	break;
      case gBIT_NOT_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].bit_not_expr.expr1.op0,"bit_not_expr_op0");
	break;
      case gTRUTH_ANDIF_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].truth_andif_expr.op0,"truth_andif_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].truth_andif_expr.op1,"truth_andif_expr_op1");
	break;
      case gTRUTH_ORIF_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].truth_orif_expr.op0,"truth_orif_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].truth_orif_expr.op1,"truth_orif_expr_op1");
	break;
      case gTRUTH_AND_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].truth_and_expr.op0,"truth_and_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].truth_and_expr.op1,"truth_and_expr_op1");
	break;
      case gTRUTH_OR_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].truth_or_expr.op0,"truth_or_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].truth_or_expr.op1,"truth_or_expr_op1");
	break;
      case gTRUTH_XOR_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].truth_xor_expr.op0,"truth_xor_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].truth_xor_expr.op1,"truth_xor_expr_op1");
	break;
      case gTRUTH_NOT_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].truth_not_expr.op0,"truth_not_expr_op0");
	break;
      case gLT_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].lt_expr.expr_ineq.op0,"lt_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].lt_expr.expr_ineq.op1,"lt_expr_op1");
	break;
      case gLE_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].le_expr.expr_ineq.op0,"le_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].le_expr.expr_ineq.op1,"le_expr_op1");
	break;
      case gGT_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].gt_expr.expr_ineq.op0,"gt_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].gt_expr.expr_ineq.op1,"gt_expr_op1");
	break;
      case gGE_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].ge_expr.expr_ineq.op0,"ge_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].ge_expr.expr_ineq.op1,"ge_expr_op1");
	break;
      case gEQ_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].eq_expr.expr_ineq.op0,"eq_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].eq_expr.expr_ineq.op1,"eq_expr_op1");
	break;
      case gNE_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].ne_expr.expr_ineq.op0,"ne_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].ne_expr.expr_ineq.op1,"ne_expr_op1");
	break;
      case gUNORDERED_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].unordered_expr.expr_ineq.op0,"unordered_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].unordered_expr.expr_ineq.op1,"unordered_expr_op1");
	break;
      case gORDERED_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].ordered_expr.expr_ineq.op0,"ordered_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].ordered_expr.expr_ineq.op1,"ordered_expr_op1");
	break;
      case gUNLT_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].unlt_expr.expr_ineq.op0,"unlt_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].unlt_expr.expr_ineq.op1,"unlt_expr_op1");
	break;
      case gUNLE_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].unle_expr.expr_ineq.op0,"unle_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].unle_expr.expr_ineq.op1,"unle_expr_op1");
	break;
      case gUNGT_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].ungt_expr.expr_ineq.op0,"ungt_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].ungt_expr.expr_ineq.op1,"ungt_expr_op1");
	break;
      case gUNGE_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].unge_expr.expr_ineq.op0,"unge_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].unge_expr.expr_ineq.op1,"unge_expr_op1");
	break;
      case gUNEQ_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].uneq_expr.expr_ineq.op0,"uneq_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].uneq_expr.expr_ineq.op1,"uneq_expr_op1");
	break;
      case gIN_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].in_expr.expr2.op0,"in_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].in_expr.expr2.op1,"in_expr_op1");
	break;
      case gSET_LE_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].set_le_expr.expr_ineq.op0,"set_le_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].set_le_expr.expr_ineq.op1,"set_le_expr_op1");
	break;
      case gCARD_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].card_expr.expr1.op0,"card_expr_op0");
	break;
      case gRANGE_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].range_expr.expr2.op0,"range_expr_op1");
	write_edge(nodenum,g.nodeBuffer[nodenum].range_expr.expr2.op1,"range_expr_op0");
	break;
      case gCONVERT_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].convert_expr.expr1.op0,"convert_expr_op0");
	break;
      case gNOP_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].nop_expr.expr1.op0,"nop_expr_expr0");
	break;
      case gNON_LVALUE_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].non_lvalue_expr.expr1.op0,"non_lvalue_expr_op0");
	break;
      case gSAVE_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].save_expr.op0,"save_expr_op0");
	break;
      case gUNSAVE_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	break;
      case gRTL_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	break;
      case gADDR_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].addr_expr.op0,"addr_expr_op0");
	break;
      case gREFERENCE_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	break;
      case gENTRY_VALUE_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	break;
      case gCOMPLEX_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].complex_expr.expr2.op0,"complex_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].complex_expr.expr2.op1,"complex_expr_op1");
	break;
      case gCONJ_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].conj_expr.expr1.op0,"conj_expr_op0");
	break;
      case gREALPART_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].realpart_expr.expr1.op0,"realpart_expr_op0");
	break;
      case gPREDECREMENT_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].predecrement_expr.op0,"predecrement_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].predecrement_expr.op1,"predecrement_expr_op1");
	break;
      case gPREINCREMENT_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].preincrement_expr.op0,"preincrement_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].preincrement_expr.op1,"preincrement_expr_op1");
	break;
      case gPOSTDECREMENT_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].postdecrement_expr.op0,"postdecrement_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].postdecrement_expr.op1,"postdecrement_expr_op1");
	break;
      case gPOSTINCREMENT_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].postincrement_expr.op0,"postincrement_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].postincrement_expr.op1,"postincrement_expr_op1");
	break;
      case gVA_ARG_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].va_arg_expr.op0,"va_arg_expr_op0");
	break;
      case gTRY_CATCH_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].try_catch_expr.op0,"try_catch_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].try_catch_expr.op1,"try_catch_expr_op1");
	break;
      case gTRY_FINALLY_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].try_finally_expr.op0,"try_finally_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].try_finally_expr.op1,"try_finally_expr_op1");
	break;
      case gGOTO_SUBROUTINE_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].goto_subroutine_expr.op0,"goto_subroutine_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].goto_subroutine_expr.op1,"goto_subroutine_expr_op1");
	break;
      case gPOPDHC_EXPR:
	attr_common(g.nodeBuffer[nodenum].common);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	break;
      case gPOPDCC_EXPR:
	attr_common(g.nodeBuffer[nodenum].common);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	break;
      case gLABEL_EXPR:
	attr_common(g.nodeBuffer[nodenum].common);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	break;
      case gGOTO_EXPR:
	attr_common(g.nodeBuffer[nodenum].common);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	break;
      case gRETURN_EXPR:
	attr_common(g.nodeBuffer[nodenum].common);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	break;
      case gEXIT_EXPR:
	attr_common(g.nodeBuffer[nodenum].common);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].exit_expr.cond,"exit_expr_cond");
	break;
      case gLOOP_EXPR:
	attr_common(g.nodeBuffer[nodenum].common);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].loop_expr.body,"loop_expr_body");
	break;
      case gLABELED_BLOCK_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	break;
      case gEXPR_WITH_FILE_LOCATION:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].expr_with_file_location.expr2,"expr_with_file_location_expr2");
	break;
      case gSWITCH_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].switch_expr.op0,"switch_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].switch_expr.op1,"switch_expr_op1");
	break;
      case gSIZEOF_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].sizeof_expr.op0,"sizeof_expr_op0");
	break;
      case gARROW_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].arrow_expr.op0,"arrow_expr_op0");
	break;
      case gALIGNOF_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].alignof_expr.op0,"alignof_expr_op0");
	break;
      case gNEW_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].new_expr.op0,"new_expr_op0");
	break;
      case gVEC_NEW_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].vec_new_expr.op0,"vec_new_expr_op0");
	break;
      case gDELETE_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].delete_expr.op0,"delete_expr_op0");
	break;
      case gVEC_DELETE_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].vec_delete_expr.op0,"vec_delete_expr_op0");
	break;
      case gTYPE_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].type_expr.op0,"type_expr_op0");
	break;
      case gAGGR_INIT_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	write_number_attr("ctor",g.nodeBuffer[nodenum].aggr_init_expr.ctor);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].aggr_init_expr.fn,"aggr_init_expr_fn");
	write_edge(nodenum,g.nodeBuffer[nodenum].aggr_init_expr.args,"aggr_init_expr_args");
	write_edge(nodenum,g.nodeBuffer[nodenum].aggr_init_expr.decl,"aggr_init_expr_decl");
	break;
      case gEMPTY_CLASS_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	break;
      case gTEMPLATE_ID_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].template_id_expr.op0,"template_id_expr_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].template_id_expr.op1,"template_id_expr_op1");
	break;
      case gLOOKUP_EXPR:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].lookup_expr.op0,"lookup_expr_op0");
	break;
      case gFUNCTION_NAME:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	break;
      case gSUBOBJECT:
	attr_expr(g.nodeBuffer[nodenum].expr);
	write_number_attr("line",g.nodeBuffer[nodenum].subobject.line);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].subobject.clnp,"subobject_clnp");
	write_edge(nodenum,g.nodeBuffer[nodenum].subobject.next,"subobject_next");
	break;
      case gCTOR_STMT:
	attr_expr(g.nodeBuffer[nodenum].expr);
	write_number_attr("line",g.nodeBuffer[nodenum].ctor_stmt.line);
	write_number_attr("beg_end",g.nodeBuffer[nodenum].ctor_stmt.beg_end);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].ctor_stmt.next,"ctor_stmt_next");
	break;
      case gCTOR_INITIALIZER:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].ctor_initializer.op0,"ctor_initializer_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].ctor_initializer.op1,"ctor_initializer_op1");
	break;
      case gRETURN_INIT:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].return_init.op0,"return_init_op0");
	write_edge(nodenum,g.nodeBuffer[nodenum].return_init.op1,"return_init_op1");
	break;
      case gTAG_DEFN:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	break;
      case gIDENTITY_CONV:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].identity_conv.op0,"identity_conv_op0");
	break;
      case gLVALUE_CONV:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].lvalue_conv.op0,"lvalue_conv_op0");
	break;
      case gQUAL_CONV:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].qual_conv.op0,"qual_conv_op0");
	break;
      case gSTD_CONV:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].std_conv.op0,"std_conv_op0");
	break;
      case gPTR_CONV:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].ptr_conv.op0,"ptr_conv_op0");
	break;
      case gPMEM_CONV:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].pmem_conv.op0,"pmem_conv_op0");
	break;
      case gBASE_CONV:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].base_conv.op0,"base_conv_op0");
	break;
      case gREF_BIND:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].ref_bind.op0,"ref_bind_op0");
	break;
      case gUSER_CONV:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].user_conv.op0,"user_conv_op0");
	break;
      case gAMBIG_CONV:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].ambig_conv.op0,"ambig_conv_op0");
	break;
      case gRVALUE_CONV:
	attr_expr(g.nodeBuffer[nodenum].expr);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].rvalue_conv.op0,"rvalue_conv_op0");
	break;
      case gOP_IDENTIFIER:
	attr_common(g.nodeBuffer[nodenum].common);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	break;
      case gIDENTIFIER_NODE:
	attr_common(g.nodeBuffer[nodenum].common);
	write_number_attr("lngt",g.nodeBuffer[nodenum].identifier_node.lngt);
	write_string_attr("strg",&(g.stringBuffer[g.nodeBuffer[nodenum].identifier_node.strg]));
	write_number_attr("op_type_un",g.nodeBuffer[nodenum].identifier_node.op_type_un);
	write_number_attr("tynm",g.nodeBuffer[nodenum].identifier_node.tynm);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	break;
      case gINTEGER_CST:
	attr_cst(g.nodeBuffer[nodenum].gcc_cst);
	write_number_attr("high", g.nodeBuffer[nodenum].integer_cst.high);
	write_number_attr("low", g.nodeBuffer[nodenum].integer_cst.low);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_cst(g.nodeBuffer[nodenum].gcc_cst,nodenum);
	break;
      case gREAL_CST:
	attr_cst(g.nodeBuffer[nodenum].gcc_cst);
	write_double_attr("value",&(g.stringBuffer[g.nodeBuffer[nodenum].real_cst.value]));
	printf("  </node>\n",nodenum);
	/* edges */
	edge_cst(g.nodeBuffer[nodenum].gcc_cst,nodenum);
	break;
      case gPTRMEM_CST:
	attr_cst(g.nodeBuffer[nodenum].gcc_cst);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_cst(g.nodeBuffer[nodenum].gcc_cst,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].ptrmem_cst.clas,"ptrmem_cst_clas");
	write_edge(nodenum,g.nodeBuffer[nodenum].ptrmem_cst.mbr,"ptrmem_cst_mbr");
	break;
      case gCOMPLEX_CST:
	attr_cst(g.nodeBuffer[nodenum].gcc_cst);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_cst(g.nodeBuffer[nodenum].gcc_cst,nodenum);
	break;
      case gSTRING_CST:
	attr_cst(g.nodeBuffer[nodenum].gcc_cst);
	write_number_attr("lngt", g.nodeBuffer[nodenum].string_cst.lngt);
	write_string_attr("strg",&(g.stringBuffer[g.nodeBuffer[nodenum].string_cst.strg]));
	printf("  </node>\n",nodenum);
	/* edges */
	edge_cst(g.nodeBuffer[nodenum].gcc_cst,nodenum);
	break;
      case gTREE_LIST:
	attr_common(g.nodeBuffer[nodenum].common);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].tree_list.purpose,"tree_list_purpose");
	write_edge(nodenum,g.nodeBuffer[nodenum].tree_list.value,"tree_list_value");
	write_edge(nodenum,g.nodeBuffer[nodenum].tree_list.chan,"tree_list_chan");
	break;
      case gTREE_VEC:
	if (g.nodeBuffer[nodenum].tree_vec.binfo){
	  attr_common(g.nodeBuffer[nodenum].common);
	  write_boolean_attr("pub",g.nodeBuffer[nodenum].tree_vec.pub);
	  write_boolean_attr("prot",g.nodeBuffer[nodenum].tree_vec.prot);
	  write_boolean_attr("priv",g.nodeBuffer[nodenum].tree_vec.priv);
	  write_boolean_attr("virt",g.nodeBuffer[nodenum].tree_vec.virt);
	  printf("  </node>\n",nodenum);
	  /* edges */
	  edge_common(g.nodeBuffer[nodenum].common,nodenum);
	  write_edge(nodenum,g.nodeBuffer[nodenum].tree_vec.inherit,"tree_vec_inherit");
	  write_edge(nodenum,g.nodeBuffer[nodenum].tree_vec.type,"tree_vec_type");
	  write_edge(nodenum,g.nodeBuffer[nodenum].tree_vec.base,"tree_vec_base");
	} else {
	  attr_common(g.nodeBuffer[nodenum].common);
	  printf("  </node>\n",nodenum);
	  edge_common(g.nodeBuffer[nodenum].common,nodenum);
	}
	break;
      case gERROR_MARK:
	attr_common(g.nodeBuffer[nodenum].common);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	break;
      case gDEFAULT_ARG:
	attr_common(g.nodeBuffer[nodenum].common);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	break;
      case gCPLUS_BINDING:
	attr_common(g.nodeBuffer[nodenum].common);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	break;
      case gOVERLOAD:
	attr_common(g.nodeBuffer[nodenum].common);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	write_edge(nodenum,g.nodeBuffer[nodenum].overload.crnt,"overload_crnt");
	write_edge(nodenum,g.nodeBuffer[nodenum].overload.chan,"overload_chan");
	break;
      case gWRAPPER:
	attr_common(g.nodeBuffer[nodenum].common);
	printf("  </node>\n",nodenum);
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	break;
      case NODE_DELTED:
      case NODE_NONE:
	break;

	/* datrix types */
	/* datrix edges are handled differently than gcc edges */
 
      case dC_ASG_ND:
	datrix_asg_nd(g, g.nodeBuffer[nodenum].dc_asg_nd);
	printf("  </node>\n",nodenum);
	break;
      case dC_IDENTIFIER:
	datrix_identifier(g, g.nodeBuffer[nodenum].dc_identifier);
	printf("  </node>\n",nodenum);
	break;
      case dC_TYPE:
      case dC_BUILT_IN_TYPE:
	datrix_type(g, g.nodeBuffer[nodenum].dc_type);
	printf("  </node>\n",nodenum);
	break;
      case dC_AGGR_TYPE:
	datrix_type(g, g.nodeBuffer[nodenum].dc_type);
	switch(g.nodeBuffer[nodenum].dc_aggr_type.aggr_type){
	  case aggr_class:
	    write_string_attr("type","class");
	    break;
	  case aggr_record:
	    write_string_attr("type","record");
	    break;
	  case aggr_struct:
	    write_string_attr("type","struct");
	    break;
	  case aggr_union:
	    write_string_attr("type","union");
	    break;
	  case aggr_area:
	    write_string_attr("type","area");
	    break;
	  case aggr_interface:
	    write_string_attr("type","interface");
	    break;
	  default:
	    fprintf(stderr,"Unknown datrix aggregate type %d\n",
		g.nodeBuffer[nodenum].dc_aggr_type.aggr_type);
	    exit(1);
	}
	printf("  </node>\n",nodenum);
	break;
      case dC_ENUM_TYPE:
      case dC_TEMPL_PARM_TYPE:
      case dC_GENERIC_TYPE:
      case dC_GENERATED_TYPE:
      case dC_ALIAS_TYPE:
      case dC_FORWARD_TYPE:
      case dC_ARRAY_TYPE:
     case dC_PTR_TYPE:
      case dC_REF_TYPE:
      case dC_FCT_TYPE:
	datrix_type(g, g.nodeBuffer[nodenum].dc_type);
	printf("  </node>\n",nodenum);
	break;
      case dC_SCOPE:
      case dC_SCOPE_GLB:
      case dC_SCOPE_COMPIL:
      case dC_SCOPE_FILE:
      case dC_PACKAGE:
      case dC_NAME_SPACE:
      case dC_BLOCK:
      case dC_STATIC_BLOCK:
      case dC_TRY_BLOCK:
      case dC_CATCH_BLOCK:
      case dC_FINALLY_BLOCK:
      case dC_SYNCHRONIZED_BLOCK:
	datrix_scope(g, g.nodeBuffer[nodenum].dc_scope);
	printf("  </node>\n",nodenum);
	break;
      case dC_OBJECT:
      case dC_FORMAL_FCT_PARAM:
      case dC_FORMAL_CATCH_PARAM:
	datrix_object(g, g.nodeBuffer[nodenum].dc_object);
	printf("  </node>\n",nodenum);
	break;
      case dC_FUNCTION:
	datrix_identifier(g, g.nodeBuffer[nodenum].dc_identifier);
	write_boolean_attr("static",g.nodeBuffer[nodenum].dc_function.static_flag);
	write_boolean_attr("extern",g.nodeBuffer[nodenum].dc_function.extern_flag);
	write_boolean_attr("const",g.nodeBuffer[nodenum].dc_function.const_flag);
	switch(g.nodeBuffer[nodenum].dc_function.virtual_code){
	  case 0: 
	    write_string_attr("virtual","non-virtual");
	    break;
	  case 1: 
	    write_string_attr("virtual","virtual");
	    break;
	  case 2: 
	    write_string_attr("virtual","pure-virtual");
	    break;
	  case 3:
	    fprintf(stderr,"unknown virtual flag in function\n");
	    exit(1);
	}
	printf("  </node>\n",nodenum);
	break;
      case dC_LABEL:
      case dC_CASE_LABEL:
      case dC_DEFAULT_LABEL:
	datrix_label(g, g.nodeBuffer[nodenum].dc_label);
	printf("  </node>\n",nodenum);
	break;
      case dC_ENUMERATOR:
	datrix_identifier(g, g.nodeBuffer[nodenum].dc_identifier);
	printf("  </node>\n",nodenum);
	break;
      case dC_EXPRESSION:
      case dC_EMPTY_EXPR:
      case dC_SYNTAX_ERROR:
	datrix_expression(g, g.nodeBuffer[nodenum].dc_expression);
	printf("  </node>\n",nodenum);
	break;
      case dC_LITERAL:
	datrix_expression(g, g.nodeBuffer[nodenum].dc_expression);
	write_string_attr("value",&(g.stringBuffer[g.nodeBuffer[nodenum].dc_literal.value]));
	printf("  </node>\n",nodenum);
	break;
      case dC_NAME_REF:
	datrix_expression(g, g.nodeBuffer[nodenum].dc_expression);
	write_string_attr("name",&(g.stringBuffer[g.nodeBuffer[nodenum].dc_name_ref.name]));
	printf("  </node>\n",nodenum);
	break;
      case dC_OPERATOR:
      case dC_UNARY_OPERATOR:
      case dC_BINARY_OPERATOR:
      case dC_QUESTION_OPERATOR:
	datrix_operator(g, g.nodeBuffer[nodenum].dc_operator);
	printf("  </node>\n",nodenum);
	break;
      case dC_INIT_LIST:
      case dC_ACTUAL_FCT_PARAM_LIST:
      case dC_ACTUAL_TEMPL_PARAM_LIST:
      case dC_CREATION_EXPR:
      case dC_DESTROY_EXPR:
	datrix_expression(g, g.nodeBuffer[nodenum].dc_expression);
	printf("  </node>\n",nodenum);
	break;
      case dC_CNTRL_STMT:
      case dC_SELECTION:
      case dC_IF:
      case dC_SWITCH:
	datrix_ctrl_stmt(g, g.nodeBuffer[nodenum].dc_ctrl_stmt);
	printf("  </node>\n",nodenum);
	break;
      case dC_JUMP:
      case dC_GOTO:
      case dC_RETURN:
      case dC_BREAK:
      case dC_CONTINUE:
	datrix_jump(g, g.nodeBuffer[nodenum].dc_jump);
	printf("  </node>\n",nodenum);
	break;
      case dC_LOOP:
      case dC_DO_WHILE_LOOP:
      case dC_WHILE_DO_LOOP:
      case dC_FOR_LOOP:
	datrix_loop(g, g.nodeBuffer[nodenum].dc_loop);
	printf("  </node>\n",nodenum);
	break;
      case dC_COMMENT:
      case dC_JAVA_DOC_COMMENT:
	datrix_comment(g, g.nodeBuffer[nodenum].dc_comment);
	printf("  </node>\n",nodenum);
	break;
      case dC_SQL_STMT:
      case dC_THROW_SPEC:
      case dC_ASM_BLOCK:
	datrix_asg_nd(g, g.nodeBuffer[nodenum].dc_asg_nd);
	printf("  </node>\n",nodenum);
	break;
      case dC_ARC:
	datrix_arc(g.nodeBuffer[nodenum].dc_arc);
	break;
      case dC_INHERITANCE:
	datrix_arc_reference(g.nodeBuffer[nodenum].dc_arc_reference);
	write_boolean_attr("virt",g.nodeBuffer[nodenum].dc_inheritance.virt);
	switch (g.nodeBuffer[nodenum].dc_inheritance.visb){
	  case visb_unk:
	    write_string_attr("visb","unk");
	    break;
	  case visb_pri:
	    write_string_attr("visb","pri");
	    break;
	  case visb_pro:
	    write_string_attr("visb","pro");
	    break;
	  case visb_pub:
	    write_string_attr("visb","pub");
	    break;
	  default:
	    fprintf(stderr,"Illegal visibility\n");
	    exit(1);
	}
	break;
      case dC_ARC_REFERENCE:
      case dC_INSTANCE:
      case dC_IMPLEMENTS:
      case dC_ARC_FRIEND:
      case dC_ARC_DEF_LOC:
      case dC_REFERS_TO:
	datrix_arc_reference(g.nodeBuffer[nodenum].dc_arc_reference);
	break;
      case dC_ARC_SYNTAX:
      case dC_ARC_SON:
      case dC_ARC_TRUE:
      case dC_ARC_FALSE:
      case dC_ARC_OPD:
      case dC_IMPORT:
      case dC_ARC_COND:
      case dC_ARC_FOR_INIT:
      case dC_ARC_FOR_INCR:
      case dC_ARC_INIT_VAL:
      case dC_ARC_BIT_DIM:
      case dC_ARC_ARRAY_DIM:
	datrix_arc_syntax(g.nodeBuffer[nodenum].dc_arc_syntax);
	break;
      default: {
	int i;
	unsigned char *x;
	fprintf(stderr,"Node number %d\n",nodenum);
	fprintf(stderr,"unknown node type: %d\n" ,g.nodeBuffer[nodenum].common.type);
	fprintf(stderr,"node data:\n");
	  x = (unsigned char *)(&(g.nodeBuffer[nodenum]));
	i = 0;
	while (i < sizeof(gd_graph_node)){
	  fprintf(stderr," %2x",*x);
	  x++;
	  i++;
	  if (i % 16 == 0) {
	    fprintf(stderr,"\n");
	  }
	}
	fprintf(stderr,"\n");
      }
    }
    if (g.nodeBuffer[nodenum].common.type != NODE_DELTED &&
	g.nodeBuffer[nodenum].common.type != NODE_NONE) {
      if (g.nodeBuffer[nodenum].common.type < dC_ARC||
	 g.nodeBuffer[nodenum].common.type > dC_ARC_ARRAY_DIM){
	//printf("  </node>\n",nodenum);
      } else  {
	printf("  </edge>\n",
	    g.nodeBuffer[nodenum].dc_arc.src,
	    g.nodeBuffer[nodenum].dc_arc.dest);
      }
    }
  }
  printf(" </graph>\n");
  printf("</gxl>\n");

  return 0;
}

void datrix_asg_nd(gd_graph g, t_dc_asg_nd nd){
  /* for now nothing*/
  //write_string_attr("File",&(g.stringBuffer[nd.file]));
  //write_number_attr("Line",d.line);
}

void datrix_identifier(gd_graph g, t_dc_identifier ident){
  datrix_asg_nd(g, ident.dc_asg_nd);
  if (ident.name != -1){
    write_string_attr("name",&(g.stringBuffer[ident.name]));
  }
  switch (ident.visb){
    case visb_unk:
      write_string_attr("visb","unk");
      break;
    case visb_pri:
      write_string_attr("visb","pri");
      break;
    case visb_pro:
      write_string_attr("visb","pro");
      break;
    case visb_pub:
      write_string_attr("visb","pub");
      break;
    default:
      fprintf(stderr,"Illegal visibility\n");
      exit(1);
  }
}

void datrix_loop(gd_graph g, t_dc_loop loop){
  datrix_ctrl_stmt(g, loop.ctrl_stmt);
}

void datrix_ctrl_stmt(gd_graph g, t_dc_ctrl_stmt cntrl){
  datrix_asg_nd(g, cntrl.dc_asg_nd);
}

void datrix_comment(gd_graph g, t_dc_comment comment){
  datrix_asg_nd(g, comment.dc_asg_nd);
}

void datrix_jump(gd_graph g, t_dc_jump jump) {
  datrix_ctrl_stmt(g,jump.ctrl_stmt);
}

void datrix_expression(gd_graph g, t_dc_expression expr){
  datrix_asg_nd(g, expr.dc_asg_nd);
}
void datrix_operator(gd_graph g, t_dc_operator operator){
  datrix_expression(g, operator.expression);
  if (operator.op < 0 || operator.op > dc_range){
    fprintf(stderr,"unknown operator %d\n",operator.op);
  }
  write_string_attr("op",dc_op_string[operator.op]);
}

void datrix_type(gd_graph g, t_dc_type type){
  datrix_identifier(g, type.dc_identifier);
}

void datrix_object(gd_graph g, t_dc_object object){
  datrix_identifier(g, object.dc_identifier);
  write_boolean_attr("static",object.static_flag);
  write_boolean_attr("extern",object.extern_flag);
  write_boolean_attr("const",object.const_flag);
}
void datrix_label(gd_graph g, t_dc_label label){
  datrix_identifier(g, label.dc_identifier);
}

void datrix_scope(gd_graph g, t_dc_scope scope){
  datrix_identifier(g, scope.dc_identifier);
}

void datrix_arc(t_dc_arc  dc_arc){
    if (dc_arc.ordinal != 0){
      write_number_attr("order", dc_arc.ordinal);
    }
}
void datrix_arc_reference(t_dc_arc_reference  dc_arc_reference){
  datrix_arc(dc_arc_reference.arc);
}

void datrix_arc_syntax(t_dc_arc_syntax  dc_arc_syntax){
  datrix_arc(dc_arc_syntax.arc);
}

void attr_decl(gd_graph g, t_gcc_decl decl) {
  attr_common(decl.common);
  if (decl.srcl != -1) {
    if (decl.srcf >= g.curStringPos){
      fprintf(stderr,"string index is out of bounds\n");
      exit(1);
    } else {
      write_string_attr("File", &(g.stringBuffer[decl.srcf]));
      write_number_attr("Line", decl.srcl);
    }
  }
  switch (decl.lang) {
    case src_lang_c:
      write_string_attr("src_lang", "c");
      break;
    case src_lang_cp:
      write_string_attr("src_lang", "c++");
      break;
    case src_lang_java:
      write_string_attr("src_lang", "java");
      break;
    default:
      break;
  }
  write_boolean_attr("artificial",decl.artificial);
}

void edge_decl(gd_graph g, t_gcc_decl decl, int pos) {
  edge_common(decl.common,pos);
  write_edge(pos,decl.name,"decl_name");
  write_edge(pos,decl.mngl,"decl_mngl");
  write_edge(pos,decl.type,"decl_type");
  write_edge(pos,decl.chan,"decl_chan");
  write_edge(pos,decl.scpe,"decl_scpe");
}

void edge_common(gd_graph_common comm, int pos) {
  /* no edges from common block at this time */
}

void attr_common(gd_graph_common comm){
  printf("   <attr name=\"unsigned_flag\"><boolean>%d</boolean></attr>\n",
      comm.unsigned_flag);
  printf("   <attr name=\"constant_flag\"><boolean>%d</boolean></attr>\n",
      comm.constant_flag);
  printf("   <attr name=\"static_flag\"><boolean>%d</boolean></attr>\n",
      comm.static_flag);
  printf("   <attr name=\"public_flag\"><boolean>%d</boolean></attr>\n",
      comm.public_flag);
  printf("   <attr name=\"private_flag\"><boolean>%d</boolean></attr>\n",
      comm.private_flag);
  printf("   <attr name=\"protected_flag\"><boolean>%d</boolean></attr>\n",
      comm.protected_flag);
}

void attr_type(t_gcc_type type) {
  attr_common(type.common);
  printf("   <attr name=\"algn\"><number>%d</number></attr>\n",type.algn);
  printf("   <attr name=\"qual_const\"><boolean>%d</boolean></attr>\n",
	type.qual_const);
  printf("   <attr name=\"qual_volatile\"><boolean>%d</boolean></attr>\n",
	type.qual_volatile);
  printf("   <attr name=\"qual_restrict\"><boolean>%d</boolean></attr>\n",
	type.qual_restrict);
}

void edge_type(t_gcc_type type,int pos) {
  edge_common(type.common,pos);
  write_edge(pos,type.name,"type_name");
  write_edge(pos,type.size,"type_size");
  write_edge(pos,type.unql,"type_unql");
}

void attr_cst(t_gcc_const cst) {
  attr_common(cst.common);
}

void edge_cst(t_gcc_const cst,int pos) {
  edge_common(cst.common,pos);
  write_edge(pos,cst.type,"cst_type");
}

void attr_expr(t_gcc_expr expr) {
  attr_common(expr.common);
}

void edge_expr(t_gcc_expr expr, int pos) {
  edge_common(expr.common,pos);
  write_edge(pos,expr.type,"expr_type");
}

void write_edge(int from, int to, const char * type){
  if (to != -1){
    printf("  <edge from=\"n%d\" to=\"n%d\">\n",from,to);
    write_gxl_type(type);
    printf("  </edge>\n");
  }
}

void write_boolean_attr(const char * attr, int val){
  printf("   <attr name=\"%s\"><bool>%s</bool></attr>\n",attr,val?"true":"false");
}

void write_number_attr(const char * attr, int val){
  printf("   <attr name=\"%s\"><int>%d</int></attr>\n",attr,val);
}

void write_double_attr(const char * attr, char * val){
  printf("   <attr name=\"%s\"><double>%s</double></attr>\n",attr,val);
}

void write_string_attr(const char * attr, char * val){
  char * rover;
  printf("   <attr name=\"%s\"><string>",attr);
  for(rover = val; *rover != '\0'; rover++){
    /* print according to xml rules */
    /* TODO */
    if (isprint(*rover)) {
      switch (*rover) {
	case '&':
	  fputs("&amp;",stdout);
	  break;
	case '\'':
	  fputs("&apos;",stdout);
	  break;
	case '>':
	  fputs("&gt;",stdout);
	  break;
	case '<':
	  fputs("&lt;",stdout);
	  break;
	case '"':
	  fputs("&quot;",stdout);
	  break;
	default:
	  printf("%c",*rover);
      }
    } else {
      printf("&#x%x;",*rover);
    }
  }
  printf("</string></attr>\n");
}

void write_gxl_type(const char * typename) {
    printf("   <type xlink:href=\"gd.xml#%s\" />\n",typename);
}


