#include <stdio.h>
#include <stdlib.h>

#include "gd_graph.h"
#include "gd_utils.h"

void attr_decl(gd_graph g, t_gcc_decl decl);
void attr_common(gd_graph_common comm);
void attr_type(t_gcc_type type);
void attr_cst(t_gcc_const cst);
void attr_expr(t_gcc_expr expr);

void edge_decl(gd_graph g, t_gcc_decl decl,int pos);
void edge_common(gd_graph_common comm,int pos);
void edge_type(t_gcc_type type,int pos);
void edge_cst(t_gcc_const cst,int pos);
void edge_expr(t_gcc_expr expr,int pos);

void write_edge_tuple(int from, int to, const char * type);
void write_boolean_attr(const char * attr, int val);
void write_number_attr(const char * attr, int val);
void write_string_attr(const char * attr, char * val);
void write_double_attr(const char * attr, char * val);

void write_inst_tuple(const char * typename,int nodenum);

void datrix_arc_syntax(t_dc_arc_syntax  dc_arc_syntax);
void datrix_arc(t_dc_arc  dc_arc_syntax);

void datrix_scope(gd_graph g, t_dc_scope scope);
void datrix_identifier(gd_graph g, t_dc_identifier ident);
void datrix_asg_nd(gd_graph g, t_dc_asg_nd nd);
void datrix_type(gd_graph g, t_dc_type type);
void datrix_object(gd_graph g, t_dc_object object);
void datrix_label(gd_graph g, t_dc_label label);
void datrix_expression(gd_graph g, t_dc_expression expr);
void datrix_operator(gd_graph g, t_dc_operator operator);
void datrix_ctrl_stmt(gd_graph g, t_dc_ctrl_stmt cntrl);
void datrix_jump(gd_graph g, t_dc_jump jump);
void datrix_loop(gd_graph g, t_dc_loop loop);
void datrix_comment(gd_graph g, t_dc_comment comment);
void datrix_arc_reference(t_dc_arc_reference  dc_arc_reference);

void write_tuples (gd_graph);
void write_attributes (gd_graph);
void write_stuff (gd_graph, int);
void write_inst_tuple (const char *, node_ptr);
void begin_inst_attributes (const char *, node_ptr);
void begin_edge_attributes (const char *, node_ptr, node_ptr);
void end_attributes ();

// Attributes are printed when there are some.
// But we don't know there are some until we try to print them.
// So the following variables retain the state until we're sure:
static char attrib_header [100];
static int attrib_count;

char * progname;

void usage(){
  fprintf(stderr,"usage: %s dumpfile\n",progname);
  exit(1);
}

int used_nodes[NUM_NODES];

int main(int argc, char * argv[]){
  gd_graph g;
  char * filename;
  int nodenum,i;
  progname = argv[0];

  if (argc != 2) {
    usage();
  }

  filename = argv[1];

  g = gd_graph_read_binary_file(filename,1.0,1.0,1.0);
  if (g.curStringPos == 0) {
    fprintf(stderr,"could not get graph\n");
    exit(1);
  }

  /* find all used node types */
  for (i = 0; i < NUM_NODES; i++){
    used_nodes[i] = 0;
  }
  for (nodenum = 0; nodenum < g.curNodePos; nodenum++) {
    used_nodes[g.nodeBuffer[nodenum].common.type] = 1;
  }
  write_tuples (g);
  write_attributes (g);

  return 0;
}

// This variable controls the writing of whatever.
static enum { tuples, attributes } writing;

void write_tuples (gd_graph g)
{
    writing = tuples;
    printf ("FACT TUPLE :\n");
    write_stuff (g, 0);
}

void write_attributes (gd_graph g)
{
    writing = attributes;
    printf ("FACT ATTRIBUTE :\n");
    write_stuff (g, 1);
}

void write_stuff (gd_graph g, int attribs)
{
  node_ptr nodenum;
  for (nodenum = 0; nodenum < g.curNodePos; nodenum++) {
    switch (g.nodeBuffer[nodenum].common.type) {
      case gNAMESPACE_DECL:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_decl(g,g.nodeBuffer[nodenum].decl);
	end_attributes ();
	/* edges */
	edge_decl(g,g.nodeBuffer[nodenum].decl,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].namespace_decl.dcls,"dcls");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].namespace_decl.alis,"alis");
	break;
      case gFUNCTION_DECL:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_decl(g,g.nodeBuffer[nodenum].decl);
	write_boolean_attr("undef",g.nodeBuffer[nodenum].function_decl.undef);
	write_number_attr ("ext_static", g.nodeBuffer[nodenum].function_decl.ext_static);
	write_boolean_attr("member", g.nodeBuffer[nodenum].function_decl.member);
	write_boolean_attr("constr", g.nodeBuffer[nodenum].function_decl.constr);
	write_boolean_attr("decon", g.nodeBuffer[nodenum].function_decl.decon);
	write_boolean_attr("oper", g.nodeBuffer[nodenum].function_decl.oper);
	write_boolean_attr("conver",g.nodeBuffer[nodenum].function_decl.conver);
	write_boolean_attr("ginit",g.nodeBuffer[nodenum].function_decl.ginit);
	write_boolean_attr("gfini",g.nodeBuffer[nodenum].function_decl.gfini);
	end_attributes ();
	/* edges */
	edge_decl(g,g.nodeBuffer[nodenum].decl,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].function_decl.args,"args");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].function_decl.body,"body");
	break;
      case gVAR_DECL:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_decl(g,g.nodeBuffer[nodenum].decl);
	write_number_attr("algn",g.nodeBuffer[nodenum].var_decl.algn);
	write_number_attr("used",g.nodeBuffer[nodenum].var_decl.used);
	write_number_attr("reg",g.nodeBuffer[nodenum].var_decl.reg);
	end_attributes ();
	/* edges */
	edge_decl(g,g.nodeBuffer[nodenum].decl,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].var_decl.init,"init");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].var_decl.size,"size");
	break;
      case gPARM_DECL:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_decl(g,g.nodeBuffer[nodenum].decl);
	write_number_attr("algn",g.nodeBuffer[nodenum].parm_decl.algn);
	write_number_attr("used",g.nodeBuffer[nodenum].parm_decl.used);
	write_number_attr("reg",g.nodeBuffer[nodenum].parm_decl.reg);
	end_attributes ();
	/* edges */
	edge_decl(g,g.nodeBuffer[nodenum].decl,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].parm_decl.argt,"argt");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].parm_decl.size,"size");
	break;
      case gTYPE_DECL:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_decl(g,g.nodeBuffer[nodenum].decl);
	end_attributes ();
	/* edges */
	edge_decl(g,g.nodeBuffer[nodenum].decl,nodenum);
	break;
      case gLABEL_DECL:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_decl(g,g.nodeBuffer[nodenum].decl);
	end_attributes ();
	/* edges */
	edge_decl(g,g.nodeBuffer[nodenum].decl,nodenum);
	break;
      case gCONST_DECL:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_decl(g,g.nodeBuffer[nodenum].decl);
	end_attributes ();
	/* edges */
	edge_decl(g,g.nodeBuffer[nodenum].decl,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].const_decl.cnst,"cnst");
	break;
      case gRESULT_DECL:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_decl(g,g.nodeBuffer[nodenum].decl);
	write_number_attr("algn",g.nodeBuffer[nodenum].result_decl.algn);
	end_attributes ();
	/* edges */
	edge_decl(g,g.nodeBuffer[nodenum].decl,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].result_decl.init,"init");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].result_decl.size,"size");
	break;
      case gFIELD_DECL:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_decl(g,g.nodeBuffer[nodenum].decl);
	write_number_attr("algn",g.nodeBuffer[nodenum].field_decl.algn);
	write_number_attr("bitfield",g.nodeBuffer[nodenum].field_decl.bitfield);
	end_attributes ();
	/* edges */
	edge_decl(g,g.nodeBuffer[nodenum].decl,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].field_decl.init,"init");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].field_decl.size,"size");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].field_decl.bpos,"bpos");
	break;
      case gTEMPLATE_DECL:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_decl(g,g.nodeBuffer[nodenum].decl);
	end_attributes ();
	/* edges */
	edge_decl(g,g.nodeBuffer[nodenum].decl,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].template_decl.rslt,"rslt");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].template_decl.inst,"inst");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].template_decl.spcs,"spcs");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].template_decl.parms,"parms");
	break;
      case gUSING_DECL:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_decl(g,g.nodeBuffer[nodenum].decl);
	end_attributes ();
	/* edges */
	edge_decl(g,g.nodeBuffer[nodenum].decl,nodenum);
	break;
      case gFUNCTION_TYPE:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_type(g.nodeBuffer[nodenum].type);
	end_attributes ();
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].function_type.retn,"retn");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].function_type.prms,"prms");
	break;
      case gINTEGER_TYPE:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_type(g.nodeBuffer[nodenum].type);
	write_boolean_attr("unsigned",g.nodeBuffer[nodenum].integer_type.unsgnd);
	write_number_attr("prec", g.nodeBuffer[nodenum].integer_type.prec);
	end_attributes ();
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].integer_type.min,"min");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].integer_type.max,"max");
	break;
      case gARRAY_TYPE:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_type(g.nodeBuffer[nodenum].type);
	end_attributes ();
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].array_type.elts,"elts");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].array_type.domn,"domn");
	break;
      case gVOID_TYPE:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_type(g.nodeBuffer[nodenum].type);
	end_attributes ();
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	break;
      case gREAL_TYPE:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_type(g.nodeBuffer[nodenum].type);
	write_number_attr("prec", g.nodeBuffer[nodenum].real_type.prec);
	end_attributes ();
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	break;
      case gCOMPLEX_TYPE:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_type(g.nodeBuffer[nodenum].type);
	end_attributes ();
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	break;
      case gVECTOR_TYPE:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_type(g.nodeBuffer[nodenum].type);
	end_attributes ();
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	break;
      case gENUMERAL_TYPE:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_type(g.nodeBuffer[nodenum].type);
	write_boolean_attr("unsigned",g.nodeBuffer[nodenum].enumeral_type.unsgnd);
	write_number_attr("prec", g.nodeBuffer[nodenum].enumeral_type.prec);
	end_attributes ();
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].enumeral_type.min,"min");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].enumeral_type.max,"max");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].enumeral_type.csts,"csts");
	break;
      case gBOOLEAN_TYPE:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_type(g.nodeBuffer[nodenum].type);
	end_attributes ();
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	break;
      case gCHAR_TYPE:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_type(g.nodeBuffer[nodenum].type);
	end_attributes ();
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	break;
      case gPOINTER_TYPE:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_type(g.nodeBuffer[nodenum].type);
	write_boolean_attr("ptrmem",g.nodeBuffer[nodenum].pointer_type.ptrmem);
	end_attributes ();
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].pointer_type.ptd,"ptd");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].pointer_type.cls,"cls");
	break;
      case gOFFSET_TYPE:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_type(g.nodeBuffer[nodenum].type);
	end_attributes ();
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	break;
      case gREFERENCE_TYPE:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_type(g.nodeBuffer[nodenum].type);
	end_attributes ();
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].reference_type.refd,"refd");
	break;
      case gMETHOD_TYPE:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_type(g.nodeBuffer[nodenum].type);
	end_attributes ();
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].method_type.clas,"clas");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].method_type.retn,"retn");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].method_type.prms,"prms");
	break;
      case gFILE_TYPE:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_type(g.nodeBuffer[nodenum].type);
	end_attributes ();
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	break;
      case gSET_TYPE:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_type(g.nodeBuffer[nodenum].type);
	end_attributes ();
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	break;
      case gRECORD_TYPE:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_type(g.nodeBuffer[nodenum].type);
	write_boolean_attr("ptrmem",g.nodeBuffer[nodenum].record_type.ptrmem);
	write_boolean_attr("isClass",g.nodeBuffer[nodenum].record_type.isClass);
	end_attributes ();
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].record_type.flds,"flds");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].record_type.fncs,"fncs");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].record_type.ptd,"ptd");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].record_type.cls,"cls");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].record_type.vfld,"vfld");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].record_type.binf,"binf");
	break;
      case gUNION_TYPE:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_type(g.nodeBuffer[nodenum].type);
	write_boolean_attr("ptrmem",g.nodeBuffer[nodenum].union_type.ptrmem);
	end_attributes ();
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].union_type.flds,"flds");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].union_type.fncs,"fncs");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].union_type.ptd,"ptd");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].union_type.cls,"cls");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].union_type.vfld,"vfld");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].union_type.binf,"binf");
	break;
      case gQUAL_UNION_TYPE:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_type(g.nodeBuffer[nodenum].type);
	end_attributes ();
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	break;
      case gLANG_TYPE:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_type(g.nodeBuffer[nodenum].type);
	end_attributes ();
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	break;
      case gTEMPLATE_TYPE_PARM:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_type(g.nodeBuffer[nodenum].type);
	end_attributes ();
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	break;
      case gTEMPLATE_PARM_INDEX:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_common(g.nodeBuffer[nodenum].common);
	end_attributes ();
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	break;
      case gTEMPLATE_TEMPLATE_PARM:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_type(g.nodeBuffer[nodenum].type);
	end_attributes ();
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	break;
      case gBOUND_TEMPLATE_TEMPLATE_PARM:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_type(g.nodeBuffer[nodenum].type);
	end_attributes ();
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	break;
      case gTYPENAME_TYPE:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_type(g.nodeBuffer[nodenum].type);
	end_attributes ();
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	break;
      case gTYPEOF_TYPE:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_type(g.nodeBuffer[nodenum].type);
	end_attributes ();
	/* edges */
	edge_type(g.nodeBuffer[nodenum].type,nodenum);
	break;
      case gBLOCK:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_common(g.nodeBuffer[nodenum].common);
	end_attributes ();
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	break;
      case gCOMPOUND_STMT:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	write_number_attr("line", g.nodeBuffer[nodenum].compound_stmt.line);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].compound_stmt.body,"body");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].compound_stmt.next,"next");
	break;
      case gEXPR_STMT:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	write_number_attr("line", g.nodeBuffer[nodenum].expr_stmt.line);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].expr_stmt.texpr,"expr");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].expr_stmt.next,"next");
	break;
      case gDECL_STMT:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	write_number_attr("line", g.nodeBuffer[nodenum].decl_stmt.line);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].decl_stmt.decl,"decl");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].decl_stmt.next,"next");
	break;
      case gSCOPE_STMT:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	write_number_attr("line", g.nodeBuffer[nodenum].scope_stmt.line);
	write_number_attr("null", g.nodeBuffer[nodenum].scope_stmt.null);
	write_number_attr("begin_end", g.nodeBuffer[nodenum].scope_stmt.begin_end);
	write_number_attr("clnp", g.nodeBuffer[nodenum].scope_stmt.clnp);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].scope_stmt.next,"next");
	break;
      case gRETURN_STMT:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	write_number_attr("line", g.nodeBuffer[nodenum].return_stmt.line);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].return_stmt.ret_expr,"expr");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].return_stmt.next,"next");
	break;
      case gSTMT_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].stmt_expr.stmt,"stmt");
	break;
      case gBREAK_STMT:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	write_number_attr("line", g.nodeBuffer[nodenum].break_stmt.line);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].break_stmt.next,"next");
	break;
      case gCONTINUE_STMT:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	write_number_attr("line", g.nodeBuffer[nodenum].continue_stmt.line);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].continue_stmt.next,"next");
	break;
      case gIF_STMT:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	write_number_attr("line", g.nodeBuffer[nodenum].if_stmt.line);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].if_stmt.cond,"cond");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].if_stmt.thenc,"thenc");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].if_stmt.elsec,"elsec");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].if_stmt.next,"next");
	break;
      case gSWITCH_STMT:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	write_number_attr("line", g.nodeBuffer[nodenum].switch_stmt.line);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].switch_stmt.cond,"cond");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].switch_stmt.body,"body");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].switch_stmt.next,"next");
	break;
      case gLABEL_STMT:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	write_number_attr("line", g.nodeBuffer[nodenum].label_stmt.line);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].label_stmt.label,"label");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].label_stmt.next,"next");
	break;
      case gTRY_BLOCK:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	write_number_attr("line", g.nodeBuffer[nodenum].try_block.line);
	write_boolean_attr("cleanup", g.nodeBuffer[nodenum].try_block.cleanup);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].try_block.body,"body");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].try_block.hdlr,"hdlr");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].try_block.next,"next");
	break;
      case gHANDLER:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	write_number_attr("line", g.nodeBuffer[nodenum].handler.line);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].handler.body,"body");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].handler.next,"next");
	break;
      case gSTART_CATCH_STMT:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	write_number_attr("line", g.nodeBuffer[nodenum].start_catch_stmt.line);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].start_catch_stmt.catch_type,"catch_type");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].start_catch_stmt.next,"next");
	break;
      case gCLEANUP_STMT:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	write_number_attr("line", g.nodeBuffer[nodenum].cleanup_stmt.line);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].cleanup_stmt.decl,"decl");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].cleanup_stmt.clean_expr,"expr");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].cleanup_stmt.next,"next");
	break;
      case gCASE_LABEL:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].case_label.low,"low");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].case_label.high,"high");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].case_label.next,"next");
	break;
      case gGOTO_STMT:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	write_number_attr("line", g.nodeBuffer[nodenum].goto_stmt.line);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].goto_stmt.dest,"dest");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].goto_stmt.next,"next");
	break;
      case gFOR_STMT:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	write_number_attr("line", g.nodeBuffer[nodenum].for_stmt.line);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].for_stmt.init,"init");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].for_stmt.cond,"cond");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].for_stmt.inc_expr,"expr");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].for_stmt.body,"body");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].for_stmt.next,"next");
	break;
      case gWHILE_STMT:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	write_number_attr("line", g.nodeBuffer[nodenum].while_stmt.line);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].while_stmt.cond,"cond");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].while_stmt.body,"body");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].while_stmt.next,"next");
	break;
      case gDO_STMT:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	write_number_attr("line", g.nodeBuffer[nodenum].do_stmt.line);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].do_stmt.cond,"cond");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].do_stmt.body,"body");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].do_stmt.next,"next");
	break;
      case gASM_STMT:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	write_number_attr("line", g.nodeBuffer[nodenum].asm_stmt.line);
	write_number_attr("vol", g.nodeBuffer[nodenum].asm_stmt.vol);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].asm_stmt.strg,"strg");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].asm_stmt.outs,"outs");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].asm_stmt.ins,"ins");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].asm_stmt.clbr,"clbr");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].asm_stmt.next,"next");
	break;
      case gCOMPONENT_REF:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_common(g.nodeBuffer[nodenum].common);
	end_attributes ();
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].component_ref.op0,"op0");
	break;
      case gBIT_FIELD_REF:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_common(g.nodeBuffer[nodenum].common);
	end_attributes ();
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	break;
      case gINDIRECT_REF:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_common(g.nodeBuffer[nodenum].common);
	end_attributes ();
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].indirect_ref.op0,"op0");
	break;
      case gBUFFER_REF:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_common(g.nodeBuffer[nodenum].common);
	end_attributes ();
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	break;
      case gARRAY_REF:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_common(g.nodeBuffer[nodenum].common);
	end_attributes ();
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].array_ref.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].array_ref.op1,"op1");
	break;
      case gOFFSET_REF:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_common(g.nodeBuffer[nodenum].common);
	end_attributes ();
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].offset_ref.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].offset_ref.op1,"op1");
	break;
      case gSCOPE_REF:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_common(g.nodeBuffer[nodenum].common);
	end_attributes ();
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].scope_ref.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].scope_ref.op1,"op1");
	break;
      case gMEMBER_REF:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_common(g.nodeBuffer[nodenum].common);
	end_attributes ();
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].member_ref.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].member_ref.op1,"op1");
	break;
      case gCONSTRUCTOR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].constructor.elts,"elts");
	break;
      case gCOMPOUND_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_common(g.nodeBuffer[nodenum].common);
	end_attributes ();
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].compound_expr.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].compound_expr.op1,"op1");
	break;
      case gMODIFY_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].modify_expr.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].modify_expr.op1,"op1");
	break;
      case gINIT_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].init_expr.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].init_expr.op1,"op1");
	break;
      case gTHROW_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].throw_expr.op0,"op0");
	break;
      case gTARGET_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].target_expr.decl,"decl");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].target_expr.init,"init");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].target_expr.clnp,"clnp");
	break;
      case gCOND_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].cond_expr.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].cond_expr.op1,"op1");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].cond_expr.op2,"op2");
	break;
      case gBIND_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].bind_expr.vars,"vars");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].bind_expr.body,"body");
	break;
      case gCALL_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].call_expr.fn,"fn");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].call_expr.args,"args");
	break;
      case gMETHOD_CALL_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	break;
      case gWITH_CLEANUP_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	break;
      case gCLEANUP_POINT_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].cleanup_point_expr.op0,"op0");
	break;
      case gPLACEHOLDER_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_common(g.nodeBuffer[nodenum].common);
	end_attributes ();
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	break;
      case gWITH_RECORD_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	break;
      case gPLUS_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].plus_expr.expr2.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].plus_expr.expr2.op1,"op1");
	break;
      case gMINUS_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].minus_expr.expr2.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].minus_expr.expr2.op1,"op1");
	break;
      case gMULT_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].mult_expr.expr2.op0,"expr0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].mult_expr.expr2.op1,"expr1");
	break;
      case gTRUNC_DIV_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].trunc_div_expr.expr2.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].trunc_div_expr.expr2.op1,"op1");
	break;
      case gCEIL_DIV_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].ceil_div_expr.expr2.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].ceil_div_expr.expr2.op1,"op1");
	break;
      case gFLOOR_DIV_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].floor_div_expr.expr2.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].floor_div_expr.expr2.op1,"op1");
	break;
      case gROUND_DIV_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].round_div_expr.expr2.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].round_div_expr.expr2.op1,"op1");
	break;
      case gTRUNC_MOD_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].trunc_mod_expr.expr2.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].trunc_mod_expr.expr2.op1,"op1");
	break;
      case gCEIL_MOD_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].ceil_mod_expr.expr2.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].ceil_mod_expr.expr2.op1,"op1");
	break;
      case gFLOOR_MOD_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].floor_mod_expr.expr2.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].floor_mod_expr.expr2.op1,"op1");
	break;
      case gROUND_MOD_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].round_mod_expr.expr2.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].round_mod_expr.expr2.op1,"op1");
	break;
      case gRDIV_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].rdiv_expr.expr2.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].rdiv_expr.expr2.op1,"op1");
	break;
      case gEXACT_DIV_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].exact_div_expr.expr2.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].exact_div_expr.expr2.op1,"op1");
	break;
      case gFIX_TRUNC_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].fix_trunc_expr.expr1.op0,"op0");
	break;
      case gFIX_CEIL_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].fix_ceil_expr.expr1.op0,"op0");
	break;
      case gFIX_FLOOR_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].fix_floor_expr.expr1.op0,"op0");
	break;
      case gFIX_ROUND_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].fix_round_expr.expr1.op0,"op0");
	break;
      case gFLOAT_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].float_expr.expr1.op0,"op0");
	break;
      case gEXPON_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].expon_expr.expr2.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].expon_expr.expr2.op1,"op1");
	break;
      case gNEGATE_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].negate_expr.expr1.op0,"op0");
	break;
      case gMIN_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].min_expr.expr2.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].min_expr.expr2.op1,"op1");
	break;
      case gMAX_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].max_expr.expr2.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].max_expr.expr2.op1,"op1");
	break;
      case gABS_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].abs_expr.expr1.op0,"op0");
	break;
      case gFFS_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].ffs_expr.expr1.op0,"op0");
	break;
      case gLSHIFT_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].lshift_expr.expr2.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].lshift_expr.expr2.op1,"op1");
	break;
      case gRSHIFT_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].rshift_expr.expr2.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].rshift_expr.expr2.op1,"op1");
	break;
      case gLROTATE_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].lrotate_expr.expr2.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].lrotate_expr.expr2.op1,"op1");
	break;
      case gRROTATE_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].rrotate_expr.expr2.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].rrotate_expr.expr2.op1,"op1");
	break;
      case gBIT_IOR_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].bit_ior_expr.expr2.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].bit_ior_expr.expr2.op1,"op1");
	break;
      case gBIT_XOR_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].bit_xor_expr.expr2.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].bit_xor_expr.expr2.op1,"op1");
	break;
      case gBIT_AND_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].bit_and_expr.expr2.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].bit_and_expr.expr2.op1,"op1");
	break;
      case gBIT_ANDTC_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].bit_andtc_expr.expr2.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].bit_andtc_expr.expr2.op1,"op1");
	break;
      case gBIT_NOT_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].bit_not_expr.expr1.op0,"op0");
	break;
      case gTRUTH_ANDIF_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].truth_andif_expr.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].truth_andif_expr.op1,"op1");
	break;
      case gTRUTH_ORIF_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].truth_orif_expr.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].truth_orif_expr.op1,"op1");
	break;
      case gTRUTH_AND_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].truth_and_expr.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].truth_and_expr.op1,"op1");
	break;
      case gTRUTH_OR_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].truth_or_expr.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].truth_or_expr.op1,"op1");
	break;
      case gTRUTH_XOR_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].truth_xor_expr.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].truth_xor_expr.op1,"op1");
	break;
      case gTRUTH_NOT_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].truth_not_expr.op0,"op0");
	break;
      case gLT_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].lt_expr.expr_ineq.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].lt_expr.expr_ineq.op1,"op1");
	break;
      case gLE_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].le_expr.expr_ineq.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].le_expr.expr_ineq.op1,"op1");
	break;
      case gGT_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].gt_expr.expr_ineq.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].gt_expr.expr_ineq.op1,"op1");
	break;
      case gGE_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].ge_expr.expr_ineq.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].ge_expr.expr_ineq.op1,"op1");
	break;
      case gEQ_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].eq_expr.expr_ineq.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].eq_expr.expr_ineq.op1,"op1");
	break;
      case gNE_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].ne_expr.expr_ineq.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].ne_expr.expr_ineq.op1,"op1");
	break;
      case gUNORDERED_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].unordered_expr.expr_ineq.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].unordered_expr.expr_ineq.op1,"op1");
	break;
      case gORDERED_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].ordered_expr.expr_ineq.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].ordered_expr.expr_ineq.op1,"op1");
	break;
      case gUNLT_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].unlt_expr.expr_ineq.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].unlt_expr.expr_ineq.op1,"op1");
	break;
      case gUNLE_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].unle_expr.expr_ineq.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].unle_expr.expr_ineq.op1,"op1");
	break;
      case gUNGT_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].ungt_expr.expr_ineq.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].ungt_expr.expr_ineq.op1,"op1");
	break;
      case gUNGE_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].unge_expr.expr_ineq.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].unge_expr.expr_ineq.op1,"op1");
	break;
      case gUNEQ_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].uneq_expr.expr_ineq.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].uneq_expr.expr_ineq.op1,"op1");
	break;
      case gIN_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].in_expr.expr2.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].in_expr.expr2.op1,"op1");
	break;
      case gSET_LE_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].set_le_expr.expr_ineq.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].set_le_expr.expr_ineq.op1,"op1");
	break;
      case gCARD_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].card_expr.expr1.op0,"op0");
	break;
      case gRANGE_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].range_expr.expr2.op0,"op1");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].range_expr.expr2.op1,"op0");
	break;
      case gCONVERT_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].convert_expr.expr1.op0,"op0");
	break;
      case gNOP_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].nop_expr.expr1.op0,"op0");
	break;
      case gNON_LVALUE_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].non_lvalue_expr.expr1.op0,"op0");
	break;
      case gSAVE_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].save_expr.op0,"op0");
	break;
      case gUNSAVE_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	break;
      case gRTL_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	break;
      case gADDR_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].addr_expr.op0,"op0");
	break;
      case gREFERENCE_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	break;
      case gENTRY_VALUE_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	break;
      case gCOMPLEX_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].complex_expr.expr2.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].complex_expr.expr2.op1,"op1");
	break;
      case gCONJ_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].conj_expr.expr1.op0,"op0");
	break;
      case gREALPART_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].realpart_expr.expr1.op0,"op0");
	break;
      case gPREDECREMENT_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].predecrement_expr.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].predecrement_expr.op1,"op1");
	break;
      case gPREINCREMENT_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].preincrement_expr.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].preincrement_expr.op1,"op1");
	break;
      case gPOSTDECREMENT_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].postdecrement_expr.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].postdecrement_expr.op1,"op1");
	break;
      case gPOSTINCREMENT_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].postincrement_expr.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].postincrement_expr.op1,"op1");
	break;
      case gVA_ARG_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].va_arg_expr.op0,"op0");
	break;
      case gTRY_CATCH_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].try_catch_expr.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].try_catch_expr.op1,"op1");
	break;
      case gTRY_FINALLY_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].try_finally_expr.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].try_finally_expr.op1,"op1");
	break;
      case gGOTO_SUBROUTINE_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].goto_subroutine_expr.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].goto_subroutine_expr.op1,"op1");
	break;
      case gPOPDHC_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_common(g.nodeBuffer[nodenum].common);
	end_attributes ();
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	break;
      case gPOPDCC_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_common(g.nodeBuffer[nodenum].common);
	end_attributes ();
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	break;
      case gLABEL_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_common(g.nodeBuffer[nodenum].common);
	end_attributes ();
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	break;
      case gGOTO_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_common(g.nodeBuffer[nodenum].common);
	end_attributes ();
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	break;
      case gRETURN_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_common(g.nodeBuffer[nodenum].common);
	end_attributes ();
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	break;
      case gEXIT_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_common(g.nodeBuffer[nodenum].common);
	end_attributes ();
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].exit_expr.cond,"cond");
	break;
      case gLOOP_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_common(g.nodeBuffer[nodenum].common);
	end_attributes ();
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].loop_expr.body,"body");
	break;
      case gLABELED_BLOCK_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	break;
      case gEXPR_WITH_FILE_LOCATION:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].expr_with_file_location.expr2,"expr");
	break;
      case gSWITCH_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].switch_expr.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].switch_expr.op1,"op1");
	break;
      case gSIZEOF_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].sizeof_expr.op0,"op0");
	break;
      case gARROW_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].arrow_expr.op0,"op0");
	break;
      case gALIGNOF_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].alignof_expr.op0,"op0");
	break;
      case gNEW_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].new_expr.op0,"op0");
	break;
      case gVEC_NEW_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].vec_new_expr.op0,"op0");
	break;
      case gDELETE_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].delete_expr.op0,"op0");
	break;
      case gVEC_DELETE_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].vec_delete_expr.op0,"op0");
	break;
      case gTYPE_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].type_expr.op0,"op0");
	break;
      case gAGGR_INIT_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	write_number_attr("ctor",g.nodeBuffer[nodenum].aggr_init_expr.ctor);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].aggr_init_expr.fn,"fn");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].aggr_init_expr.args,"args");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].aggr_init_expr.decl,"decl");
	break;
      case gEMPTY_CLASS_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	break;
      case gTEMPLATE_ID_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].template_id_expr.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].template_id_expr.op1,"op1");
	break;
      case gLOOKUP_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].lookup_expr.op0,"op0");
	break;
      case gFUNCTION_NAME:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	break;
      case gSUBOBJECT:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	write_number_attr("line",g.nodeBuffer[nodenum].subobject.line);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].subobject.clnp,"clnp");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].subobject.next,"_next");
	break;
      case gCTOR_STMT:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	write_number_attr("line",g.nodeBuffer[nodenum].ctor_stmt.line);
	write_number_attr("beg_end",g.nodeBuffer[nodenum].ctor_stmt.beg_end);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].ctor_stmt.next,"next");
	break;
      case gCTOR_INITIALIZER:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].ctor_initializer.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].ctor_initializer.op1,"op1");
	break;
      case gRETURN_INIT:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].return_init.op0,"op0");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].return_init.op1,"op1");
	break;
      case gTAG_DEFN:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	break;
      case gIDENTITY_CONV:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].identity_conv.op0,"op0");
	break;
      case gLVALUE_CONV:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].lvalue_conv.op0,"op0");
	break;
      case gQUAL_CONV:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].qual_conv.op0,"op0");
	break;
      case gSTD_CONV:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].std_conv.op0,"op0");
	break;
      case gPTR_CONV:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].ptr_conv.op0,"op0");
	break;
      case gPMEM_CONV:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].pmem_conv.op0,"op0");
	break;
      case gBASE_CONV:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].base_conv.op0,"op0");
	break;
      case gREF_BIND:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].ref_bind.op0,"op0");
	break;
      case gUSER_CONV:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].user_conv.op0,"op0");
	break;
      case gAMBIG_CONV:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].ambig_conv.op0,"op0");
	break;
      case gRVALUE_CONV:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_expr(g.nodeBuffer[nodenum].expr);
	end_attributes ();
	/* edges */
	edge_expr(g.nodeBuffer[nodenum].expr,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].rvalue_conv.op0,"op0");
	break;
      case gOP_IDENTIFIER:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_common(g.nodeBuffer[nodenum].common);
	end_attributes ();
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	break;
      case gIDENTIFIER_NODE:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_common(g.nodeBuffer[nodenum].common);
	write_number_attr("lngt",g.nodeBuffer[nodenum].identifier_node.lngt);
	write_string_attr("strg",&(g.stringBuffer[g.nodeBuffer[nodenum].identifier_node.strg]));
	write_number_attr("op_type_un",g.nodeBuffer[nodenum].identifier_node.op_type_un);
	write_number_attr("tynm",g.nodeBuffer[nodenum].identifier_node.tynm);
	end_attributes ();
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	break;
      case gINTEGER_CST:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_cst(g.nodeBuffer[nodenum].gcc_cst);
	write_number_attr("high", g.nodeBuffer[nodenum].integer_cst.high);
	write_number_attr("low", g.nodeBuffer[nodenum].integer_cst.low);
	end_attributes ();
	/* edges */
	edge_cst(g.nodeBuffer[nodenum].gcc_cst,nodenum);
	break;
      case gREAL_CST:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_cst(g.nodeBuffer[nodenum].gcc_cst);
	write_double_attr("val",&(g.stringBuffer[g.nodeBuffer[nodenum].real_cst.value]));
	end_attributes ();
	/* edges */
	edge_cst(g.nodeBuffer[nodenum].gcc_cst,nodenum);
	break;
      case gPTRMEM_CST:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_cst(g.nodeBuffer[nodenum].gcc_cst);
	end_attributes ();
	/* edges */
	edge_cst(g.nodeBuffer[nodenum].gcc_cst,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].ptrmem_cst.clas,"clas");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].ptrmem_cst.mbr,"mbr");
	break;
      case gCOMPLEX_CST:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_cst(g.nodeBuffer[nodenum].gcc_cst);
	end_attributes ();
	/* edges */
	edge_cst(g.nodeBuffer[nodenum].gcc_cst,nodenum);
	break;
      case gSTRING_CST:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_cst(g.nodeBuffer[nodenum].gcc_cst);
	write_number_attr("lngt", g.nodeBuffer[nodenum].string_cst.lngt);
	write_string_attr("strg",&(g.stringBuffer[g.nodeBuffer[nodenum].string_cst.strg]));
	end_attributes ();
	/* edges */
	edge_cst(g.nodeBuffer[nodenum].gcc_cst,nodenum);
	break;
      case gTREE_LIST:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_common(g.nodeBuffer[nodenum].common);
	end_attributes ();
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].tree_list.purpose,"purpose");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].tree_list.value,"value");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].tree_list.chan,"chan");
	break;
      case gTREE_VEC:
	if (g.nodeBuffer[nodenum].tree_vec.binfo){
	  write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	  begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	  attr_common(g.nodeBuffer[nodenum].common);
	  write_boolean_attr("pub",g.nodeBuffer[nodenum].tree_vec.pub);
	  write_boolean_attr("prot",g.nodeBuffer[nodenum].tree_vec.prot);
	  write_boolean_attr("priv",g.nodeBuffer[nodenum].tree_vec.priv);
	  write_boolean_attr("virt",g.nodeBuffer[nodenum].tree_vec.virt);
	  end_attributes ();
	  /* edges */
	  edge_common(g.nodeBuffer[nodenum].common,nodenum);
	  write_edge_tuple(nodenum,g.nodeBuffer[nodenum].tree_vec.inherit,"inherit");
	  write_edge_tuple(nodenum,g.nodeBuffer[nodenum].tree_vec.type,"type");
	  write_edge_tuple(nodenum,g.nodeBuffer[nodenum].tree_vec.base,"base");
	} else {
	  write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	  begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	  attr_common(g.nodeBuffer[nodenum].common);
	  end_attributes ();
	  edge_common(g.nodeBuffer[nodenum].common,nodenum);
	}
	break;
      case gERROR_MARK:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_common(g.nodeBuffer[nodenum].common);
	end_attributes ();
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	break;
      case gDEFAULT_ARG:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_common(g.nodeBuffer[nodenum].common);
	end_attributes ();
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	break;
      case gCPLUS_BINDING:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_common(g.nodeBuffer[nodenum].common);
	end_attributes ();
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	break;
      case gOVERLOAD:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_common(g.nodeBuffer[nodenum].common);
	end_attributes ();
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].overload.crnt,"crnt");
	write_edge_tuple(nodenum,g.nodeBuffer[nodenum].overload.chan,"chan");
	break;
      case gWRAPPER:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	attr_common(g.nodeBuffer[nodenum].common);
	end_attributes ();
	/* edges */
	edge_common(g.nodeBuffer[nodenum].common,nodenum);
	break;
      case NODE_NONE:
      case NODE_DELTED:
	break;

	/* datrix types */
	/* datrix edges are handled differently than gcc edges */

      case dC_ASG_ND:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_asg_nd(g, g.nodeBuffer[nodenum].dc_asg_nd);
	end_attributes ();
	break;
      case dC_IDENTIFIER:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_identifier(g, g.nodeBuffer[nodenum].dc_identifier);
	end_attributes ();
	break;
      case dC_TYPE:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_type(g, g.nodeBuffer[nodenum].dc_type);
	end_attributes ();
	break;
      case dC_BUILT_IN_TYPE:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_type(g, g.nodeBuffer[nodenum].dc_type);
	end_attributes ();
	break;
      case dC_AGGR_TYPE:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_type(g, g.nodeBuffer[nodenum].dc_type);
	switch(g.nodeBuffer[nodenum].dc_aggr_type.aggr_type){
	  case aggr_class:
	    write_string_attr("aggr_type","class");
	    break;
	  case aggr_record:
	    write_string_attr("aggr_type","record");
	    break;
	  case aggr_struct:
	    write_string_attr("aggr_type","struct");
	    break;
	  case aggr_union:
	    write_string_attr("aggr_type","union");
	    break;
	  case aggr_area:
	    write_string_attr("aggr_type","area");
	    break;
	  case aggr_interface:
	    write_string_attr("aggr_type","interface");
	    break;
	  default:
	    fprintf(stderr,"Unknown datrix aggregate type %d\n",
		g.nodeBuffer[nodenum].dc_aggr_type.aggr_type);
	    exit(1);
	}
	end_attributes ();
	break;
      case dC_ENUM_TYPE:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_type(g, g.nodeBuffer[nodenum].dc_type);
	end_attributes ();
	break;
      case dC_TEMPL_PARM_TYPE:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_type(g, g.nodeBuffer[nodenum].dc_type);
	end_attributes ();
	break;
      case dC_GENERIC_TYPE:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_type(g, g.nodeBuffer[nodenum].dc_type);
	end_attributes ();
	break;
      case dC_GENERATED_TYPE:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_type(g, g.nodeBuffer[nodenum].dc_type);
	end_attributes ();
	break;
      case dC_ALIAS_TYPE:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_type(g, g.nodeBuffer[nodenum].dc_type);
	end_attributes ();
	break;
      case dC_FORWARD_TYPE:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_type(g, g.nodeBuffer[nodenum].dc_type);
	end_attributes ();
	break;
      case dC_ARRAY_TYPE:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_type(g, g.nodeBuffer[nodenum].dc_type);
	end_attributes ();
	break;
     case dC_PTR_TYPE:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_type(g, g.nodeBuffer[nodenum].dc_type);
	end_attributes ();
	break;
      case dC_REF_TYPE:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_type(g, g.nodeBuffer[nodenum].dc_type);
	end_attributes ();
	break;
      case dC_FCT_TYPE:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_type(g, g.nodeBuffer[nodenum].dc_type);
	end_attributes ();
	break;
      case dC_SCOPE:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_scope(g, g.nodeBuffer[nodenum].dc_scope);
	end_attributes ();
	break;
      case dC_SCOPE_GLB:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_scope(g, g.nodeBuffer[nodenum].dc_scope);
	end_attributes ();
	break;
      case dC_SCOPE_COMPIL:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_scope(g, g.nodeBuffer[nodenum].dc_scope);
	end_attributes ();
	break;
      case dC_SCOPE_FILE:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_scope(g, g.nodeBuffer[nodenum].dc_scope);
	end_attributes ();
	break;
      case dC_PACKAGE:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_scope(g, g.nodeBuffer[nodenum].dc_scope);
	end_attributes ();
	break;
      case dC_NAME_SPACE:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_scope(g, g.nodeBuffer[nodenum].dc_scope);
	end_attributes ();
	break;
      case dC_BLOCK:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_scope(g, g.nodeBuffer[nodenum].dc_scope);
	end_attributes ();
	break;
      case dC_STATIC_BLOCK:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_scope(g, g.nodeBuffer[nodenum].dc_scope);
	end_attributes ();
	break;
      case dC_TRY_BLOCK:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_scope(g, g.nodeBuffer[nodenum].dc_scope);
	end_attributes ();
	break;
      case dC_CATCH_BLOCK:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_scope(g, g.nodeBuffer[nodenum].dc_scope);
	end_attributes ();
	break;
      case dC_FINALLY_BLOCK:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_scope(g, g.nodeBuffer[nodenum].dc_scope);
	end_attributes ();
	break;
      case dC_SYNCHRONIZED_BLOCK:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_scope(g, g.nodeBuffer[nodenum].dc_scope);
	end_attributes ();
	break;
      case dC_OBJECT:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_object(g, g.nodeBuffer[nodenum].dc_object);
	end_attributes ();
	break;
      case dC_FORMAL_FCT_PARAM:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_object(g, g.nodeBuffer[nodenum].dc_object);
	end_attributes ();
	break;
      case dC_FORMAL_CATCH_PARAM:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_object(g, g.nodeBuffer[nodenum].dc_object);
	end_attributes ();
	break;
      case dC_FUNCTION:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_identifier(g, g.nodeBuffer[nodenum].dc_identifier);
	write_boolean_attr("static",g.nodeBuffer[nodenum].dc_function.static_flag);
	write_boolean_attr("extern",g.nodeBuffer[nodenum].dc_function.extern_flag);
	write_boolean_attr("const",g.nodeBuffer[nodenum].dc_function.const_flag);
	switch(g.nodeBuffer[nodenum].dc_function.virtual_code){
	  case 0: 
	    write_string_attr("virtual","non-virtual");
	    break;
	  case 1: 
	    write_string_attr("virtual","virtual");
	    break;
	  case 2: 
	    write_string_attr("virtual","pure-virtual");
	    break;
	  case 3:
	    fprintf(stderr,"unknown virtual flag in function\n");
	    exit(1);
	}
	end_attributes ();
	break;
      case dC_LABEL:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_label(g, g.nodeBuffer[nodenum].dc_label);
	end_attributes ();
	break;
      case dC_CASE_LABEL:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_label(g, g.nodeBuffer[nodenum].dc_label);
	end_attributes ();
	break;
      case dC_DEFAULT_LABEL:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_label(g, g.nodeBuffer[nodenum].dc_label);
	end_attributes ();
	break;
      case dC_ENUMERATOR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_identifier(g, g.nodeBuffer[nodenum].dc_identifier);
	end_attributes ();
	break;
      case dC_EXPRESSION:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_expression(g, g.nodeBuffer[nodenum].dc_expression);
	end_attributes ();
	break;
      case dC_EMPTY_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_expression(g, g.nodeBuffer[nodenum].dc_expression);
	end_attributes ();
	break;
      case dC_LITERAL:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_expression(g, g.nodeBuffer[nodenum].dc_expression);
	write_string_attr("value",&(g.stringBuffer[g.nodeBuffer[nodenum].dc_literal.value]));
	end_attributes ();
	break;
      case dC_SYNTAX_ERROR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_expression(g, g.nodeBuffer[nodenum].dc_expression);
	end_attributes ();
	break;
      case dC_NAME_REF:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_expression(g, g.nodeBuffer[nodenum].dc_expression);
	write_string_attr("name",&(g.stringBuffer[g.nodeBuffer[nodenum].dc_name_ref.name]));
	end_attributes ();
	break;
      case dC_OPERATOR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_operator(g, g.nodeBuffer[nodenum].dc_operator);
	end_attributes ();
	break;
      case dC_UNARY_OPERATOR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_operator(g, g.nodeBuffer[nodenum].dc_operator);
	end_attributes ();
	break;
      case dC_BINARY_OPERATOR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_operator(g, g.nodeBuffer[nodenum].dc_operator);
	end_attributes ();
	break;
      case dC_QUESTION_OPERATOR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_operator(g, g.nodeBuffer[nodenum].dc_operator);
	end_attributes ();
	break;
      case dC_INIT_LIST:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_expression(g, g.nodeBuffer[nodenum].dc_expression);
	end_attributes ();
	break;
      case dC_ACTUAL_FCT_PARAM_LIST:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_expression(g, g.nodeBuffer[nodenum].dc_expression);
	end_attributes ();
	break;
      case dC_ACTUAL_TEMPL_PARAM_LIST:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_expression(g, g.nodeBuffer[nodenum].dc_expression);
	end_attributes ();
	break;
      case dC_CREATION_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_expression(g, g.nodeBuffer[nodenum].dc_expression);
	end_attributes ();
	break;
      case dC_DESTROY_EXPR:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_expression(g, g.nodeBuffer[nodenum].dc_expression);
	end_attributes ();
	break;
      case dC_CNTRL_STMT:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_ctrl_stmt(g, g.nodeBuffer[nodenum].dc_ctrl_stmt);
	end_attributes ();
	break;
      case dC_SELECTION:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_ctrl_stmt(g, g.nodeBuffer[nodenum].dc_ctrl_stmt);
	end_attributes ();
	break;
      case dC_IF:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_ctrl_stmt(g, g.nodeBuffer[nodenum].dc_ctrl_stmt);
	end_attributes ();
	break;
      case dC_SWITCH:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_ctrl_stmt(g, g.nodeBuffer[nodenum].dc_ctrl_stmt);
	end_attributes ();
	break;
      case dC_JUMP:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_jump(g, g.nodeBuffer[nodenum].dc_jump);
	end_attributes ();
	break;
      case dC_GOTO:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_jump(g, g.nodeBuffer[nodenum].dc_jump);
	end_attributes ();
	break;
      case dC_RETURN:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_jump(g, g.nodeBuffer[nodenum].dc_jump);
	end_attributes ();
	break;
      case dC_BREAK:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_jump(g, g.nodeBuffer[nodenum].dc_jump);
	end_attributes ();
	break;
      case dC_CONTINUE:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_jump(g, g.nodeBuffer[nodenum].dc_jump);
	end_attributes ();
	break;
      case dC_LOOP:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_loop(g, g.nodeBuffer[nodenum].dc_loop);
	end_attributes ();
	break;
      case dC_DO_WHILE_LOOP:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_loop(g, g.nodeBuffer[nodenum].dc_loop);
	end_attributes ();
	break;
      case dC_WHILE_DO_LOOP:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_loop(g, g.nodeBuffer[nodenum].dc_loop);
	end_attributes ();
	break;
      case dC_FOR_LOOP:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_loop(g, g.nodeBuffer[nodenum].dc_loop);
	end_attributes ();
	break;
      case dC_SQL_STMT:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_asg_nd(g, g.nodeBuffer[nodenum].dc_asg_nd);
	end_attributes ();
	break;
      case dC_COMMENT:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_comment(g, g.nodeBuffer[nodenum].dc_comment);
	end_attributes ();
	break;
      case dC_JAVA_DOC_COMMENT:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_comment(g, g.nodeBuffer[nodenum].dc_comment);
	end_attributes ();
	break;
      case dC_THROW_SPEC:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_asg_nd(g, g.nodeBuffer[nodenum].dc_asg_nd);
	end_attributes ();
	break;
      case dC_ASM_BLOCK:
	write_inst_tuple (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],nodenum);
	begin_inst_attributes(gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type], nodenum);
	datrix_asg_nd(g, g.nodeBuffer[nodenum].dc_asg_nd);
	end_attributes ();
	break;
      case dC_ARC:
	begin_edge_attributes (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],
	    g.nodeBuffer[nodenum].dc_arc.src,
	    g.nodeBuffer[nodenum].dc_arc.dest);
	datrix_arc(g.nodeBuffer[nodenum].dc_arc);
	end_attributes ();
	break;
      case dC_ARC_REFERENCE:
	begin_edge_attributes (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],
	    g.nodeBuffer[nodenum].dc_arc.src,
	    g.nodeBuffer[nodenum].dc_arc.dest);
	datrix_arc_reference(g.nodeBuffer[nodenum].dc_arc_reference);
	end_attributes ();
	break;
      case dC_INSTANCE:
	begin_edge_attributes (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],
	    g.nodeBuffer[nodenum].dc_arc.src,
	    g.nodeBuffer[nodenum].dc_arc.dest);
	datrix_arc_reference(g.nodeBuffer[nodenum].dc_arc_reference);
	end_attributes ();
	break;
      case dC_INHERITANCE:
	begin_edge_attributes (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],
	    g.nodeBuffer[nodenum].dc_arc.src,
	    g.nodeBuffer[nodenum].dc_arc.dest);
	datrix_arc_reference(g.nodeBuffer[nodenum].dc_arc_reference);
	write_boolean_attr("virt",g.nodeBuffer[nodenum].dc_inheritance.virt);
	switch (g.nodeBuffer[nodenum].dc_inheritance.visb){
	  case visb_unk:
	    write_string_attr("visb","unk");
	    break;
	  case visb_pri:
	    write_string_attr("visb","pri");
	    break;
	  case visb_pro:
	    write_string_attr("visb","pro");
	    break;
	  case visb_pub:
	    write_string_attr("visb","pub");
	    break;
	  default:
	    fprintf(stderr,"Illegal visibility\n");
	    exit(1);
	}
	end_attributes ();
	break;
      case dC_IMPLEMENTS:
	begin_edge_attributes (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],
	    g.nodeBuffer[nodenum].dc_arc.src,
	    g.nodeBuffer[nodenum].dc_arc.dest);
	datrix_arc_reference(g.nodeBuffer[nodenum].dc_arc_reference);
	end_attributes ();
	break;
      case dC_ARC_FRIEND:
	begin_edge_attributes (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],
	    g.nodeBuffer[nodenum].dc_arc.src,
	    g.nodeBuffer[nodenum].dc_arc.dest);
	datrix_arc_reference(g.nodeBuffer[nodenum].dc_arc_reference);
	end_attributes ();
	break;
      case dC_ARC_DEF_LOC:
	begin_edge_attributes (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],
	    g.nodeBuffer[nodenum].dc_arc.src,
	    g.nodeBuffer[nodenum].dc_arc.dest);
	datrix_arc_reference(g.nodeBuffer[nodenum].dc_arc_reference);
	end_attributes ();
	break;
      case dC_REFERS_TO:
	begin_edge_attributes (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],
	    g.nodeBuffer[nodenum].dc_arc.src,
	    g.nodeBuffer[nodenum].dc_arc.dest);
	datrix_arc_reference(g.nodeBuffer[nodenum].dc_arc_reference);
	end_attributes ();
	break;
      case dC_ARC_SYNTAX:
	begin_edge_attributes (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],
	    g.nodeBuffer[nodenum].dc_arc.src,
	    g.nodeBuffer[nodenum].dc_arc.dest);
	datrix_arc_syntax(g.nodeBuffer[nodenum].dc_arc_syntax);
	end_attributes ();
	break;
      case dC_ARC_SON:
	begin_edge_attributes (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],
	    g.nodeBuffer[nodenum].dc_arc.src,
	    g.nodeBuffer[nodenum].dc_arc.dest);
	datrix_arc_syntax(g.nodeBuffer[nodenum].dc_arc_syntax);
	end_attributes ();
	break;
      case dC_ARC_TRUE:
	begin_edge_attributes (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],
	    g.nodeBuffer[nodenum].dc_arc.src,
	    g.nodeBuffer[nodenum].dc_arc.dest);
	datrix_arc_syntax(g.nodeBuffer[nodenum].dc_arc_syntax);
	end_attributes ();
	break;
      case dC_ARC_FALSE:
	begin_edge_attributes (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],
	    g.nodeBuffer[nodenum].dc_arc.src,
	    g.nodeBuffer[nodenum].dc_arc.dest);
	datrix_arc_syntax(g.nodeBuffer[nodenum].dc_arc_syntax);
	end_attributes ();
	break;
      case dC_ARC_OPD:
	begin_edge_attributes (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],
	    g.nodeBuffer[nodenum].dc_arc.src,
	    g.nodeBuffer[nodenum].dc_arc.dest);
	datrix_arc_syntax(g.nodeBuffer[nodenum].dc_arc_syntax);
	end_attributes ();
	break;
      case dC_IMPORT:
	begin_edge_attributes (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],
	    g.nodeBuffer[nodenum].dc_arc.src,
	    g.nodeBuffer[nodenum].dc_arc.dest);
	datrix_arc_syntax(g.nodeBuffer[nodenum].dc_arc_syntax);
	end_attributes ();
	break;
      case dC_ARC_COND:
	begin_edge_attributes (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],
	    g.nodeBuffer[nodenum].dc_arc.src,
	    g.nodeBuffer[nodenum].dc_arc.dest);
	datrix_arc_syntax(g.nodeBuffer[nodenum].dc_arc_syntax);
	end_attributes ();
	break;
      case dC_ARC_FOR_INIT:
	begin_edge_attributes (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],
	    g.nodeBuffer[nodenum].dc_arc.src,
	    g.nodeBuffer[nodenum].dc_arc.dest);
	datrix_arc_syntax(g.nodeBuffer[nodenum].dc_arc_syntax);
	end_attributes ();
	break;
      case dC_ARC_FOR_INCR:
	begin_edge_attributes (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],
	    g.nodeBuffer[nodenum].dc_arc.src,
	    g.nodeBuffer[nodenum].dc_arc.dest);
	datrix_arc_syntax(g.nodeBuffer[nodenum].dc_arc_syntax);
	end_attributes ();
	break;
      case dC_ARC_INIT_VAL:
	begin_edge_attributes (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],
	    g.nodeBuffer[nodenum].dc_arc.src,
	    g.nodeBuffer[nodenum].dc_arc.dest);
	datrix_arc_syntax(g.nodeBuffer[nodenum].dc_arc_syntax);
	end_attributes ();
	break;
      case dC_ARC_BIT_DIM:
	begin_edge_attributes (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],
	    g.nodeBuffer[nodenum].dc_arc.src,
	    g.nodeBuffer[nodenum].dc_arc.dest);
	datrix_arc_syntax(g.nodeBuffer[nodenum].dc_arc_syntax);
	end_attributes ();
	break;
      case dC_ARC_ARRAY_DIM:
	begin_edge_attributes (gd_node_type_ascii [g. nodeBuffer [nodenum]. common. type],
	    g.nodeBuffer[nodenum].dc_arc.src,
	    g.nodeBuffer[nodenum].dc_arc.dest);
	datrix_arc_syntax(g.nodeBuffer[nodenum].dc_arc_syntax);
	end_attributes ();
	break;

      default: {
	int i;
	unsigned char *x;
	fprintf(stderr,"Node number %d\n",nodenum);
	fprintf(stderr,"unknown node type: %d\n" ,g.nodeBuffer[nodenum].common.type);
	fprintf(stderr,"node data:\n");
	  x = (unsigned char *)(&(g.nodeBuffer[nodenum]));
	i = 0;
	while (i < sizeof(gd_graph_node)){
	  fprintf(stderr," %2x",*x);
	  x++;
	  i++;
	  if (i % 16 == 0) {
	    fprintf(stderr,"\n");
	  }
	}
	fprintf(stderr,"\n");
      }
    }
  }
}

void datrix_asg_nd(gd_graph g, t_dc_asg_nd nd){
  /* for now nothing*/
  //write_string_attr("File",&(g.stringBuffer[nd.file]));
  //write_number_attr("Line",d.line);
}

void datrix_identifier(gd_graph g, t_dc_identifier ident){
  datrix_asg_nd(g, ident.dc_asg_nd);
  if (ident.name != -1){
    write_string_attr("name",&(g.stringBuffer[ident.name]));
  }
  switch (ident.visb){
    case visb_unk:
      write_string_attr("visb","unk");
      break;
    case visb_pri:
      write_string_attr("visb","pri");
      break;
    case visb_pro:
      write_string_attr("visb","pro");
      break;
    case visb_pub:
      write_string_attr("visb","pub");
      break;
    default:
      fprintf(stderr,"Illegal visibility\n");
      exit(1);
  }
}
void datrix_loop(gd_graph g, t_dc_loop loop){
  datrix_ctrl_stmt(g, loop.ctrl_stmt);
}

void datrix_ctrl_stmt(gd_graph g, t_dc_ctrl_stmt cntrl){
  datrix_asg_nd(g, cntrl.dc_asg_nd);
}

void datrix_comment(gd_graph g, t_dc_comment comment){
  datrix_asg_nd(g, comment.dc_asg_nd);
}

void datrix_jump(gd_graph g, t_dc_jump jump) {
  datrix_ctrl_stmt(g,jump.ctrl_stmt);
}

void datrix_expression(gd_graph g, t_dc_expression expr){
  datrix_asg_nd(g, expr.dc_asg_nd);
}
void datrix_operator(gd_graph g, t_dc_operator operator){
  datrix_expression(g, operator.expression);
  if (operator.op < 0 || operator.op > dc_range){
    fprintf(stderr,"unknown operator %d\n",operator.op);
  }
  write_string_attr("op",dc_op_string[operator.op]);
}

void datrix_type(gd_graph g, t_dc_type type){
  datrix_identifier(g, type.dc_identifier);
}

void datrix_object(gd_graph g, t_dc_object object){
  datrix_identifier(g, object.dc_identifier);
  write_boolean_attr("static",object.static_flag);
  write_boolean_attr("extern",object.extern_flag);
  write_boolean_attr("const",object.const_flag);
}
void datrix_label(gd_graph g, t_dc_label label){
  datrix_identifier(g, label.dc_identifier);
}

void datrix_scope(gd_graph g, t_dc_scope scope){
  datrix_identifier(g, scope.dc_identifier);
}

void datrix_arc(t_dc_arc  dc_arc){
    if (dc_arc.ordinal != 0){
      write_number_attr("order", dc_arc.ordinal);
    }
}
void datrix_arc_reference(t_dc_arc_reference  dc_arc_reference){
  datrix_arc(dc_arc_reference.arc);
}

void datrix_arc_syntax(t_dc_arc_syntax  dc_arc_syntax){
  datrix_arc(dc_arc_syntax.arc);
}

void attr_decl(gd_graph g, t_gcc_decl decl) {
  if (writing != attributes) return;
  attr_common(decl.common);
  if (decl.srcl != -1) {
    if (decl.srcf >= g.curStringPos){
      fprintf(stderr,"string index is out of bounds\n");
      exit(1);
    } else {
//      write_string_attr("File", &(g.stringBuffer[decl.srcf]));
//      write_number_attr("Line", decl.srcl);
    }
  }
  switch (decl.lang) {
    case src_lang_c:
      write_string_attr("src_lang", "c");
      break;
    case src_lang_cp:
      write_string_attr("src_lang", "c++");
      break;
    case src_lang_java:
      write_string_attr("src_lang", "java");
      break;
    default:
      break;
  }
  write_boolean_attr("artificial",decl.artificial);
}

void edge_decl(gd_graph g, t_gcc_decl decl, int pos) {
  edge_common(decl.common,pos);
  write_edge_tuple(pos,decl.name,"name");
  write_edge_tuple(pos,decl.mngl,"mngl");
  write_edge_tuple(pos,decl.type,"type");
  write_edge_tuple(pos,decl.chan,"chan");
  write_edge_tuple(pos,decl.scpe,"scpe");
}

void edge_common(gd_graph_common comm, int pos) {
  /* no edges from common block at this time */
}

void attr_common(gd_graph_common comm){
  write_boolean_attr("unsigned_flag",comm.unsigned_flag);
  write_boolean_attr("constant_flag",comm.constant_flag);
  write_boolean_attr("static_flag",comm.static_flag);
  write_boolean_attr("public_flag",comm.public_flag);
  write_boolean_attr("private_flag",comm.private_flag);
  write_boolean_attr("protected_flag",comm.protected_flag);
}

void attr_type(t_gcc_type type) {
  attr_common(type.common);
  write_number_attr("algn",type.algn);
  write_boolean_attr("qual_const",type.qual_const);
  write_boolean_attr("qual_volatile",type.qual_volatile);
  write_boolean_attr("qual_restrict",type.qual_restrict);
}

void edge_type(t_gcc_type type,int pos) {
  edge_common(type.common,pos);
  write_edge_tuple(pos,type.name,"name");
  write_edge_tuple(pos,type.size,"size");
  write_edge_tuple(pos,type.unql,"unql");
}

void attr_cst(t_gcc_const cst) {
  attr_common(cst.common);
}

void edge_cst(t_gcc_const cst,int pos) {
  edge_common(cst.common,pos);
  write_edge_tuple(pos,cst.type,"type");
}

void attr_expr(t_gcc_expr expr) {
  attr_common(expr.common);
}

void edge_expr(t_gcc_expr expr, int pos) {
  edge_common(expr.common,pos);
  write_edge_tuple(pos,expr.type,"type");
}

void write_boolean_attr(const char * attr, int val)
{
    if (writing == attributes)
    {
	if (attrib_count++ == 0)
	    printf ("%s", attrib_header);
	else
	    printf (" ");
	printf(" %s = %s",attr,(val)?"true":"false");
    }
}

void write_number_attr(const char * attr, int val)
{
    if (writing == attributes)
    {
	if (attrib_count++ == 0)
	    printf ("%s", attrib_header);
	else
	    printf (" ");
	printf(" %s = %d",attr,val);
    }
}

void write_double_attr(const char * attr, char * val){
    if (writing == attributes)
    {
	if (attrib_count++ == 0)
	    printf ("%s", attrib_header);
	else
	    printf (" ");
	printf(" %s = %s",attr,val);
    }
}

void write_string_attr(const char * attr, char * val)
{
    char * rover;
    int all_token;
    if (writing == attributes)
    {
	if (attrib_count++ == 0)
	    printf ("%s", attrib_header);
	else
	    printf (" ");
	// Is this a wholly-graphic string with no duoblequotes?
	all_token = 1;
	rover = val;
	while (all_token && *rover != '\0')
	{
	    all_token = (*rover != '\"') && isgraph (*rover);
	    rover++;
	}
	
	printf (" %s = ", attr);
	if (all_token)
	    printf ("%s", val);
	else
	{
	    printf ("\"");
	    for(rover = val; *rover != '\0'; rover++)
		/* print according to xml rules */
		/* TODO */
		if (isprint(*rover))
		    switch (*rover) {
		    case '"':
			printf ("\\\"");
			break;
		    default:
			printf("%c",*rover);
		    }
		else
		    printf("\%03c",*rover);
	    printf("\"");
	}
    }
}

void write_inst_tuple(const char * typename, node_ptr nodenum)
{
    if (writing == tuples)
	printf("$INSTANCE @%d %s\n",nodenum,typename);
}

void write_edge_tuple(int from, int to, const char * type)
{
    if (writing == tuples)
	if (to != -1)
	    printf("%s @%d @%d\n",type,from,to);
}

void begin_inst_attributes(const char * typename, node_ptr nodenum) 
{
    if (writing == attributes)
    {
	attrib_count = 0;
	sprintf(attrib_header, "@%d {",nodenum);
    }
}

void begin_edge_attributes(const char * edgename, node_ptr src, node_ptr dest)
{
    if (writing == attributes)
    {
	attrib_count = 0;
	sprintf(attrib_header,"(%s @%d @%d) {", edgename, src, dest);
    }
    else
	write_edge_tuple (src, dest, edgename);
}

void end_attributes ()
{
    if (writing == attributes)
	if (attrib_count > 0)
	    printf (" }\n");
}

