/*+
 * Name:	alias.c
 *
 * Purpose:	transforms alias types to dcc alias nodes
 *              -- alias types are those with an unql pointer, and
 *                 are not a built in type - link using an instance edge
 *              Must be run after gd_builtin
 *              Must be run before gd_derived and funct_type since they
 *                throw out unql pointers.
 *
 * Author:	Thomas R. Dean
 *
 * Date:	March 13, 2001
 *
-*/

#include <stdio.h>
#include <stdlib.h>

#include "gd_graph.h"
#include "gd_utils.h"

char * progname;

void usage(){
  fprintf(stderr,"usage: %s infile outfile\n",progname);
  exit(1);
}

int main(int argc, char * argv[]){
  gd_graph g;
  char * filename, *outfilename;
  int nodenum,i, count;
  progname = argv[0];

  if (argc != 3) {
    usage();
  }

  filename = argv[1];
  outfilename = argv[2];

  g = gd_graph_read_binary_file(filename,1.0,1.0,1.0);
  if (g.curStringPos == 0) {
    fprintf(stderr,"could not get graph\n");
    exit(1);
  }

  for(i = 0; i < g.curNodePos;i++){
    if ((g.nodeBuffer[i].common.type == gARRAY_TYPE ||
	 g.nodeBuffer[i].common.type == gFUNCTION_TYPE ||
	 g.nodeBuffer[i].common.type == gREFERENCE_TYPE ||
	 g.nodeBuffer[i].common.type == gBOOLEAN_TYPE ||
	 g.nodeBuffer[i].common.type == gINTEGER_TYPE ||
	 g.nodeBuffer[i].common.type == gPOINTER_TYPE ||
	 g.nodeBuffer[i].common.type == gREAL_TYPE ||
	 g.nodeBuffer[i].common.type == gUNION_TYPE ||
	 g.nodeBuffer[i].common.type == gENUMERAL_TYPE ||
	 g.nodeBuffer[i].common.type == gMETHOD_TYPE ||
	 g.nodeBuffer[i].common.type == gRECORD_TYPE ||
	 g.nodeBuffer[i].common.type == gVOID_TYPE) &&
        (g.nodeBuffer[i].type.unql != -1) &&
	(g.nodeBuffer[i].type.unql != i)){

      /* fields to save */

      /* from type decl */
      node_ptr decl;   /* the type_decl associated with the record type */
      node_ptr name;   /*  |-- the gcc identifier node if it exists     */
      str_ptr srcf;    /*  |-- source file                              */
      int srcl;	       /*  |-- source line                              */
      str_ptr namestr; /*  |-- str_ptr for name                         */

      /* from record_type */
      node_ptr unql; /* The other type */

      /* other locals */
      node_ptr new_node;

      /* does the decl exist */
      decl = g.nodeBuffer[i].type.name;
      if (decl != -1){
	if (g.nodeBuffer[decl].common.type == gTYPE_DECL) {
	  srcf = g.nodeBuffer[decl].decl.srcf;
	  srcl = g.nodeBuffer[decl].decl.srcl;
	  name = g.nodeBuffer[decl].decl.name;
	} else if (g.nodeBuffer[decl].common.type == gIDENTIFIER_NODE) {
	  srcf = -1;
	  srcl = -1;
	  name = decl;
	} else {
	  fprintf(stderr,"type with name not decl or identifier??");
	  exit(1);
	}
	if (name != -1){
	  namestr = g.nodeBuffer[name].identifier_node.strg;
	} else {
	  namestr = -1;
	}
      } else {
	srcf = -1;
	srcl = -1;
	name = -1;
	namestr = -1;
      }

      /* record_type fields to save */
      unql  = g.nodeBuffer[i].type.unql;

      /* switch node type */
      g.nodeBuffer[i].common.type = dC_ALIAS_TYPE;

      g.nodeBuffer[i].dc_asg_nd.file = srcf;
      g.nodeBuffer[i].dc_asg_nd.line = srcl;

      /* determin datrix visibility from common flags */
      /* gd_common doesn't change, no need to have saved flags */
      g.nodeBuffer[i].dc_identifier.visb = visb_unk;
      if (decl != -1){
	if (g.nodeBuffer[decl].common.private_flag){
	  g.nodeBuffer[decl].dc_identifier.visb = visb_pri;
	}
	if (g.nodeBuffer[decl].common.public_flag){
	  g.nodeBuffer[decl].dc_identifier.visb = visb_pub;
	}
	if (g.nodeBuffer[decl].common.protected_flag){
	  g.nodeBuffer[decl].dc_identifier.visb = visb_pro;
	}
      } else {
	if (g.nodeBuffer[i].common.private_flag){
	  g.nodeBuffer[i].dc_identifier.visb = visb_pri;
	}
	if (g.nodeBuffer[i].common.public_flag){
	  g.nodeBuffer[i].dc_identifier.visb = visb_pub;
	}
	if (g.nodeBuffer[i].common.protected_flag){
	  g.nodeBuffer[i].dc_identifier.visb = visb_pro;
	}
      }

      /* name -- if available */
      g.nodeBuffer[i].dc_identifier.name = namestr;

      /* unused fields */
      g.nodeBuffer[i].common.other_node = -1;
      g.nodeBuffer[i].dc_identifier.defLoc = -1;
      g.nodeBuffer[i].dc_identifier.arcFriend = -1;
      g.nodeBuffer[i].dc_identifier.array_dim = -1;
      g.nodeBuffer[i].dc_type.implements = -1;

      /* allocat cInstance Node */
      new_node = newNode(&g);
      g.nodeBuffer[new_node].common.type = dC_INSTANCE;
      g.nodeBuffer[new_node].dc_arc.src = i;
      g.nodeBuffer[new_node].dc_arc.dest = unql;
      g.nodeBuffer[new_node].dc_arc.next = -1;
      g.nodeBuffer[new_node].dc_arc.ordinal = 0;
      g.nodeBuffer[i].dc_alias_type.original = new_node;
      g.nodeBuffer[i].dc_identifier.instance = new_node;
    }
  }
  /* all done, write the file */
  if (gd_graph_write_binary_file(outfilename,g) == 0){
    fprintf(stderr,"problem writing file\n");
    exit(1);
  }

  return 0;
}
