/*+
 * Name:	gd_break.c
 *
 * Purpose:	Transforms gcc break_stmt and continue_stmt to datrix break and
 * 		continue.
 *              -- Uses other_ptr to maintain the link between statements
 *
 * Author:	Thomas R. Dean
 *
 * Date:	March 26, 2001
 *
-*/

#include <stdio.h>
#include <stdlib.h>

#include "gd_graph.h"
#include "gd_utils.h"

char * progname;

void do_break(gd_graph *g, node_ptr i);
void do_continue(gd_graph *g, node_ptr i);

void usage(){
  fprintf(stderr,"usage: %s infile outfile\n",progname);
  exit(1);
}

int main(int argc, char * argv[]){
  gd_graph g;
  char * filename, *outfilename;
  int nodenum,i, count;
  progname = argv[0];

  if (argc != 3) {
    usage();
  }

  filename = argv[1];
  outfilename = argv[2];

  g = gd_graph_read_binary_file(filename,1.0,1.0,1.0);
  if (g.curStringPos == 0) {
    fprintf(stderr,"could not get graph\n");
    exit(1);
  }

  for(i = 0; i < g.curNodePos;i++){
    if (g.nodeBuffer[i].common.type == gBREAK_STMT){
      do_break(&g,i);
    }
    if (g.nodeBuffer[i].common.type == gCONTINUE_STMT){
      do_continue(&g,i);
    }
  }
  /* all done, write the file */
  if (gd_graph_write_binary_file(outfilename,g) == 0){
    fprintf(stderr,"problem writing file\n");
    exit(1);
  }

  return 0;
}

void do_break(gd_graph *g, node_ptr i){

  /*
   * fields to save
   */

  /* from break_stmt */
  int line;		/* src line				*/
  node_ptr next;	/* next in chain of statments		*/


  /* stuff to save from the current node */
  line = g->nodeBuffer[i].break_stmt.line;
  next = g->nodeBuffer[i].break_stmt.next;

  /****************************
   * generate datrix nodes
   ****************************/

  /* switch node type */
  g->nodeBuffer[i].common.type = dC_BREAK;

  g->nodeBuffer[i].dc_asg_nd.file = -1;
  g->nodeBuffer[i].dc_asg_nd.line = line;

  /* store chain in other_node, since datrix nodes do not have <next> ptrs */
  g->nodeBuffer[i].common.other_node = next;

  g-> nodeBuffer[i].dc_break.dest = -1; /* dest is Java not C++ */
}

void do_continue(gd_graph *g, node_ptr i){

  /*
   * fields to save
   */

  /* from continue_stmt */
  int line;		/* src line				*/
  node_ptr next;	/* next in chain of statments		*/


  /* stuff to save from the current node */
  line = g->nodeBuffer[i].continue_stmt.line;
  next = g->nodeBuffer[i].continue_stmt.next;

  /****************************
   * generate datrix nodes
   ****************************/

  /* switch node type */
  g->nodeBuffer[i].common.type = dC_CONTINUE;

  g->nodeBuffer[i].dc_asg_nd.file = -1;
  g->nodeBuffer[i].dc_asg_nd.line = line;

  /* store chain in other_node, since datrix nodes do not have <next> ptrs */
  g->nodeBuffer[i].common.other_node = next;

  g-> nodeBuffer[i].dc_break.dest = -1; /* dest is Java not C++ */
}
