/* Make cBuilitin entities and associated edges. */

/* AJM 2001 Mar 7 */

#include <stdio.h>
#include <gd_graph.h>

// Lookup string in hash table.
int main (int argc, char *argv [])
{
    void do_builtins ();
    char *infilename;
    char *outfilename;
    int i;
    gd_graph g;		// the graph

    if (argc != 3)
    {
      fprintf (stderr, "usage: %s ingdfile outgdfile\n", argv [0]);
      exit (1);
    }

    infilename = argv[1];
    outfilename = argv[2];

    g = gd_graph_read_binary_file (infilename, 1.0, 1.0, 1.0);
    if (g. curStringPos == 0) {
	fprintf (stderr,"can't read gd graph on %s\n", infilename);
	exit (1);
    }

    do_builtins (&g);

    if (gd_graph_write_binary_file (outfilename, g) == 0)
    {
	fprintf (stderr, "can't write %s\n", outfilename);
	exit (1);
    }

    return 0;
}

/* Builtin types on input are integer types whose source file is "<internal>". */
void do_builtins (gd_graph *g)
{
    str_ptr internal_ptr;		// ptr to "<internal>"
    node_ptr type_name_ptr;		// ptr to the name of a type

    node_ptr ptr;			// ptr to a builtin-type node

    gd_graph_node new_builtin_type;	// where the new builtin_type_p is accumulated

    internal_ptr = gd_lookup_string (g, "<internal>");
    if (internal_ptr == -1)
	// nothing to do: there are no internal types!
	return;

    for (ptr = 0; ptr < g-> curNodePos; ++ ptr)
	// Test for builtin.
	if (
		(g-> nodeBuffer [ptr]. common. type == gINTEGER_TYPE
	    ||  g-> nodeBuffer [ptr]. common. type == gBOOLEAN_TYPE
	    ||  g-> nodeBuffer [ptr]. common. type == gVOID_TYPE
	    ||  g-> nodeBuffer [ptr]. common. type == gREAL_TYPE)
	    &&	g-> nodeBuffer [g-> nodeBuffer [ptr]. type. name]. decl. srcf == internal_ptr
	    )
	{
	    // Make a new node for the builtin type.

	    new_builtin_type. common. type = dC_BUILT_IN_TYPE;
	    new_builtin_type. common. other_node = -1;
	    type_name_ptr = g-> nodeBuffer [ptr]. type. name;
	    new_builtin_type. dc_identifier. name =
		g-> nodeBuffer [type_name_ptr]. common. type == gIDENTIFIER_NODE?
		    g-> nodeBuffer [type_name_ptr]. identifier_node. strg:
		g-> nodeBuffer [type_name_ptr]. common. type == gTYPE_DECL?
		    g-> nodeBuffer [ g-> nodeBuffer [type_name_ptr]. decl. name  ]. identifier_node. strg:
		-1;	// can't find name.

	    new_builtin_type. dc_identifier. visb =
		g-> nodeBuffer [ptr]. common. public_flag? visb_pub:
		g-> nodeBuffer [ptr]. common. private_flag? visb_pri:
		g-> nodeBuffer [ptr]. common. protected_flag? visb_pro:
		visb_unk;
	    
	    new_builtin_type. dc_identifier. instance = -1;
	    new_builtin_type. dc_identifier. defLoc = -1;
	    new_builtin_type. dc_identifier. arcFriend = -1;
	    new_builtin_type. dc_identifier. array_dim = -1;
	    new_builtin_type. dc_type. implements = -1;

	    // Save the new node.
	    g-> nodeBuffer [ptr] = new_builtin_type;
	}
}
