/* Make enumerated type and members from EnumeralType */

/* AJM 2001 Mar 12 */

#include <stdio.h>
#include <gd_graph.h>
#include "gd_utils.h"

int main (int argc, char *argv [])
{
    void do_enum ();
    char *infilename;
    char *outfilename;
    int i;
    gd_graph g;		// the graph

    if (argc != 3)
    {
      fprintf (stderr, "usage: %s ingdfile outgdfile\n", argv [0]);
      exit (1);
    }

    infilename = argv[1];
    outfilename = argv[2];

    g = gd_graph_read_binary_file (infilename, 1.1, 1.0, 1.0);
    if (g. curStringPos == 0) {
	fprintf (stderr,"can't read gd graph on %s\n", infilename);
	exit (1);
    }

    do_enum (&g);

    if (gd_graph_write_binary_file (outfilename, g) == 0)
    {
	fprintf (stderr, "can't write %s\n", outfilename);
	exit (1);
    }

    return 0;
}

// The Belgians (gcc) call these "enumeral types".
// In gcc, the enumerated constants are available either as the csts: treelist of a
// t_enumeral_type, or as t_const_decls having type:s of type t_enumeral_type.  The treelist nodes
// have purp:s which are the names as t_identifier_nodes, and valu:s which give the values
// as t_integer_cst.   I use up the const decls to represent cEnumerators.

// In Datrix, the values declared for enumerated types are in the cEnumerator class,
// with their names, and values attached by cArcInitVal.

void do_enum (gd_graph *g)
{
    node_ptr ptr;			// Ptr to node which is type or constants

    gd_graph_node new_type;		// New Datrix node for type.
    gd_graph_node new_cnst;		// New Datrix node for constant.
    gd_graph_node new_edge;		// New Datrix node for edge.

    node_ptr name_ptr;			// ptr to the gcc name of a type

    // First step.  Convert all the gcc enuerated types to Datrix enumerated types.
    for (ptr = 0; ptr < g-> curNodePos; ++ ptr)
	if (g-> nodeBuffer [ptr]. common. type == gENUMERAL_TYPE)
	{
	    // Construct a new node for the type.
	    new_type. common. type = dC_ENUM_TYPE;
	    new_type. common. other_node = -1;
	    new_type. dc_asg_nd. file = g-> nodeBuffer [g-> nodeBuffer [ptr]. type. name]. decl. srcf;
	    new_type. dc_asg_nd. line = g-> nodeBuffer [g-> nodeBuffer [ptr]. type. name]. decl. srcl;

	    // Chase the type name.
	    name_ptr = g-> nodeBuffer [ptr]. type. name;
	    switch (g-> nodeBuffer [name_ptr]. common. type)
	    {
	    case gIDENTIFIER_NODE:
		new_type. dc_identifier. name = g-> nodeBuffer [name_ptr]. identifier_node. strg;
		break;
	    case gTYPE_DECL:
		name_ptr = g-> nodeBuffer [name_ptr]. decl. name;
		new_type. dc_identifier. name = g-> nodeBuffer [name_ptr]. identifier_node. strg;
		break;
	    default:
		// can't find name due to unexpected structure
		new_type. dc_identifier. name = -1;
		break;
	    }

	    new_type. dc_identifier. visb =
		    g-> nodeBuffer [ptr]. common. public_flag? visb_pub:
		    g-> nodeBuffer [ptr]. common. private_flag? visb_pri:
		    g-> nodeBuffer [ptr]. common. protected_flag? visb_pro:
		    visb_unk
		;
	    new_type. dc_identifier. instance = -1;
	    new_type. dc_identifier. defLoc = -1;
	    new_type. dc_identifier. arcFriend = -1;
	    new_type. dc_identifier. array_dim = -1;
	    new_type. dc_type. implements = -1;

	    // The values attribute should point at the first of a chain of dc_arc_son edges.
	    // Since they'll be converted 1-1 from the tree list on the csts pointer, 
	    // the values attribute can point to the first such node.
	    new_type. dc_enum_type. values = g-> nodeBuffer [ptr]. enumeral_type. csts;

	    // save it
	    g-> nodeBuffer [ptr] = new_type;
	}

    // Second step.  Each const_decl is converted to a dc_enumerator.
    for (ptr = 0; ptr < g-> curNodePos; ++ ptr)
	// We're handling enumerated constants, which are constant decls of enumerated types.
	if (
		g-> nodeBuffer [ptr]. common. type == gCONST_DECL
	    &&	g-> nodeBuffer [g-> nodeBuffer [ptr]. const_decl. decl. type]. common. type == dC_ENUM_TYPE
	)
	{
	    // Make a new enumerator node.
	    
	    new_cnst. common. type = dC_ENUMERATOR;
	    new_cnst. common. other_node = -1;
	    new_cnst. dc_asg_nd. file = g-> nodeBuffer [ptr]. decl. srcf;
	    new_cnst. dc_asg_nd. line = g-> nodeBuffer [ptr]. decl. srcl;
	    name_ptr = g-> nodeBuffer [ptr]. const_decl. decl. name;
	    new_cnst. dc_identifier. name = g-> nodeBuffer [name_ptr]. identifier_node. strg;
	    new_cnst. dc_identifier. visb =
		    g-> nodeBuffer [ptr]. common. public_flag? visb_pub:
		    g-> nodeBuffer [ptr]. common. private_flag? visb_pri:
		    g-> nodeBuffer [ptr]. common. protected_flag? visb_pro:
		    visb_unk
		;
	    new_cnst. dc_identifier. instance = -1;
	    new_cnst. dc_identifier. defLoc = -1;
	    new_cnst. dc_identifier. arcFriend = -1;
	    new_cnst. dc_identifier. array_dim = -1;
	    new_cnst. dc_enumerator. value = newNode (g);

	    // Construct and save a new edge for the initial value.
	    new_edge. common. type = dC_ARC_INIT_VAL;
	    new_edge. common. other_node = -1;
	    new_edge. dc_arc. ordinal = 0;
	    new_edge. dc_arc. src = ptr;
	    new_edge. dc_arc. dest = g-> nodeBuffer [ptr]. const_decl. cnst;
	    new_edge. dc_arc. next = -1;
	    g-> nodeBuffer [new_cnst. dc_enumerator. value] = new_edge;

	    // Make the value refer back to its enumerator.
	    // This is so that we can pass through enumerated types in the next step,
	    // and convert each tree-node into an arc-son edge pointing at the enumerator.
	    g-> nodeBuffer [new_edge. dc_arc. dest]. common. other_node = ptr;

	    // save it
	    g-> nodeBuffer [ptr] = new_cnst;
	}

    // Third step: reuse the tree list hanging off each enumerated type,  with each tree node
    // reused to be an arc-son edge from the type to the enumerator.

    for (ptr = 0; ptr < g-> curNodePos; ++ ptr)
	// Note: the types have been converted by now.
	if (g-> nodeBuffer [ptr]. common. type == dC_ENUM_TYPE)
	{
	    // Walk the csts tree list, converting them into C_ARC_SON edges.
	    node_ptr son_ptr;
	    node_ptr next_son_ptr;
	    int ordinal = 0;
	    for (son_ptr = g-> nodeBuffer [ptr]. dc_enum_type. values; son_ptr != -1; son_ptr = next_son_ptr)
	    {
		// Save the next pointer.
		next_son_ptr = g-> nodeBuffer [son_ptr]. tree_list. chan;

		// Construct the new edge
		new_edge. common. type = dC_ARC_SON;
		new_edge. common. other_node = -1;
		new_edge. dc_arc. ordinal = ++ ordinal;
		new_edge. dc_arc. src = ptr;
		// The destination is the new enumerator, which used to be a const_decl.  The other_node of its
		// integer constant was directed at the const_decl when we converted it to an enumerator.
		new_edge. dc_arc. dest =
		    g-> nodeBuffer [g-> nodeBuffer [son_ptr]. tree_list. value]. common. other_node;
		new_edge. dc_arc. next = -1;

		g-> nodeBuffer [son_ptr] = new_edge;
	    }
	}
}
