/*+
 * Name:	gd_func.c
 *
 * Purpose:	Transforms gcc functgion_decl and mthod_decl to datrix function.
 *              -- Turns chan linked pam_decl into arcson to FormalFctParam.
 *              -- gets return type from dc_fct_type given type link.
 *              Note: Must be run after funct_type.
 *
 * Author:	Thomas R. Dean
 *
 * Date:	March 7, 2001
 *
-*/

#include <stdio.h>
#include <stdlib.h>

#include "gd_graph.h"
#include "gd_utils.h"

char * progname;

void do_function_decl(gd_graph *g, node_ptr i);
void do_method_decl(gd_graph *g, node_ptr i);

node_ptr nextNode(gd_graph * g, node_ptr i);
void setNext(gd_graph * g, node_ptr i, node_ptr val);

void usage(){
  fprintf(stderr,"usage: %s infile outfile\n",progname);
  exit(1);
}

int main(int argc, char * argv[]){
  gd_graph g;
  char * filename, *outfilename;
  int nodenum,i, count;
  progname = argv[0];

  if (argc != 3) {
    usage();
  }

  filename = argv[1];
  outfilename = argv[2];

  g = gd_graph_read_binary_file(filename,1.0,1.0,1.0);
  if (g.curStringPos == 0) {
    fprintf(stderr,"could not get graph\n");
    exit(1);
  }

  for(i = 0; i < g.curNodePos;i++){
    if (g.nodeBuffer[i].common.type == gFUNCTION_DECL){
      do_function_decl(&g,i);
    }
  }
  /* all done, write the file */
  if (gd_graph_write_binary_file(outfilename,g) == 0){
    fprintf(stderr,"problem writing file\n");
    exit(1);
  }

  return 0;
}

void do_function_decl(gd_graph *g, node_ptr i){

  /*
   * fields to save
   */

  /* from type decl */
  node_ptr type;	/* points to function_type if it exists   */
  node_ptr instance;	/*  -- Instance arc to return type        */

  /* from function_decl */
  str_ptr srcf;		/* src file 				*/
  int srcl;		/* src line				*/
  node_ptr name;	/* identifier node			*/
  str_ptr  namestr;	/*    -- str_ptr for name		*/
  node_ptr args;
  node_ptr body;
  node_ptr funct_type_args;
  unsigned ext_static;
  unsigned undef;


  /* other locals */
  node_ptr new_node, 	/* two variables for node */
  	new_node2,	/*  allocation */
	next_node,	/* used when traversing paramters */
	prev_node,
	tmp;

  srcf = g->nodeBuffer[i].decl.srcf;
  srcl = g->nodeBuffer[i].decl.srcl;
  name = g->nodeBuffer[i].decl.name;
  if (name != -1){
    if (g->nodeBuffer[name].common.type != gIDENTIFIER_NODE) {
      fprintf(stderr,"decl name that is not a identifier!!(%d)\n",i);
      exit(1);
    }
    namestr = g->nodeBuffer[name].identifier_node.strg;
  } else {
    namestr = -1;
  }

  /* go get type information if it exists */
  type = g-> nodeBuffer[i].decl.type;
  if (type != -1){
    if (g->nodeBuffer[type].common.type != dC_FCT_TYPE){
      fprintf(stderr,"type link (%d,%d) is not to dc_fct_type!!\n",i,type);
      exit(1);
    }
    instance = g->nodeBuffer[type].dc_identifier.instance;
    funct_type_args = g->nodeBuffer[type].dc_fct_type.formalParams;
  }else {
    fprintf(stderr,"unknown type for function decl, needed for return type\n");
    exit(1);
  }

  /* stuff to save from the current node */
  args = g->nodeBuffer[i].function_decl.args;
  body = g->nodeBuffer[i].function_decl.body;
  ext_static = g->nodeBuffer[i].function_decl.ext_static;
  undef = g->nodeBuffer[i].function_decl.undef;

  /****************************
   * generate datrix nodes
   ****************************/

  /* switch node type */
  g->nodeBuffer[i].common.type = dC_FUNCTION;

  /* determine datrix visibility from common flags */
  /* gd_common doesn't change, no need to have saved flags */

  g->nodeBuffer[i].dc_identifier.visb = visb_unk;
  if (g->nodeBuffer[i].common.private_flag){
    g->nodeBuffer[i].dc_identifier.visb = visb_pri;
  }
  if (g->nodeBuffer[i].common.public_flag){
    g->nodeBuffer[i].dc_identifier.visb = visb_pub;
  }
  if (g->nodeBuffer[i].common.protected_flag){
    g->nodeBuffer[i].dc_identifier.visb = visb_pro;
  }

  /* other identifier and asg attributes */
  g->nodeBuffer[i].dc_asg_nd.line = srcl;
  g->nodeBuffer[i].dc_asg_nd.file = srcf;
  g->nodeBuffer[i].dc_identifier.name = namestr;

  /* unused fields */
  g->nodeBuffer[i].dc_identifier.defLoc = -1;
  g->nodeBuffer[i].dc_identifier.arcFriend = -1;

  /*
   * function fields
   */

  /* simple fields */
  g->nodeBuffer[i].dc_function.static_flag = !ext_static;
  g->nodeBuffer[i].dc_function.extern_flag = undef; /* extern flag is not documented,
						     assuming means defined elsewhere */

  g->nodeBuffer[i].dc_function.const_flag = 0; /* TODO */
  g->nodeBuffer[i].dc_function.virtual_code = 0; /* TODO */
  g->nodeBuffer[i].dc_function.exception = -1; /* TODO */

  /* relations from datrix nodes need nodes to hold the arc */
  g->nodeBuffer[i].dc_function.block = -1;
  /* replace block of code to make closer to datrix */
//  prev_node = -1;
//  while (body != -1){
//    if (g->nodeBuffer[body].common.type == gSCOPE_STMT) {
//	body = g->nodeBuffer[body].scope_stmt.next;
//    } else {
//      new_node = newNode(g);
//      if (new_node == -1){
//	fprintf(stderr,"out of nodes\n");
//	exit(1);
//      }
//      g-> nodeBuffer[new_node].common.type = dC_ARC_SON;
//      g-> nodeBuffer[new_node].common.other_node = -1;
//      g-> nodeBuffer[new_node].dc_arc.ordinal = 0;
//      g-> nodeBuffer[new_node].dc_arc.src = i;
//      g-> nodeBuffer[new_node].dc_arc.dest = body;
//      g-> nodeBuffer[new_node].dc_arc.next = -1;
//      if (prev_node != -1){
//         g-> nodeBuffer[prev_node].dc_arc.next = new_node;
//      }
//      prev_node = new_node;
//      if (g->nodeBuffer[i].dc_function.block== -1){
//	g->nodeBuffer[i].dc_function.block = new_node;
//      }
//      tmp = body;
//      body = nextNode(g,body);
//      setNext(g,tmp,-1);
//    }
//  } 

  /* structure of a gcc function body is:
   *     scope statment (begin)
   *     compound statment (contains entire function body)
   *     return statment (handles falling off end of function
   *     scope statement (end)
   * all we want is the compound statement.
   */

  if(body != -1){
    if (g->nodeBuffer[body].common.type == gSCOPE_STMT) {
	body = g->nodeBuffer[body].scope_stmt.next;
    }
    new_node = newNode(g);
    g-> nodeBuffer[new_node].common.type = dC_ARC_SON;
    g-> nodeBuffer[new_node].common.other_node = -1;
    g-> nodeBuffer[new_node].dc_arc.ordinal = 0;
    g-> nodeBuffer[new_node].dc_arc.src = i;
    g-> nodeBuffer[new_node].dc_arc.dest = body;
    g-> nodeBuffer[new_node].dc_arc.next = -1;
    g->nodeBuffer[i].dc_function.block = new_node;
    setNext(g,body,-1);
  }

  /* handle return type of function type */
  if (instance != -1){
    new_node = newNode(g);
    if (new_node == -1){
      fprintf(stderr,"out of nodes\n");
      exit(1);
    }
    g-> nodeBuffer[new_node].common.type = dC_INSTANCE;
    g-> nodeBuffer[new_node].dc_arc.ordinal = 0;
    g-> nodeBuffer[new_node].dc_arc.next = -1;
    g-> nodeBuffer[new_node].dc_arc.src = i;
    g-> nodeBuffer[new_node].dc_arc.dest = g->nodeBuffer[instance].dc_arc.dest;
    g-> nodeBuffer[i].dc_identifier.instance = new_node;
  }

  printf("args %d\n",args);
  /* parameters */
  g-> nodeBuffer[i].dc_function.formal_fct = -1;
  if (args != -1) {
    /* since gcc shares information up the wazoo, check
     * to see if parameter list has been already converted */
    if (g->nodeBuffer[args].common.type == gPARM_DECL){
      while(args != -1) {
	next_node = g->nodeBuffer[args].decl.chan;
	/* save information from parm_decl node */
	type = g->nodeBuffer[args].parm_decl.argt; /* type of parameter */
	srcf = g->nodeBuffer[args].decl.srcf;
	srcl = g->nodeBuffer[args].decl.srcl;
	name = g->nodeBuffer[args].decl.name;
	if (name != -1){
	  if (g->nodeBuffer[name].common.type != gIDENTIFIER_NODE) {
	    fprintf(stderr,"decl name that is not a identifier!!(%d)\n",i);
	    exit(1);
	  }
	  namestr = g->nodeBuffer[name].identifier_node.strg;
	} else {
	  namestr = -1;
	}


	new_node = newNode(g);
	if (new_node == -1) {
	  fprintf(stderr,"out of nodes\n");
	  exit(1);
	}
	g-> nodeBuffer[args].common.type = dC_FORMAL_FCT_PARAM;
	g->nodeBuffer[args].common.other_node = next_node;

	/* set attributes etc. */
	g->nodeBuffer[args].dc_identifier.visb = visb_unk;
	if (g->nodeBuffer[args].common.private_flag){
	  g->nodeBuffer[args].dc_identifier.visb = visb_pri;
	}
	if (g->nodeBuffer[args].common.public_flag){
	  g->nodeBuffer[args].dc_identifier.visb = visb_pub;
	}
	if (g->nodeBuffer[args].common.protected_flag){
	  g->nodeBuffer[args].dc_identifier.visb = visb_pro;
	}

	g->nodeBuffer[args].dc_object.extern_flag = 0;
	g->nodeBuffer[args].dc_object.static_flag = 0;
	g->nodeBuffer[args].dc_object.const_flag = 0; /* TODO */
	g->nodeBuffer[args].dc_identifier.array_dim = -1; /* TODO */
	g->nodeBuffer[args].dc_object.bitfield_dim = -1;
	g->nodeBuffer[args].dc_object.initial_val = -1;

	g->nodeBuffer[args].dc_identifier.instance = -1;
	g->nodeBuffer[args].dc_identifier.defLoc = -1;
	g->nodeBuffer[args].dc_identifier.arcFriend = -1;

	g->nodeBuffer[args].dc_asg_nd.file = srcf;
	g->nodeBuffer[args].dc_asg_nd.line = srcf;
	g->nodeBuffer[args].dc_identifier.name = namestr;

	/* create ArcSon link to parameter - reuse parameter node */
	new_node = newNode(g);
	g->nodeBuffer[new_node].common.type = dC_ARC_SON;
	g-> nodeBuffer[new_node].dc_arc.ordinal = 0;
	g-> nodeBuffer[new_node].dc_arc.next = -1;
	g-> nodeBuffer[new_node].dc_arc.src = i;
	g-> nodeBuffer[new_node].dc_arc.dest = args;
	if (g-> nodeBuffer[i].dc_function.formal_fct == -1){
	  g-> nodeBuffer[i].dc_function.formal_fct = new_node;
	} else {
	  g-> nodeBuffer[prev_node].dc_arc.next = new_node;
	}
	prev_node = new_node;

	/* create Instance link to parameter type */
	new_node2 = newNode(g);
	if (new_node2 == -1) {
	  fprintf(stderr,"out of nodes\n");
	  exit(1);
	}
	g-> nodeBuffer[new_node2].common.type = dC_INSTANCE;
	g-> nodeBuffer[new_node2].common.other_node = -1;
	g-> nodeBuffer[new_node2].dc_arc.ordinal = 0;
	g-> nodeBuffer[new_node2].dc_arc.next = -1;
	g-> nodeBuffer[new_node2].dc_arc.src = args;
	g-> nodeBuffer[new_node2].dc_arc.dest = type;
	g-> nodeBuffer[args].dc_identifier.instance = new_node2;

	/* next parameter */
	args = next_node;
      }
    } else { /* if (g->nodeBuffer[args].common.type == gPARM_DECL) */
      /* gcc has shared the parameter list, and it is already converted */
      /* all we have to do is copy the arcson list */
      if (g->nodeBuffer[args].common.type != dC_ARC_SON){
	fprintf(stderr,"formal parameter of unknown type\n");
	exit(1);
      }
      while (args != -1){
	new_node = newNode(g);
	if (new_node == -1) {
	  fprintf(stderr,"out of nodes\n");
	  exit(1);
	}
	/* copy relation information, except that src is the current node */
	g->nodeBuffer[new_node].common.type = dC_ARC_SON;
	g-> nodeBuffer[new_node].dc_arc.ordinal = 0;
	g-> nodeBuffer[new_node].dc_arc.next = -1;
	g-> nodeBuffer[new_node].dc_arc.src = i;
	g-> nodeBuffer[new_node].dc_arc.dest = g->nodeBuffer[args].dc_arc.dest;
	if (g-> nodeBuffer[i].dc_function.formal_fct == -1){
	  g-> nodeBuffer[i].dc_function.formal_fct = new_node;
	} else {
	  g-> nodeBuffer[prev_node].dc_arc.next = new_node;
	}
	prev_node = new_node;

      }
    }
  } else {
    /* the function_decl is a function prototype, and we must get the
     * parameters from the function type
     */
    fprintf(stderr,"funct_type_args: %d\n",funct_type_args);
    if (funct_type_args == -1) return; /* no args to convert */
    if (g->nodeBuffer[funct_type_args].common.type != dC_ARC_SON){
      fprintf(stderr,"not an arc??\n");
      exit(1);
    }
    printf("funct_type_args = %d\n",funct_type_args);
    while (funct_type_args != -1){
      printf("%d\n",funct_type_args);
      new_node = newNode(g);
      if (new_node == -1) {
	fprintf(stderr,"out of nodes\n");
	exit(1);
      }
      /* copy relation information, except that src is the current node */
      g->nodeBuffer[new_node].common.type = dC_ARC_SON;
      g->nodeBuffer[new_node].common.other_node = -1;
      g-> nodeBuffer[new_node].dc_arc.ordinal = 0;
      g-> nodeBuffer[new_node].dc_arc.next = -1;
      g-> nodeBuffer[new_node].dc_arc.src = i;
      g-> nodeBuffer[new_node].dc_arc.dest = g->nodeBuffer[funct_type_args].dc_arc.dest;
      if (g-> nodeBuffer[i].dc_function.formal_fct == -1){
	g-> nodeBuffer[i].dc_function.formal_fct = new_node;
      } else {
	g-> nodeBuffer[prev_node].dc_arc.next = new_node;
      }
      prev_node = new_node;
      funct_type_args = g->nodeBuffer[funct_type_args].dc_arc.next;
    }
  }
}

