// Replace t_if_stmts by dc_if and associated arcs.

/* AJM 2001 April 2 */

#include <stdio.h>
#include <gd_graph.h>
#include "gd_utils.h"

int main (int argc, char *argv [])
{
    void do_if (gd_graph *);
    char *infilename;
    char *outfilename;
    int i;
    gd_graph g;		// the graph

    if (argc != 3)
    {
      fprintf (stderr, "usage: %s ingdfile outgdfile\n", argv [0]);
      exit (1);
    }

    infilename = argv[1];
    outfilename = argv[2];

    g = gd_graph_read_binary_file (infilename, 1.1, 1.0, 1.1);
    if (g. curStringPos == 0) {
	fprintf (stderr,"can't read gd graph on %s\n", infilename);
	exit (1);
    }

    do_if (&g);

    if (gd_graph_write_binary_file (outfilename, g) == 0)
    {
	fprintf (stderr, "can't write %s\n", outfilename);
	exit (1);
    }

    return 0;
}

// Replace t_if_stmt by t_dc_if:
//	next -> other_node
// 	cond -> new cond edge
// 	thenc -> new true edge
//	elsec -> new false edge

// All very straightforward.

void do_if (gd_graph *g)
{
    node_ptr ptr;

    node_ptr ret_expr;

    gd_graph_node new_if;
    gd_graph_node new_arc_cond;
    gd_graph_node new_arc_true;
    gd_graph_node new_arc_false;

    for (ptr = 0; ptr < g-> curNodePos; ++ptr)
	if (g-> nodeBuffer [ptr]. common. type == gIF_STMT)
	{
	    // Make a new if node.
	    new_if. common. type = dC_IF;
	    new_if. common. other_node = g-> nodeBuffer [ptr]. if_stmt. next;

	    new_if. dc_if. cond = newNode (g);	// for cond arc
	    new_arc_cond. common. type = dC_ARC_COND;
	    new_arc_cond. common. other_node = -1;
	    new_arc_cond. dc_arc. ordinal = 0;
	    new_arc_cond. dc_arc. next = -1; /* TD */
	    new_arc_cond. dc_arc. src = ptr;
	    new_arc_cond. dc_arc. dest = g-> nodeBuffer [ptr]. if_stmt. cond;
	    g-> nodeBuffer [new_if. dc_if. cond] = new_arc_cond;

	    // true arc (can it be absent?)
	    if (g-> nodeBuffer [ptr]. if_stmt. thenc == -1)
	    	new_if. dc_if. true_block = -1;
	    else
	    {
	    	new_if. dc_if. true_block = newNode (g);	// for true arc
		new_arc_cond. common. type = dC_ARC_TRUE;
		new_arc_cond. common. other_node = -1;
		new_arc_cond. dc_arc. ordinal = 0;
		new_arc_cond. dc_arc. next = -1; /* TD */
		new_arc_cond. dc_arc. src = ptr;
		new_arc_cond. dc_arc. dest = g-> nodeBuffer [ptr]. if_stmt. thenc;
		g-> nodeBuffer [new_if. dc_if. true_block] = new_arc_cond;
	    }

	    // false arc, if present
	    if (g-> nodeBuffer [ptr]. if_stmt. elsec == -1)
	    	new_if. dc_if. false_block = -1;
	    else
	    {
	    	new_if. dc_if. false_block = newNode (g);	// for true arc
		new_arc_cond. common. type = dC_ARC_FALSE;
		new_arc_cond. common. other_node = -1;
		new_arc_cond. dc_arc. ordinal = 0;
		new_arc_cond. dc_arc. next = -1; /* TD */
		new_arc_cond. dc_arc. src = ptr;
		new_arc_cond. dc_arc. dest = g-> nodeBuffer [ptr]. if_stmt. elsec;
		g-> nodeBuffer [new_if. dc_if. false_block] = new_arc_cond;
	    }

	    g-> nodeBuffer [ptr] = new_if;
	}
}
