// Expression Statements (expr_stmts) in gcc represent ordinary statements in blocks.
// In the target Datrix model, these are simply expressions.

// So, after expression statements have been linked up into blocks, just replace every
// ptr in the world which pts to a expr_stmt so that it pts to the expression instead.

/* AJM 2001 Mar 14 */

#include <stdio.h>
#include <gd_graph.h>
#include "gd_utils.h"

// Defined in gd_walkers.c
void walk_ptrs (gd_graph *, void (*) (gd_graph *, node_ptr, node_ptr *, int));

// Defined here.
void cvt_expr_stmt (gd_graph *, node_ptr, node_ptr *, int);

int main (int argc, char *argv [])
{
    void do_expr_stmts (gd_graph *);
    char *infilename;
    char *outfilename;
    int i;
    gd_graph g;		// the graph

    if (argc != 3)
    {
      fprintf (stderr, "usage: %s ingdfile outgdfile\n", argv [0]);
      exit (1);
    }

    infilename = argv[1];
    outfilename = argv[2];

    g = gd_graph_read_binary_file (infilename, 1.1, 1.0, 1.1);
    if (g. curStringPos == 0) {
	fprintf (stderr,"can't read gd graph on %s\n", infilename);
	exit (1);
    }

    walk_ptrs (&g, cvt_expr_stmt);

    if (gd_graph_write_binary_file (outfilename, g) == 0)
    {
	fprintf (stderr, "can't write %s\n", outfilename);
	exit (1);
    }

    return 0;
}

void cvt_expr_stmt (gd_graph *g, node_ptr p, node_ptr *stmt_ptr, int n)
{
    // Skip any ptrs to other_node.
    if (n == 0)
	return;
    
    // Ignore nonptrs.
    if (0 <= *stmt_ptr && *stmt_ptr < g-> curNodePos); else return;

    // Ignore everything except ptrs to expression statements.
    if (g-> nodeBuffer [*stmt_ptr]. common. type != gEXPR_STMT)
	return;

    // Redirect the ptr.
    *stmt_ptr = g-> nodeBuffer [*stmt_ptr]. expr_stmt. texpr;
}
