// The return statement is explicated by gcc, so that if there's an implicit return in the code,
// it becomes explicit in the recovered graph.

// This program converts t_return_stmts to dc_returns.

/* AJM 2001 April 2 */

#include <stdio.h>
#include <gd_graph.h>
#include "gd_utils.h"

int main (int argc, char *argv [])
{
    void do_return (gd_graph *);
    char *infilename;
    char *outfilename;
    int i;
    gd_graph g;		// the graph

    if (argc != 3)
    {
      fprintf (stderr, "usage: %s ingdfile outgdfile\n", argv [0]);
      exit (1);
    }

    infilename = argv[1];
    outfilename = argv[2];

    g = gd_graph_read_binary_file (infilename, 1.1, 1.0, 1.1);
    if (g. curStringPos == 0) {
	fprintf (stderr,"can't read gd graph on %s\n", infilename);
	exit (1);
    }

    do_return (&g);

    if (gd_graph_write_binary_file (outfilename, g) == 0)
    {
	fprintf (stderr, "can't write %s\n", outfilename);
	exit (1);
    }

    return 0;
}

// Replace t_return_stmt by dc_return:
// 	next -> other_node
// 	ret_expr -> new son edge		(if any)

// In gcc the ret_expr pointer points at a t_init_expr whose op0 is a 'result decl'
// and whose op1 is the actual expression.  Perhaps this supports Pascal with its
// implicitly declared result variable.

void do_return (gd_graph *g)
{
    node_ptr ptr;

    node_ptr ret_expr;

    gd_graph_node new_return;
    gd_graph_node new_arc_son;

    for (ptr = 0; ptr < g-> curNodePos; ++ptr)
	if (g-> nodeBuffer [ptr]. common. type == gRETURN_STMT)
	{
	    // Make a new return statement.
	    new_return. common. type = dC_RETURN;
	    new_return. common. other_node = g-> nodeBuffer [ptr]. return_stmt. next;

	    // Is there a return expression?
	    if (g-> nodeBuffer [ptr]. return_stmt. ret_expr == -1)
		// Has no return expression.
		new_return. dc_return. value = -1;
	    else
	    {
		// Has return expression.
		new_return. dc_return. value = newNode (g);

		new_arc_son. common. type = dC_ARC_SON;
		new_arc_son. common. other_node = -1;
		new_arc_son. dc_arc. ordinal = 0;
		new_arc_son. dc_arc. src = ptr;

		// Check the ret. expr. is as expected...
		ret_expr = g-> nodeBuffer [ptr]. return_stmt. ret_expr;
		if (g-> nodeBuffer [ret_expr]. common. type != gINIT_EXPR)
		    fprintf (stderr, "Unexpected return expression, node %d\n", ptr);

		new_arc_son. dc_arc. dest = g-> nodeBuffer [ret_expr]. init_expr. op1;
		new_arc_son. dc_arc. next = -1;

		g-> nodeBuffer [new_return. dc_return. value] = new_arc_son;
	    }

	    // Save the new return node.
	    g-> nodeBuffer [ptr] = new_return;
	}
}
