// Switch statements are t_switch_stmts in gcc and t_dc_switch in Datrix.
// The transformation is pretty straightforward, but the Datrix manual in its original
// for is wrong, because it makes the body of a switch to be a sequence of labeled blocks.
// But in C and C++ and also Java, the body is a single block with inside case labels.

/* AJM 2001 April 2 */

#include <stdio.h>
#include <gd_graph.h>
#include "gd_utils.h"

int main (int argc, char *argv [])
{
    void do_switch (gd_graph *);
    char *infilename;
    char *outfilename;
    int i;
    gd_graph g;		// the graph

    if (argc != 3)
    {
      fprintf (stderr, "usage: %s ingdfile outgdfile\n", argv [0]);
      exit (1);
    }

    infilename = argv[1];
    outfilename = argv[2];

    g = gd_graph_read_binary_file (infilename, 1.1, 1.0, 1.1);
    if (g. curStringPos == 0) {
	fprintf (stderr,"can't read gd graph on %s\n", infilename);
	exit (1);
    }

    do_switch (&g);

    if (gd_graph_write_binary_file (outfilename, g) == 0)
    {
	fprintf (stderr, "can't write %s\n", outfilename);
	exit (1);
    }

    return 0;
}

// Replace t_switch_stmt nodes by t_dc_switch nodes:
// cond   ->    new arccond dest
// body   ->    new arcson dest
// next   ->    other_node

void do_switch (gd_graph *g)
{
    node_ptr ptr;

    gd_graph_node new_switch;
    gd_graph_node new_arc_son;
    gd_graph_node new_arc_cond;

    for (ptr = 0; ptr < g-> curNodePos; ++ptr)
	if (g-> nodeBuffer [ptr]. common. type == gSWITCH_STMT)
	{
	    // Reuse this node for the new switch.
	    new_switch. common. type = dC_SWITCH;
	    new_switch. common. other_node = g-> nodeBuffer [ptr]. switch_stmt. next;
	    new_switch. dc_switch. cond = newNode (g);	// for new arccond
	    new_switch. dc_switch. body = newNode (g);	// for new arcson

	    // Make the new condition edge.
	    new_arc_cond. common. type = dC_ARC_COND;
	    new_arc_cond. common. other_node = -1;
	    new_arc_cond. dc_arc. ordinal = 1;
	    new_arc_cond. dc_arc. src = ptr;
	    // The cond is (as usual) a tree_list, whose value is the thing we need.
	    new_arc_cond. dc_arc. dest = g-> nodeBuffer [g-> nodeBuffer [ptr]. switch_stmt. cond]. tree_list. value;
	    new_arc_cond. dc_arc. next = -1;

	    // Make the new son (block) edge.
	    new_arc_son. common. type = dC_ARC_SON;
	    new_arc_son. common. other_node = -1;
	    new_arc_son. dc_arc. ordinal = 1;
	    new_arc_son. dc_arc. src = ptr;
	    new_arc_son. dc_arc. dest = g-> nodeBuffer [ptr]. switch_stmt. body;
	    new_arc_son. dc_arc. next = -1;

	    // Save them back.
	    g-> nodeBuffer [ptr] = new_switch;
	    g-> nodeBuffer [new_switch. dc_switch. cond] = new_arc_cond;
	    g-> nodeBuffer [new_switch. dc_switch. body] = new_arc_son;
	}
}
