#include <stdio.h>
#include <stdlib.h>
#include <gd_utils.h>

node_ptr newNode(gd_graph *g){
  node_ptr i;
  for (i = 0; i < g->curNodePos; i++){
    if (g->nodeBuffer[i].common.type == NODE_DELTED){
      g->nodeBuffer[i].common.type = NODE_NONE;
      return i;
    }
  }
  if (g->curNodePos < g->nodeBuffLen){
    g->nodeBuffer[g->curNodePos].common.type = NODE_NONE;
    return g->curNodePos++;
  }
  /* should do something about reallocating memory here */
  if(!gd_grow_nodes(g,DEFAULT_GROW_NODES)){
    fprintf(stderr,"Out of nodes!!\n");
    exit(1);
  }
  if (g->curNodePos < g->nodeBuffLen){
    g->nodeBuffer[g->curNodePos].common.type = NODE_NONE;
    return g->curNodePos++;
  }
  /* if we get here, something went wrong with allocation */

  /* no more nodes */
  fprintf(stderr,"error in newNode\n");
  exit(1);
}

int UsedBy(gd_graph g, int dest, int useFlag) {
  int i;
  for(i = 0;i < g.curNodePos; i++){
    if (useFlag && g.nodeBuffer[i].common.mark_flag1){
      /* if we are to check the flag and the flag is
       * set -- skip this node */
      continue;
    } 
    if (nodeUses(g, i, dest)) {
      return i;
    }
  }
  /* no node uses the destination node, return
   * null node_ptr which is -1 */
  return -1;
}

static int edge_decl(t_gcc_decl decl, int dest);
static int edge_common(gd_graph_common common, int dest);
static int edge_type(t_gcc_type type, int dest);
static int edge_cst(t_gcc_const cst, int dest);
static int edge_expr(t_gcc_expr expr, int dest);

static int datrix_scope(t_dc_scope scope, node_ptr dest);
static int datrix_identifier(t_dc_identifier dc_identifier, node_ptr dest);
static int datrix_asg_nd(t_dc_asg_nd dc_asg_nd, node_ptr dest);
static int datrix_type(t_dc_type type, node_ptr dest);
static int datrix_object(t_dc_object object, node_ptr dest);
static int datrix_label(t_dc_label label, node_ptr dest);
static int datrix_loop(t_dc_loop dc_loop, node_ptr dest);
static int datrix_ctrl_stmt(t_dc_ctrl_stmt dc_ctrl_stmt, node_ptr dest);
static int datrix_expression(t_dc_expression dc_expression, node_ptr dest);
static int datrix_jump(t_dc_jump dc_jump, node_ptr dest);
static int datrix_comment(t_dc_comment dc_comment, node_ptr dest);
static int datrix_operator(t_dc_operator dc_operator, node_ptr dest);

static int datrix_arc_syntax(t_dc_arc_syntax dc_arc_syntax, node_ptr dest);
static int datrix_arc_reference(t_dc_arc_reference dc_arc_reference, node_ptr dest);
static int datrix_arc(t_dc_arc dc_arc, node_ptr dest);

int nodeUses(gd_graph g, int src, int dest) {
  switch(g.nodeBuffer[src].common.type){
      case gNAMESPACE_DECL:
	if (edge_decl(g.nodeBuffer[src].decl,dest)) return 1;
	if (g.nodeBuffer[src].namespace_decl.dcls == dest) return 1;
	if (g.nodeBuffer[src].namespace_decl.alis == dest) return 1;
	break;
      case gFUNCTION_DECL:
	if (edge_decl(g.nodeBuffer[src].decl,dest)) return 1;
	if (g.nodeBuffer[src].function_decl.args == dest) return 1;
	if (g.nodeBuffer[src].function_decl.body == dest) return 1;
	break;
      case gVAR_DECL:
	if (edge_decl(g.nodeBuffer[src].decl,dest)) return 1;
	if (g.nodeBuffer[src].var_decl.init == dest) return 1;
	if (g.nodeBuffer[src].var_decl.size == dest) return 1;
	break;
      case gPARM_DECL:
	if (edge_decl(g.nodeBuffer[src].decl,dest)) return 1;
	if (g.nodeBuffer[src].parm_decl.argt == dest) return 1;
	if (g.nodeBuffer[src].parm_decl.size == dest) return 1;
	break;
      case gTYPE_DECL:
	if (edge_decl(g.nodeBuffer[src].decl,dest)) return 1;
	break;
      case gLABEL_DECL:
	if (edge_decl(g.nodeBuffer[src].decl,dest)) return 1;
	break;
      case gCONST_DECL:
	if (edge_decl(g.nodeBuffer[src].decl,dest)) return 1;
	if (g.nodeBuffer[src].const_decl.cnst == dest) return 1;
	break;
      case gRESULT_DECL:
	if (edge_decl(g.nodeBuffer[src].decl,dest)) return 1;
	if (g.nodeBuffer[src].result_decl.init == dest) return 1;
	if (g.nodeBuffer[src].result_decl.size == dest) return 1;
	break;
      case gFIELD_DECL:
	if (edge_decl(g.nodeBuffer[src].decl,dest)) return 1;
	if (g.nodeBuffer[src].field_decl.init == dest) return 1;
	if (g.nodeBuffer[src].field_decl.size == dest) return 1;
	if (g.nodeBuffer[src].field_decl.bpos == dest) return 1;
	break;
      case gTEMPLATE_DECL:
	if (edge_decl(g.nodeBuffer[src].decl,dest)) return 1;
	if (g.nodeBuffer[src].template_decl.rslt == dest) return 1;
	if (g.nodeBuffer[src].template_decl.inst == dest) return 1;
	if (g.nodeBuffer[src].template_decl.spcs == dest) return 1;
	break;
      case gUSING_DECL:
	if (edge_decl(g.nodeBuffer[src].decl,dest)) return 1;
	break;
      case gFUNCTION_TYPE:
	if (edge_type(g.nodeBuffer[src].type,dest)) return 1;
	if (g.nodeBuffer[src].function_type.retn == dest) return 1;
	if (g.nodeBuffer[src].function_type.prms == dest) return 1;
	break;
      case gINTEGER_TYPE:
	if (edge_type(g.nodeBuffer[src].type,dest)) return 1;
	if (g.nodeBuffer[src].integer_type.min == dest) return 1;
	if (g.nodeBuffer[src].integer_type.max == dest) return 1;
	break;
      case gARRAY_TYPE:
	if (edge_type(g.nodeBuffer[src].type,dest)) return 1;
	if (g.nodeBuffer[src].array_type.elts == dest) return 1;
	if (g.nodeBuffer[src].array_type.domn == dest) return 1;
	break;
      case gVOID_TYPE:
	if (edge_type(g.nodeBuffer[src].type,dest)) return 1;
	break;
      case gREAL_TYPE:
	if (edge_type(g.nodeBuffer[src].type,dest)) return 1;
	break;
      case gCOMPLEX_TYPE:
	if (edge_type(g.nodeBuffer[src].type,dest)) return 1;
	break;
      case gVECTOR_TYPE:
	if (edge_type(g.nodeBuffer[src].type,dest)) return 1;
	break;
      case gENUMERAL_TYPE:
	if (edge_type(g.nodeBuffer[src].type,dest)) return 1;
	if (g.nodeBuffer[src].enumeral_type.min == dest) return 1;
	if (g.nodeBuffer[src].enumeral_type.max == dest) return 1;
	if (g.nodeBuffer[src].enumeral_type.csts == dest) return 1;
	break;
      case gBOOLEAN_TYPE:
	if (edge_type(g.nodeBuffer[src].type,dest)) return 1;
	break;
      case gCHAR_TYPE:
	if (edge_type(g.nodeBuffer[src].type,dest)) return 1;
	break;
      case gPOINTER_TYPE:
	if (edge_type(g.nodeBuffer[src].type,dest)) return 1;
	if (g.nodeBuffer[src].pointer_type.ptd == dest) return 1;
	if (g.nodeBuffer[src].pointer_type.cls == dest) return 1;
	break;
      case gOFFSET_TYPE:
	if (edge_type(g.nodeBuffer[src].type,dest)) return 1;
	break;
      case gREFERENCE_TYPE:
	if (edge_type(g.nodeBuffer[src].type,dest)) return 1;
	if (g.nodeBuffer[src].reference_type.refd == dest) return 1;
	break;
      case gMETHOD_TYPE:
	if (edge_type(g.nodeBuffer[src].type,dest)) return 1;
	if (g.nodeBuffer[src].method_type.clas == dest) return 1;
	if (g.nodeBuffer[src].method_type.retn == dest) return 1;
	if (g.nodeBuffer[src].method_type.prms == dest) return 1;
	break;
      case gFILE_TYPE:
	if (edge_type(g.nodeBuffer[src].type,dest)) return 1;
	break;
      case gSET_TYPE:
	if (edge_type(g.nodeBuffer[src].type,dest)) return 1;
	break;
      case gRECORD_TYPE:
	if (edge_type(g.nodeBuffer[src].type,dest)) return 1;
	if (g.nodeBuffer[src].record_type.flds == dest) return 1;
	if (g.nodeBuffer[src].record_type.fncs == dest) return 1;
	if (g.nodeBuffer[src].record_type.ptd == dest) return 1;
	if (g.nodeBuffer[src].record_type.cls == dest) return 1;
	if (g.nodeBuffer[src].record_type.vfld == dest) return 1;
	if (g.nodeBuffer[src].record_type.binf == dest) return 1;
	break;
      case gUNION_TYPE:
	if (edge_type(g.nodeBuffer[src].type,dest)) return 1;
	if (g.nodeBuffer[src].record_type.flds == dest) return 1;
	if (g.nodeBuffer[src].record_type.fncs == dest) return 1;
	if (g.nodeBuffer[src].record_type.ptd == dest) return 1;
	if (g.nodeBuffer[src].record_type.cls == dest) return 1;
	if (g.nodeBuffer[src].record_type.vfld == dest) return 1;
	if (g.nodeBuffer[src].record_type.binf == dest) return 1;
	break;
      case gQUAL_UNION_TYPE:
	if (edge_type(g.nodeBuffer[src].type,dest)) return 1;
	break;
      case gLANG_TYPE:
	if (edge_type(g.nodeBuffer[src].type,dest)) return 1;
	break;
      case gTEMPLATE_TYPE_PARM:
	if (edge_type(g.nodeBuffer[src].type,dest)) return 1;
	break;
      case gTEMPLATE_PARM_INDEX:
	if (edge_common(g.nodeBuffer[src].common,dest)) return 1;
	break;
      case gTEMPLATE_TEMPLATE_PARM:
	if (edge_type(g.nodeBuffer[src].type,dest)) return 1;
	break;
      case gBOUND_TEMPLATE_TEMPLATE_PARM:
	if (edge_type(g.nodeBuffer[src].type,dest)) return 1;
	break;
      case gTYPENAME_TYPE:
	if (edge_type(g.nodeBuffer[src].type,dest)) return 1;
	break;
      case gTYPEOF_TYPE:
	if (edge_type(g.nodeBuffer[src].type,dest)) return 1;
	break;
      case gBLOCK:
	if (edge_common(g.nodeBuffer[src].common,dest)) return 1;
	break;
      case gCOMPOUND_STMT:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].compound_stmt.body == dest) return 1;
	if (g.nodeBuffer[src].compound_stmt.next == dest) return 1;
	break;
      case gEXPR_STMT:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].expr_stmt.texpr == dest) return 1;
	if (g.nodeBuffer[src].expr_stmt.next == dest) return 1;
	break;
      case gDECL_STMT:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].decl_stmt.decl == dest) return 1;
	if (g.nodeBuffer[src].decl_stmt.next == dest) return 1;
	break;
      case gSCOPE_STMT:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].scope_stmt.next == dest) return 1;
	break;
      case gRETURN_STMT:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].return_stmt.ret_expr == dest) return 1;
	if (g.nodeBuffer[src].return_stmt.next == dest) return 1;
	break;
      case gSTMT_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].stmt_expr.stmt == dest) return 1;
	break;
      case gBREAK_STMT:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].break_stmt.next == dest) return 1;
	break;
      case gCONTINUE_STMT:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].continue_stmt.next == dest) return 1;
	break;
      case gIF_STMT:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].if_stmt.cond == dest) return 1;
	if (g.nodeBuffer[src].if_stmt.thenc == dest) return 1;
	if (g.nodeBuffer[src].if_stmt.elsec == dest) return 1;
	if (g.nodeBuffer[src].if_stmt.next == dest) return 1;
	break;
      case gSWITCH_STMT:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].switch_stmt.cond == dest) return 1;
	if (g.nodeBuffer[src].switch_stmt.body == dest) return 1;
	if (g.nodeBuffer[src].switch_stmt.next == dest) return 1;
	break;
      case gLABEL_STMT:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].label_stmt.label == dest) return 1;
	if (g.nodeBuffer[src].label_stmt.next == dest) return 1;
	break;
      case gTRY_BLOCK:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].try_block.body == dest) return 1;
	if (g.nodeBuffer[src].try_block.hdlr == dest) return 1;
	if (g.nodeBuffer[src].try_block.next == dest) return 1;
	break;
      case gHANDLER:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].handler.body == dest) return 1;
	if (g.nodeBuffer[src].handler.next == dest) return 1;
	break;
      case gSTART_CATCH_STMT:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].start_catch_stmt.catch_type == dest) return 1;
	if (g.nodeBuffer[src].start_catch_stmt.next == dest) return 1;
	break;
      case gCLEANUP_STMT:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].cleanup_stmt.decl == dest) return 1;
	if (g.nodeBuffer[src].cleanup_stmt.clean_expr == dest) return 1;
	if (g.nodeBuffer[src].cleanup_stmt.next == dest) return 1;
	break;
      case gCASE_LABEL:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].case_label.low == dest) return 1;
	if (g.nodeBuffer[src].case_label.high == dest) return 1;
	if (g.nodeBuffer[src].case_label.next == dest) return 1;
	break;
      case gGOTO_STMT:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].goto_stmt.dest == dest) return 1;
	if (g.nodeBuffer[src].goto_stmt.next == dest) return 1;
	break;
      case gFOR_STMT:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].for_stmt.init == dest) return 1;
	if (g.nodeBuffer[src].for_stmt.cond == dest) return 1;
	if (g.nodeBuffer[src].for_stmt.inc_expr == dest) return 1;
	if (g.nodeBuffer[src].for_stmt.body == dest) return 1;
	if (g.nodeBuffer[src].for_stmt.next == dest) return 1;
	break;
      case gWHILE_STMT:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].while_stmt.cond == dest) return 1;
	if (g.nodeBuffer[src].while_stmt.body == dest) return 1;
	if (g.nodeBuffer[src].while_stmt.next == dest) return 1;
	break;
      case gDO_STMT:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].do_stmt.cond == dest) return 1;
	if (g.nodeBuffer[src].do_stmt.body == dest) return 1;
	if (g.nodeBuffer[src].do_stmt.next == dest) return 1;
	break;
      case gASM_STMT:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].asm_stmt.strg == dest) return 1;
	if (g.nodeBuffer[src].asm_stmt.outs == dest) return 1;
	if (g.nodeBuffer[src].asm_stmt.ins == dest) return 1;
	if (g.nodeBuffer[src].asm_stmt.clbr == dest) return 1;
	if (g.nodeBuffer[src].asm_stmt.next == dest) return 1;
	break;
      case gCOMPONENT_REF:
	if (edge_common(g.nodeBuffer[src].common,dest)) return 1;
	if (g.nodeBuffer[src].component_ref.op0 == dest) return 1;
	break;
      case gBIT_FIELD_REF:
	if (edge_common(g.nodeBuffer[src].common,dest)) return 1;
	break;
      case gINDIRECT_REF:
	if (edge_common(g.nodeBuffer[src].common,dest)) return 1;
	if (g.nodeBuffer[src].indirect_ref.op0 == dest) return 1;
	break;
      case gBUFFER_REF:
	if (edge_common(g.nodeBuffer[src].common,dest)) return 1;
	break;
      case gARRAY_REF:
	if (edge_common(g.nodeBuffer[src].common,dest)) return 1;
	if (g.nodeBuffer[src].array_ref.op0 == dest) return 1;
	if (g.nodeBuffer[src].array_ref.op1 == dest) return 1;
	break;
      case gOFFSET_REF:
	if (edge_common(g.nodeBuffer[src].common,dest)) return 1;
	if (g.nodeBuffer[src].offset_ref.op0 == dest) return 1;
	if (g.nodeBuffer[src].offset_ref.op1 == dest) return 1;
	break;
      case gSCOPE_REF:
	if (edge_common(g.nodeBuffer[src].common,dest)) return 1;
	if (g.nodeBuffer[src].scope_ref.op0 == dest) return 1;
	if (g.nodeBuffer[src].scope_ref.op1 == dest) return 1;
	break;
      case gMEMBER_REF:
	if (edge_common(g.nodeBuffer[src].common,dest)) return 1;
	if (g.nodeBuffer[src].member_ref.op0 == dest) return 1;
	if (g.nodeBuffer[src].member_ref.op1 == dest) return 1;
	break;
      case gCONSTRUCTOR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].constructor.elts == dest) return 1;
	break;
      case gCOMPOUND_EXPR:
	if (edge_common(g.nodeBuffer[src].common,dest)) return 1;
	if (g.nodeBuffer[src].compound_expr.op0 == dest) return 1;
	if (g.nodeBuffer[src].compound_expr.op1 == dest) return 1;
	break;
      case gMODIFY_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].modify_expr.op0 == dest) return 1;
	if (g.nodeBuffer[src].modify_expr.op1 == dest) return 1;
	break;
      case gINIT_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].init_expr.op0 == dest) return 1;
	if (g.nodeBuffer[src].init_expr.op1 == dest) return 1;
	break;
      case gTHROW_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].throw_expr.op0 == dest) return 1;
	break;
      case gTARGET_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].target_expr.decl == dest) return 1;
	if (g.nodeBuffer[src].target_expr.init == dest) return 1;
	if (g.nodeBuffer[src].target_expr.clnp == dest) return 1;
	break;
      case gCOND_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].cond_expr.op0 == dest) return 1;
	if (g.nodeBuffer[src].cond_expr.op1 == dest) return 1;
	if (g.nodeBuffer[src].cond_expr.op2 == dest) return 1;
	break;
      case gBIND_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].bind_expr.vars == dest) return 1;
	if (g.nodeBuffer[src].bind_expr.body == dest) return 1;
	break;
      case gCALL_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].call_expr.fn == dest) return 1;
	if (g.nodeBuffer[src].call_expr.args == dest) return 1;
	break;
      case gMETHOD_CALL_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	break;
      case gWITH_CLEANUP_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	break;
      case gCLEANUP_POINT_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].cleanup_point_expr.op0 == dest) return 1;
	break;
      case gPLACEHOLDER_EXPR:
	if (edge_common(g.nodeBuffer[src].common,dest)) return 1;
	break;
      case gWITH_RECORD_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	break;
      case gPLUS_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].plus_expr.expr2.op0 == dest) return 1;
	if (g.nodeBuffer[src].plus_expr.expr2.op1 == dest) return 1;
	break;
      case gMINUS_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].minus_expr.expr2.op0 == dest) return 1;
	if (g.nodeBuffer[src].minus_expr.expr2.op1 == dest) return 1;
	break;
      case gMULT_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].mult_expr.expr2.op0 == dest) return 1;
	if (g.nodeBuffer[src].mult_expr.expr2.op1 == dest) return 1;
	break;
      case gTRUNC_DIV_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].trunc_div_expr.expr2.op0 == dest) return 1;
	if (g.nodeBuffer[src].trunc_div_expr.expr2.op1 == dest) return 1;
	break;
      case gCEIL_DIV_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].ceil_div_expr.expr2.op0 == dest) return 1;
	if (g.nodeBuffer[src].ceil_div_expr.expr2.op1 == dest) return 1;
	break;
      case gFLOOR_DIV_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].floor_div_expr.expr2.op0 == dest) return 1;
	if (g.nodeBuffer[src].floor_div_expr.expr2.op1 == dest) return 1;
	break;
      case gROUND_DIV_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].round_div_expr.expr2.op0 == dest) return 1;
	if (g.nodeBuffer[src].round_div_expr.expr2.op1 == dest) return 1;
	break;
      case gTRUNC_MOD_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].trunc_mod_expr.expr2.op0 == dest) return 1;
	if (g.nodeBuffer[src].trunc_mod_expr.expr2.op1 == dest) return 1;
	break;
      case gCEIL_MOD_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].ceil_mod_expr.expr2.op0 == dest) return 1;
	if (g.nodeBuffer[src].ceil_mod_expr.expr2.op1 == dest) return 1;
	break;
      case gFLOOR_MOD_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].floor_mod_expr.expr2.op0 == dest) return 1;
	if (g.nodeBuffer[src].floor_mod_expr.expr2.op1 == dest) return 1;
	break;
      case gROUND_MOD_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].round_mod_expr.expr2.op0 == dest) return 1;
	if (g.nodeBuffer[src].round_mod_expr.expr2.op1 == dest) return 1;
	break;
      case gRDIV_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].rdiv_expr.expr2.op0 == dest) return 1;
	if (g.nodeBuffer[src].rdiv_expr.expr2.op1 == dest) return 1;
	break;
      case gEXACT_DIV_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].exact_div_expr.expr2.op0 == dest) return 1;
	if (g.nodeBuffer[src].exact_div_expr.expr2.op1 == dest) return 1;
	break;
      case gFIX_TRUNC_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].fix_trunc_expr.expr1.op0 == dest) return 1;
	break;
      case gFIX_CEIL_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].fix_ceil_expr.expr1.op0 == dest) return 1;
	break;
      case gFIX_FLOOR_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].fix_floor_expr.expr1.op0 == dest) return 1;
	break;
      case gFIX_ROUND_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].fix_round_expr.expr1.op0 == dest) return 1;
	break;
      case gFLOAT_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].float_expr.expr1.op0 == dest) return 1;
	break;
      case gEXPON_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].expon_expr.expr2.op0 == dest) return 1;
	if (g.nodeBuffer[src].expon_expr.expr2.op1 == dest) return 1;
	break;
      case gNEGATE_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].negate_expr.expr1.op0 == dest) return 1;
	break;
      case gMIN_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].min_expr.expr2.op0 == dest) return 1;
	if (g.nodeBuffer[src].min_expr.expr2.op1 == dest) return 1;
	break;
      case gMAX_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].max_expr.expr2.op0 == dest) return 1;
	if (g.nodeBuffer[src].max_expr.expr2.op1 == dest) return 1;
	break;
      case gABS_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].abs_expr.expr1.op0 == dest) return 1;
	break;
      case gFFS_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].ffs_expr.expr1.op0 == dest) return 1;
	break;
      case gLSHIFT_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].lshift_expr.expr2.op0 == dest) return 1;
	if (g.nodeBuffer[src].lshift_expr.expr2.op1 == dest) return 1;
	break;
      case gRSHIFT_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].rshift_expr.expr2.op0 == dest) return 1;
	if (g.nodeBuffer[src].rshift_expr.expr2.op1 == dest) return 1;
	break;
      case gLROTATE_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].lrotate_expr.expr2.op0 == dest) return 1;
	if (g.nodeBuffer[src].lrotate_expr.expr2.op1 == dest) return 1;
	break;
      case gRROTATE_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].rrotate_expr.expr2.op0 == dest) return 1;
	if (g.nodeBuffer[src].rrotate_expr.expr2.op1 == dest) return 1;
	break;
      case gBIT_IOR_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].bit_ior_expr.expr2.op0 == dest) return 1;
	if (g.nodeBuffer[src].bit_ior_expr.expr2.op1 == dest) return 1;
	break;
      case gBIT_XOR_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].bit_xor_expr.expr2.op0 == dest) return 1;
	if (g.nodeBuffer[src].bit_xor_expr.expr2.op1 == dest) return 1;
	break;
      case gBIT_AND_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].bit_and_expr.expr2.op0 == dest) return 1;
	if (g.nodeBuffer[src].bit_and_expr.expr2.op1 == dest) return 1;
	break;
      case gBIT_ANDTC_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].bit_andtc_expr.expr2.op0 == dest) return 1;
	if (g.nodeBuffer[src].bit_andtc_expr.expr2.op1 == dest) return 1;
	break;
      case gBIT_NOT_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].bit_not_expr.expr1.op0 == dest) return 1;
	break;
      case gTRUTH_ANDIF_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].truth_andif_expr.op0 == dest) return 1;
	if (g.nodeBuffer[src].truth_andif_expr.op1 == dest) return 1;
	break;
      case gTRUTH_ORIF_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].truth_orif_expr.op0 == dest) return 1;
	if (g.nodeBuffer[src].truth_orif_expr.op1 == dest) return 1;
	break;
      case gTRUTH_AND_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].truth_and_expr.op0 == dest) return 1;
	if (g.nodeBuffer[src].truth_and_expr.op1 == dest) return 1;
	break;
      case gTRUTH_OR_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].truth_or_expr.op0 == dest) return 1;
	if (g.nodeBuffer[src].truth_or_expr.op1 == dest) return 1;
	break;
      case gTRUTH_XOR_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].truth_xor_expr.op0 == dest) return 1;
	if (g.nodeBuffer[src].truth_xor_expr.op1 == dest) return 1;
	break;
      case gTRUTH_NOT_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].truth_not_expr.op0 == dest) return 1;
	break;
      case gLT_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].lt_expr.expr_ineq.op0 == dest) return 1;
	if (g.nodeBuffer[src].lt_expr.expr_ineq.op1 == dest) return 1;
	break;
      case gLE_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].le_expr.expr_ineq.op0 == dest) return 1;
	if (g.nodeBuffer[src].le_expr.expr_ineq.op1 == dest) return 1;
	break;
      case gGT_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].gt_expr.expr_ineq.op0 == dest) return 1;
	if (g.nodeBuffer[src].gt_expr.expr_ineq.op1 == dest) return 1;
	break;
      case gGE_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].ge_expr.expr_ineq.op0 == dest) return 1;
	if (g.nodeBuffer[src].ge_expr.expr_ineq.op1 == dest) return 1;
	break;
      case gEQ_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].eq_expr.expr_ineq.op0 == dest) return 1;
	if (g.nodeBuffer[src].eq_expr.expr_ineq.op1 == dest) return 1;
	break;
      case gNE_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].ne_expr.expr_ineq.op0 == dest) return 1;
	if (g.nodeBuffer[src].ne_expr.expr_ineq.op1 == dest) return 1;
	break;
      case gUNORDERED_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].unordered_expr.expr_ineq.op0 == dest) return 1;
	if (g.nodeBuffer[src].unordered_expr.expr_ineq.op1 == dest) return 1;
	break;
      case gORDERED_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].ordered_expr.expr_ineq.op0 == dest) return 1;
	if (g.nodeBuffer[src].ordered_expr.expr_ineq.op1 == dest) return 1;
	break;
      case gUNLT_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].unlt_expr.expr_ineq.op0 == dest) return 1;
	if (g.nodeBuffer[src].unlt_expr.expr_ineq.op1 == dest) return 1;
	break;
      case gUNLE_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].unle_expr.expr_ineq.op0 == dest) return 1;
	if (g.nodeBuffer[src].unle_expr.expr_ineq.op1 == dest) return 1;
	break;
      case gUNGT_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].ungt_expr.expr_ineq.op0 == dest) return 1;
	if (g.nodeBuffer[src].ungt_expr.expr_ineq.op1 == dest) return 1;
	break;
      case gUNGE_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].unge_expr.expr_ineq.op0 == dest) return 1;
	if (g.nodeBuffer[src].unge_expr.expr_ineq.op1 == dest) return 1;
	break;
      case gUNEQ_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].uneq_expr.expr_ineq.op0 == dest) return 1;
	if (g.nodeBuffer[src].uneq_expr.expr_ineq.op1 == dest) return 1;
	break;
      case gIN_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].in_expr.expr2.op0 == dest) return 1;
	if (g.nodeBuffer[src].in_expr.expr2.op1 == dest) return 1;
	break;
      case gSET_LE_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].set_le_expr.expr_ineq.op0 == dest) return 1;
	if (g.nodeBuffer[src].set_le_expr.expr_ineq.op1 == dest) return 1;
	break;
      case gCARD_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].card_expr.expr1.op0 == dest) return 1;
	break;
      case gRANGE_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].range_expr.expr2.op0 == dest) return 1;
	if (g.nodeBuffer[src].range_expr.expr2.op1 == dest) return 1;
	break;
      case gCONVERT_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].convert_expr.expr1.op0 == dest) return 1;
	break;
      case gNOP_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].nop_expr.expr1.op0 == dest) return 1;
	break;
      case gNON_LVALUE_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].non_lvalue_expr.expr1.op0 == dest) return 1;
	break;
      case gSAVE_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].save_expr.op0 == dest) return 1;
	break;
      case gUNSAVE_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	break;
      case gRTL_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	break;
      case gADDR_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].addr_expr.op0 == dest) return 1;
	break;
      case gREFERENCE_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	break;
      case gENTRY_VALUE_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	break;
      case gCOMPLEX_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].complex_expr.expr2.op0 == dest) return 1;
	if (g.nodeBuffer[src].complex_expr.expr2.op1 == dest) return 1;
	break;
      case gCONJ_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].conj_expr.expr1.op0 == dest) return 1;
	break;
      case gREALPART_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].realpart_expr.expr1.op0 == dest) return 1;
	break;
      case gPREDECREMENT_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].predecrement_expr.op0 == dest) return 1;
	if (g.nodeBuffer[src].predecrement_expr.op1 == dest) return 1;
	break;
      case gPREINCREMENT_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].preincrement_expr.op0 == dest) return 1;
	if (g.nodeBuffer[src].preincrement_expr.op1 == dest) return 1;
	break;
      case gPOSTDECREMENT_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].postdecrement_expr.op0 == dest) return 1;
	if (g.nodeBuffer[src].postdecrement_expr.op1 == dest) return 1;
	break;
      case gPOSTINCREMENT_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].postincrement_expr.op0 == dest) return 1;
	if (g.nodeBuffer[src].postincrement_expr.op1 == dest) return 1;
	break;
      case gVA_ARG_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].va_arg_expr.op0 == dest) return 1;
	break;
      case gTRY_CATCH_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].try_catch_expr.op0 == dest) return 1;
	if (g.nodeBuffer[src].try_catch_expr.op1 == dest) return 1;
	break;
      case gTRY_FINALLY_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].try_finally_expr.op0 == dest) return 1;
	if (g.nodeBuffer[src].try_finally_expr.op1 == dest) return 1;
	break;
      case gGOTO_SUBROUTINE_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].goto_subroutine_expr.op0 == dest) return 1;
	if (g.nodeBuffer[src].goto_subroutine_expr.op1 == dest) return 1;
	break;
      case gPOPDHC_EXPR:
	if (edge_common(g.nodeBuffer[src].common,dest)) return 1;
	break;
      case gPOPDCC_EXPR:
	if (edge_common(g.nodeBuffer[src].common,dest)) return 1;
	break;
      case gLABEL_EXPR:
	if (edge_common(g.nodeBuffer[src].common,dest)) return 1;
	break;
      case gGOTO_EXPR:
	if (edge_common(g.nodeBuffer[src].common,dest)) return 1;
	break;
      case gRETURN_EXPR:
	if (edge_common(g.nodeBuffer[src].common,dest)) return 1;
	break;
      case gEXIT_EXPR:
	if (edge_common(g.nodeBuffer[src].common,dest)) return 1;
	if (g.nodeBuffer[src].exit_expr.cond == dest) return 1;
	break;
      case gLOOP_EXPR:
	if (edge_common(g.nodeBuffer[src].common,dest)) return 1;
	if (g.nodeBuffer[src].loop_expr.body == dest) return 1;
	break;
      case gLABELED_BLOCK_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	break;
      case gEXPR_WITH_FILE_LOCATION:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].expr_with_file_location.expr2 == dest) return 1;
	break;
      case gSWITCH_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].switch_expr.op0 == dest) return 1;
	if (g.nodeBuffer[src].switch_expr.op1 == dest) return 1;
	break;
      case gSIZEOF_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].sizeof_expr.op0 == dest) return 1;
	break;
      case gARROW_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].arrow_expr.op0 == dest) return 1;
	break;
      case gALIGNOF_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].alignof_expr.op0 == dest) return 1;
	break;
      case gNEW_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].new_expr.op0 == dest) return 1;
	break;
      case gVEC_NEW_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].vec_new_expr.op0 == dest) return 1;
	break;
      case gDELETE_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].delete_expr.op0 == dest) return 1;
	break;
      case gVEC_DELETE_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].vec_delete_expr.op0 == dest) return 1;
	break;
      case gTYPE_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].type_expr.op0 == dest) return 1;
	break;
      case gAGGR_INIT_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].aggr_init_expr.fn == dest) return 1;
	if (g.nodeBuffer[src].aggr_init_expr.args == dest) return 1;
	if (g.nodeBuffer[src].aggr_init_expr.decl == dest) return 1;
	break;
      case gEMPTY_CLASS_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	break;
      case gTEMPLATE_ID_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].template_id_expr.op0 == dest) return 1;
	if (g.nodeBuffer[src].template_id_expr.op1 == dest) return 1;
	break;
      case gLOOKUP_EXPR:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].lookup_expr.op0 == dest) return 1;
	break;
      case gFUNCTION_NAME:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	break;
      case gSUBOBJECT:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].subobject.clnp == dest) return 1;
	if (g.nodeBuffer[src].subobject.next == dest) return 1;
	break;
      case gCTOR_STMT:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].ctor_stmt.next == dest) return 1;
	break;
      case gCTOR_INITIALIZER:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].ctor_initializer.op0 == dest) return 1;
	if (g.nodeBuffer[src].ctor_initializer.op1 == dest) return 1;
	break;
      case gRETURN_INIT:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].return_init.op0 == dest) return 1;
	if (g.nodeBuffer[src].return_init.op1 == dest) return 1;
	break;
      case gTAG_DEFN:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	break;
      case gIDENTITY_CONV:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].identity_conv.op0 == dest) return 1;
	break;
      case gLVALUE_CONV:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].lvalue_conv.op0 == dest) return 1;
	break;
      case gQUAL_CONV:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].qual_conv.op0 == dest) return 1;
	break;
      case gSTD_CONV:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].std_conv.op0 == dest) return 1;
	break;
      case gPTR_CONV:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].ptr_conv.op0 == dest) return 1;
	break;
      case gPMEM_CONV:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].pmem_conv.op0 == dest) return 1;
	break;
      case gBASE_CONV:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].base_conv.op0 == dest) return 1;
	break;
      case gREF_BIND:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].ref_bind.op0 == dest) return 1;
	break;
      case gUSER_CONV:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].user_conv.op0 == dest) return 1;
	break;
      case gAMBIG_CONV:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].ambig_conv.op0 == dest) return 1;
	break;
      case gRVALUE_CONV:
	if (edge_expr(g.nodeBuffer[src].expr,dest)) return 1;
	if (g.nodeBuffer[src].rvalue_conv.op0 == dest) return 1;
	break;
      case gOP_IDENTIFIER:
	if (edge_common(g.nodeBuffer[src].common,dest)) return 1;
	break;
      case gIDENTIFIER_NODE:
	if (edge_common(g.nodeBuffer[src].common,dest)) return 1;
	break;
      case gINTEGER_CST:
	if (edge_cst(g.nodeBuffer[src].gcc_cst,dest)) return 1;
	break;
      case gREAL_CST:
	if (edge_cst(g.nodeBuffer[src].gcc_cst,dest)) return 1;
	break;
      case gPTRMEM_CST:
	if (edge_cst(g.nodeBuffer[src].gcc_cst,dest)) return 1;
	if (g.nodeBuffer[src].ptrmem_cst.clas == dest) return 1;
	if (g.nodeBuffer[src].ptrmem_cst.mbr == dest) return 1;
	break;
      case gCOMPLEX_CST:
	if (edge_cst(g.nodeBuffer[src].gcc_cst,dest)) return 1;
	break;
      case gSTRING_CST:
	if (edge_cst(g.nodeBuffer[src].gcc_cst,dest)) return 1;
	break;
      case gTREE_LIST:
	if (edge_common(g.nodeBuffer[src].common,dest)) return 1;
	if (g.nodeBuffer[src].tree_list.purpose == dest) return 1;
	if (g.nodeBuffer[src].tree_list.value == dest) return 1;
	if (g.nodeBuffer[src].tree_list.chan == dest) return 1;
	break;
      case gTREE_VEC:
	if (g.nodeBuffer[src].tree_vec.binfo){
	  if (edge_common(g.nodeBuffer[src].common,dest)) return 1;
	  if (g.nodeBuffer[src].tree_vec.inherit == dest) return 1;
	  if (g.nodeBuffer[src].tree_vec.type == dest) return 1;
	  if (g.nodeBuffer[src].tree_vec.base == dest) return 1;
	} else {
	  if (edge_common(g.nodeBuffer[src].common,dest)) return 1;
	}
	break;
      case gERROR_MARK:
	if (edge_common(g.nodeBuffer[src].common,dest)) return 1;
	break;
      case gDEFAULT_ARG:
	if (edge_common(g.nodeBuffer[src].common,dest)) return 1;
	break;
      case gCPLUS_BINDING:
	if (edge_common(g.nodeBuffer[src].common,dest)) return 1;
	break;
      case gOVERLOAD:
	if (edge_common(g.nodeBuffer[src].common,dest)) return 1;
	if (g.nodeBuffer[src].overload.crnt == dest) return 1;
	if (g.nodeBuffer[src].overload.chan == dest) return 1;
	break;
      case gWRAPPER:
	if (edge_common(g.nodeBuffer[src].common,dest)) return 1;
	break;
      case NODE_DELTED:
      case NODE_NONE:
	return 0;
      /* datrix types */
      case dC_ASG_ND:
	if (datrix_asg_nd(g.nodeBuffer[src].dc_asg_nd,dest)) return 1;
	break;
      case dC_IDENTIFIER:
	if (datrix_identifier(g.nodeBuffer[src].dc_identifier,dest)) return 1;
	break;
      case dC_TYPE:
	if (datrix_type(g.nodeBuffer[src].dc_type,dest)) return 1;
	break;
      case dC_BUILT_IN_TYPE:
	if (datrix_type(g.nodeBuffer[src].dc_type,dest)) return 1;
	break;
      case dC_AGGR_TYPE:
	if (datrix_type(g.nodeBuffer[src].dc_type,dest)) return 1;
	if (g.nodeBuffer[src].dc_aggr_type.members == dest) return 1;
	if (g.nodeBuffer[src].dc_aggr_type.inheritance == dest) return 1;
	break;
      case dC_ENUM_TYPE:
	if (datrix_type(g.nodeBuffer[src].dc_type,dest)) return 1;
	if (g.nodeBuffer[src].dc_enum_type.values == dest) return 1;
	break;
      case dC_TEMPL_PARM_TYPE:
	if (datrix_type(g.nodeBuffer[src].dc_type,dest)) return 1;
	break;
      case dC_GENERIC_TYPE:
	if (datrix_type(g.nodeBuffer[src].dc_type,dest)) return 1;
	if (g.nodeBuffer[src].dc_generic_type.data_functs == dest) return 1;
	if (g.nodeBuffer[src].dc_generic_type.formals == dest) return 1;
	break;
      case dC_GENERATED_TYPE:
	if (datrix_type(g.nodeBuffer[src].dc_type,dest)) return 1;
	if (g.nodeBuffer[src].dc_generated_type.actuals == dest) return 1;
	break;
      case dC_ALIAS_TYPE:
	if (datrix_type(g.nodeBuffer[src].dc_type,dest)) return 1;
	if (g.nodeBuffer[src].dc_alias_type.original == dest) return 1;
	break;
      case dC_FORWARD_TYPE:
	if (datrix_type(g.nodeBuffer[src].dc_type,dest)) return 1;
	break;
      case dC_ARRAY_TYPE:
	if (datrix_type(g.nodeBuffer[src].dc_type,dest)) return 1;
	break;
      case dC_PTR_TYPE:
	if (datrix_type(g.nodeBuffer[src].dc_type,dest)) return 1;
	break;
      case dC_REF_TYPE:
	if (datrix_type(g.nodeBuffer[src].dc_type,dest)) return 1;
	break;
      case dC_FCT_TYPE:
	if (datrix_type(g.nodeBuffer[src].dc_type,dest)) return 1;
	if (g.nodeBuffer[src].dc_fct_type.formalParams == dest) return 1;
	break;
      case dC_SCOPE:
	if (datrix_scope(g.nodeBuffer[src].dc_scope,dest)) return 1;
	break;
      case dC_SYSTEM:
	if (datrix_scope(g.nodeBuffer[src].dc_scope,dest)) return 1;
	break;
      case dC_SCOPE_GLB:
	if (datrix_scope(g.nodeBuffer[src].dc_scope,dest)) return 1;
	break;
      case dC_SCOPE_COMPIL:
	if (datrix_scope(g.nodeBuffer[src].dc_scope,dest)) return 1;
	break;
      case dC_SCOPE_FILE:
	if (datrix_scope(g.nodeBuffer[src].dc_scope,dest)) return 1;
	break;
      case dC_PACKAGE:
	if (datrix_scope(g.nodeBuffer[src].dc_scope,dest)) return 1;
	break;
      case dC_NAME_SPACE:
	if (datrix_scope(g.nodeBuffer[src].dc_scope,dest)) return 1;
	break;
      case dC_BLOCK:
	if (datrix_scope(g.nodeBuffer[src].dc_scope,dest)) return 1;
	break;
      case dC_STATIC_BLOCK:
	if (datrix_scope(g.nodeBuffer[src].dc_scope,dest)) return 1;
	break;
      case dC_TRY_BLOCK:
	if (datrix_scope(g.nodeBuffer[src].dc_scope,dest)) return 1;
	break;
      case dC_CATCH_BLOCK:
	if (datrix_scope(g.nodeBuffer[src].dc_scope,dest)) return 1;
	break;
      case dC_FINALLY_BLOCK:
	if (datrix_scope(g.nodeBuffer[src].dc_scope,dest)) return 1;
	break;
      case dC_SYNCHRONIZED_BLOCK:
	if (datrix_scope(g.nodeBuffer[src].dc_scope,dest)) return 1;
	break;
      case dC_OBJECT:
	if (datrix_object(g.nodeBuffer[src].dc_object,dest)) return 1;
	break;
      case dC_FORMAL_FCT_PARAM:
	if (datrix_object(g.nodeBuffer[src].dc_object,dest)) return 1;
	break;
      case dC_FORMAL_CATCH_PARAM:
	if (datrix_object(g.nodeBuffer[src].dc_object,dest)) return 1;
	break;
      case dC_FUNCTION:
	if (datrix_identifier(g.nodeBuffer[src].dc_identifier,dest)) return 1;
	if (g.nodeBuffer[src].dc_function.formal_fct == dest) return 1;
	if (g.nodeBuffer[src].dc_function.exception == dest) return 1;
	if (g.nodeBuffer[src].dc_function.block == dest) return 1;
	break;
      case dC_LABEL:
	if (datrix_label(g.nodeBuffer[src].dc_label,dest)) return 1;
	break;
      case dC_CASE_LABEL:
	if (datrix_label(g.nodeBuffer[src].dc_label,dest)) return 1;
	break;
      case dC_DEFAULT_LABEL:
	if (datrix_label(g.nodeBuffer[src].dc_label,dest)) return 1;
	break;
      case dC_ENUMERATOR:
	if (datrix_identifier(g.nodeBuffer[src].dc_identifier,dest)) return 1;
	break;
      case dC_EXPRESSION:
	if (datrix_expression(g.nodeBuffer[src].dc_expression,dest)) return 1;
	break;
      case dC_EMPTY_EXPR:
	if (datrix_expression(g.nodeBuffer[src].dc_expression,dest)) return 1;
	break;
      case dC_LITERAL:
	if (datrix_expression(g.nodeBuffer[src].dc_expression,dest)) return 1;
	if (g.nodeBuffer[src].dc_literal.type == dest) return 1;
	break;
      case dC_SYNTAX_ERROR:
	if (datrix_expression(g.nodeBuffer[src].dc_expression,dest)) return 1;
	break;
      case dC_NAME_REF:
	if (datrix_expression(g.nodeBuffer[src].dc_expression,dest)) return 1;
	break;
      case dC_OPERATOR:
	if (datrix_operator(g.nodeBuffer[src].dc_operator,dest)) return 1;
	break;
      case dC_UNARY_OPERATOR:
	if (datrix_operator(g.nodeBuffer[src].dc_operator,dest)) return 1;
	break;
      case dC_BINARY_OPERATOR:
	if (datrix_operator(g.nodeBuffer[src].dc_operator,dest)) return 1;
	break;
      case dC_QUESTION_OPERATOR:
	if (datrix_operator(g.nodeBuffer[src].dc_operator,dest)) return 1;
	break;
      case dC_INIT_LIST:
	if (datrix_expression(g.nodeBuffer[src].dc_expression,dest)) return 1;
	if (g.nodeBuffer[src].dc_init_list.init == dest) return 1;
	break;
      case dC_ACTUAL_FCT_PARAM_LIST:
	if (datrix_expression(g.nodeBuffer[src].dc_expression,dest)) return 1;
	if (g.nodeBuffer[src].dc_actual_fct_param_list.actuals == dest) return 1;
	break;
      case dC_CREATION_EXPR:
	if (datrix_expression(g.nodeBuffer[src].dc_expression,dest)) return 1;
	if (g.nodeBuffer[src].dc_creation_expr.init == dest) return 1;
	break;
      case dC_DESTROY_EXPR:
	if (datrix_expression(g.nodeBuffer[src].dc_expression,dest)) return 1;
	if (g.nodeBuffer[src].dc_destroy_expr.what == dest) return 1;
	if (g.nodeBuffer[src].dc_destroy_expr.array == dest) return 1;
	break;
      case dC_CNTRL_STMT:
	if (datrix_ctrl_stmt(g.nodeBuffer[src].dc_ctrl_stmt,dest)) return 1;
	break;
      case dC_SELECTION:
	if (datrix_ctrl_stmt(g.nodeBuffer[src].dc_ctrl_stmt,dest)) return 1;
	break;
      case dC_IF:
	if (datrix_ctrl_stmt(g.nodeBuffer[src].dc_ctrl_stmt,dest)) return 1;
	if (g.nodeBuffer[src].dc_if.cond == dest) return 1;
	if (g.nodeBuffer[src].dc_if.true_block == dest) return 1;
	if (g.nodeBuffer[src].dc_if.false_block == dest) return 1;
	break;
      case dC_SWITCH:
	if (datrix_ctrl_stmt(g.nodeBuffer[src].dc_ctrl_stmt,dest)) return 1;
	if (g.nodeBuffer[src].dc_switch.cond == dest) return 1;
	if (g.nodeBuffer[src].dc_switch.body == dest) return 1;
	break;
      case dC_JUMP:
	if (datrix_jump(g.nodeBuffer[src].dc_jump,dest)) return 1;
	break;
      case dC_GOTO:
	if (datrix_jump(g.nodeBuffer[src].dc_jump,dest)) return 1;
	if (g.nodeBuffer[src].dc_goto.dest == dest) return 1;
	break;
      case dC_RETURN:
	if (datrix_jump(g.nodeBuffer[src].dc_jump,dest)) return 1;
	if (g.nodeBuffer[src].dc_return.value == dest) return 1;
	break;
      case dC_BREAK:
	if (datrix_jump(g.nodeBuffer[src].dc_jump,dest)) return 1;
	if (g.nodeBuffer[src].dc_break.dest == dest) return 1;
	break;
      case dC_CONTINUE:
	if (datrix_jump(g.nodeBuffer[src].dc_jump,dest)) return 1;
	if (g.nodeBuffer[src].dc_continue.dest == dest) return 1;
	break;
      case dC_LOOP:
	if (datrix_loop(g.nodeBuffer[src].dc_loop,dest)) return 1;
	break;
      case dC_DO_WHILE_LOOP:
	if (datrix_loop(g.nodeBuffer[src].dc_loop,dest)) return 1;
	if (g.nodeBuffer[src].dc_do_while_loop.cond == dest) return 1;
	if (g.nodeBuffer[src].dc_do_while_loop.block == dest) return 1;
	break;
      case dC_WHILE_DO_LOOP:
	if (datrix_loop(g.nodeBuffer[src].dc_loop,dest)) return 1;
	if (g.nodeBuffer[src].dc_while_do_loop.cond == dest) return 1;
	if (g.nodeBuffer[src].dc_while_do_loop.block == dest) return 1;
	break;
      case dC_FOR_LOOP:
	if (datrix_loop(g.nodeBuffer[src].dc_loop,dest)) return 1;
	if (g.nodeBuffer[src].dc_for_loop.init == dest) return 1;
	if (g.nodeBuffer[src].dc_for_loop.cond == dest) return 1;
	if (g.nodeBuffer[src].dc_for_loop.incr == dest) return 1;
	if (g.nodeBuffer[src].dc_for_loop.block == dest) return 1;
	break;
      case dC_SQL_STMT:
	if (datrix_asg_nd(g.nodeBuffer[src].dc_asg_nd,dest)) return 1;
	break;
      case dC_COMMENT:
	if (datrix_comment(g.nodeBuffer[src].dc_comment,dest)) return 1;
	break;
      case dC_JAVA_DOC_COMMENT:
	if (datrix_comment(g.nodeBuffer[src].dc_comment,dest)) return 1;
	break;
      case dC_THROW_SPEC:
	if (datrix_asg_nd(g.nodeBuffer[src].dc_asg_nd,dest)) return 1;
	if (g.nodeBuffer[src].dc_throw_spec.except == dest) return 1;
	break;
      case dC_ASM_BLOCK:
	if (datrix_asg_nd(g.nodeBuffer[src].dc_asg_nd,dest)) return 1;
	break;
      case dC_ARC:
	if (datrix_arc(g.nodeBuffer[src].dc_arc,dest)) return 1;
	break;
      case dC_ARC_REFERENCE:
	if (datrix_arc_reference(g.nodeBuffer[src].dc_arc_reference,dest)) return 1;
	break;
      case dC_INSTANCE:
	if (datrix_arc_reference(g.nodeBuffer[src].dc_arc_reference,dest)) return 1;
	break;
      case dC_INHERITANCE:
	if (datrix_arc_reference(g.nodeBuffer[src].dc_arc_reference,dest)) return 1;
	break;
      case dC_IMPLEMENTS:
	if (datrix_arc_reference(g.nodeBuffer[src].dc_arc_reference,dest)) return 1;
	break;
      case dC_ARC_FRIEND:
	if (datrix_arc_reference(g.nodeBuffer[src].dc_arc_reference,dest)) return 1;
	break;
      case dC_ARC_DEF_LOC:
	if (datrix_arc_reference(g.nodeBuffer[src].dc_arc_reference,dest)) return 1;
	break;
      case dC_REFERS_TO:
	if (datrix_arc_reference(g.nodeBuffer[src].dc_arc_reference,dest)) return 1;
	break;
      case dC_ARC_SYNTAX:
	if (datrix_arc_syntax(g.nodeBuffer[src].dc_arc_syntax,dest)) return 1;
	break;
      case dC_ARC_SON:
	if (datrix_arc_syntax(g.nodeBuffer[src].dc_arc_syntax,dest)) return 1;
	break;
      case dC_ARC_TRUE:
	if (datrix_arc_syntax(g.nodeBuffer[src].dc_arc_syntax,dest)) return 1;
	break;
      case dC_ARC_FALSE:
	if (datrix_arc_syntax(g.nodeBuffer[src].dc_arc_syntax,dest)) return 1;
	break;
      case dC_ARC_OPD:
	if (datrix_arc_syntax(g.nodeBuffer[src].dc_arc_syntax,dest)) return 1;
	break;
      case dC_IMPORT:
	if (datrix_arc_syntax(g.nodeBuffer[src].dc_arc_syntax,dest)) return 1;
	break;
      case dC_ARC_COND:
	if (datrix_arc_syntax(g.nodeBuffer[src].dc_arc_syntax,dest)) return 1;
	break;
      case dC_ARC_FOR_INIT:
	if (datrix_arc_syntax(g.nodeBuffer[src].dc_arc_syntax,dest)) return 1;
	break;
      case dC_ARC_FOR_INCR:
	if (datrix_arc_syntax(g.nodeBuffer[src].dc_arc_syntax,dest)) return 1;
	break;
      case dC_ARC_INIT_VAL:
	if (datrix_arc_syntax(g.nodeBuffer[src].dc_arc_syntax,dest)) return 1;
	break;
      case dC_ARC_BIT_DIM:
	if (datrix_arc_syntax(g.nodeBuffer[src].dc_arc_syntax,dest)) return 1;
	break;
      case dC_ARC_ARRAY_DIM:
	if (datrix_arc_syntax(g.nodeBuffer[src].dc_arc_syntax,dest)) return 1;
	break;
      default:
	fprintf(stderr,"node_uses: Unhandled node %d type %d(%s)\n",src,g.nodeBuffer[src].common.type,
	    gd_node_type_ascii[g.nodeBuffer[src].common.type]);
	exit(1);
  }
  return 0;
}

static int datrix_arc(t_dc_arc dc_arc, node_ptr dest){
  if (edge_common(dc_arc.common,dest)) return 1;
  if (dc_arc.src == dest) return 1;
  if (dc_arc.dest == dest) return 1;
  if (dc_arc.next == dest) return 1;
  return 0;
}

static int datrix_arc_reference(t_dc_arc_reference dc_arc_reference, node_ptr dest){
  if (datrix_arc(dc_arc_reference.arc,dest)) return 1;
  return 0;
}

static int datrix_arc_syntax(t_dc_arc_syntax dc_arc_syntax, node_ptr dest){
  if (datrix_arc(dc_arc_syntax.arc,dest)) return 1;
  return 0;
}

static int datrix_asg_nd(t_dc_asg_nd dc_asg_nd, node_ptr dest){
  if (edge_common(dc_asg_nd.common,dest)) return 1;
  return 0;
}

static int datrix_identifier(t_dc_identifier dc_identifier, node_ptr dest){
  if (datrix_asg_nd(dc_identifier.dc_asg_nd,dest)) return 1;
  if (dc_identifier.instance == dest) return 1;
  if (dc_identifier.defLoc == dest) return 1;
  if (dc_identifier.arcFriend == dest) return 1;
  if (dc_identifier.array_dim == dest) return 1;
  return 0;
}

static int datrix_expression(t_dc_expression dc_expression, node_ptr dest){
  if (datrix_asg_nd(dc_expression.dc_asg_nd,dest)) return 1;
  if (dc_expression.refers == dest) return 1;
  return 0;
}

static int datrix_operator(t_dc_operator dc_operator, node_ptr dest){
  if (datrix_expression(dc_operator.expression,dest)) return 1;
  if (dc_operator.ops == dest) return 1;
  return 0;
}

static int datrix_comment(t_dc_comment dc_comment, node_ptr dest){
  if (datrix_asg_nd(dc_comment.dc_asg_nd,dest)) return 1;
  return 0;
}

static int datrix_ctrl_stmt(t_dc_ctrl_stmt dc_ctrl_stmt, node_ptr dest){
  if (datrix_asg_nd(dc_ctrl_stmt.dc_asg_nd,dest)) return 1;
  return 0;
}

static int datrix_loop(t_dc_loop dc_loop, node_ptr dest){
  if (datrix_ctrl_stmt(dc_loop.ctrl_stmt,dest)) return 1;
  return 0;
}

static int datrix_jump(t_dc_jump dc_jump, node_ptr dest){
  if (datrix_ctrl_stmt(dc_jump.ctrl_stmt,dest)) return 1;
  return 0;
}

static int datrix_scope(t_dc_scope scope, node_ptr dest) {
  if (datrix_identifier(scope.dc_identifier,dest)) return 1;
  if (scope.contents == dest) return 1;
  return 0;
}

static int datrix_type(t_dc_type type, node_ptr dest) {
  if (datrix_identifier(type.dc_identifier,dest)) return 1;
  if(type.implements == dest) return 1;
  return 0;
}

static int datrix_label(t_dc_label label, node_ptr dest) {
  if (datrix_identifier(label.dc_identifier,dest)) return 1;
  return 0;
}

static int datrix_object(t_dc_object object, node_ptr dest) {
  if (datrix_identifier(object.dc_identifier,dest)) return 1;
  if (object.bitfield_dim == dest) return 1;
  if (object.initial_val == dest) return 1;
  return 0;
}


static int edge_decl(t_gcc_decl decl, int dest) {
  if (edge_common(decl.common,dest)) return 1;
  if (decl.name == dest) return 1;
  if (decl.mngl == dest) return 1;
  if (decl.type == dest) return 1;
  if (decl.chan == dest) return 1;
  if (decl.scpe == dest) return 1;
  return 0;
}

static int edge_common(gd_graph_common common, int dest) {
  if (common.other_node == dest) return 1;
  return 0;
}

static int edge_type(t_gcc_type type, int dest) {
  if (edge_common(type.common,dest)) return 1;
  if (type.name == dest) return 1;
  if (type.size == dest) return 1;
  if (type.unql == dest) return 1;
  return 0;
}

static int edge_cst(t_gcc_const cst, int dest) {
  if (edge_common(cst.common,dest)) return 1;
  if (cst.type == dest) return 1;
  return 0;
}

static int edge_expr(t_gcc_expr expr, int dest) {
  if (edge_common(expr.common,dest)) return 1;
  if (expr.type == dest) return 1;
  return 0;
}


char * dc_op_string[] = {
  "uplus",
  "uminus",
  "pre_inc",
  "pre_dec",
  "post_inc",
  "post_dec",
  "bplus",
  "bminus",
  "modulo",
  "mult",
  "div",
  "bit_and",
  "bit_or",
  "bit_xor",
  "rshift",
  "lshift",
  "bit_not",
  "tst_eq",
  "tst_ne",
  "tst_ge",
  "tst_le",
  "tst_gt",
  "tst_lt",
  "tst_min",
  "tst_max",
  "not",
  "xor",
  "and",
  "or",
  "asgn_eq",
  "asgn_plus",
  "asgn_minus",
  "asgn_mult",
  "asgn_div",
  "asgn_mod",
  "asgn_lshift",
  "asgn_rshift",
  "asgn_bit_and",
  "asgn_bit_or",
  "asgn_bit_xor",
  "asgn_lt",
  "asgn_gt",
  "question",
  "addr",
  "deref",
  "pointsat",
  "pointsat_deref",
  "array_ref",
  "fct_call",
  "cast_call",
  "field_access",
  "field_deref",
  "sizeof",
  "comma",
  "const_cast",
  "static_cast",
  "reint_cast",
  "dynam_cast",
  "type_id",
  "sign_of",
  "range"

};

