#include <stdio.h>
#include <stdlib.h>

#include "gd_graph.h"
#include "gd_utils.h"

void about_decl(gd_graph g, t_gcc_decl decl);
void about_common(gd_graph_common comm);
void about_type(t_gcc_type type);
void about_cst(t_gcc_const cst);
void about_expr(t_gcc_expr expr);

void datrix_asg_nd(gd_graph g, t_dc_asg_nd nd);
void datrix_identifier(gd_graph g, t_dc_identifier ident);
void datrix_loop(gd_graph g, t_dc_loop loop);
void datrix_ctrl_stmt(gd_graph g, t_dc_ctrl_stmt cntrl);
void datrix_comment(gd_graph g, t_dc_comment comment);
void datrix_jump(gd_graph g, t_dc_jump jump);
void datrix_expression(gd_graph g, t_dc_expression expr);
void datrix_operator(gd_graph g, t_dc_operator operator);
void datrix_type(gd_graph g, t_dc_type type);
void datrix_object(gd_graph g, t_dc_object object);
void datrix_label(gd_graph g, t_dc_label label);
void datrix_scope(gd_graph g, t_dc_scope scope);
void datrix_arc(t_dc_arc  dc_arc);
void datrix_arc_reference(t_dc_arc_reference  dc_arc_reference);
void datrix_arc_syntax(t_dc_arc_syntax  dc_arc_syntax);

char * progname;

void usage(){
  fprintf(stderr,"usage: %s dumpfile nodenumber\n",progname);
  fprintf(stderr,"0 < nodenumber < num_nodes_in_file\n");
  exit(1);
}

int main(int argc, char * argv[]){
  gd_graph g;
  char * filename;
  int nodenum;
  progname = argv[0];

  if (argc != 3) {
    usage();
  }

  printf("Node Size: %d\n", sizeof(gd_graph_node));

  filename = argv[1];
  nodenum = atoi(argv[2]);
  g = gd_graph_read_binary_file(filename,1.0,1.0,1.0);
  if (g.curStringPos == 0) {
    fprintf(stderr,"could not get graph\n");
    exit(1);
  }

  if (nodenum < 0 || nodenum >= g.curNodePos) {
    usage();
  }

  printf("FirstNode: %d\n", g.firstNode);
  printf("CharSize: %d\n", g.curStringPos);

  printf("Node number %d(%d)\n",nodenum, g.curNodePos);
  switch (g.nodeBuffer[nodenum].common.type) {
    case gNAMESPACE_DECL:
      printf("Type: namespace_decl\n");
      about_decl(g,g.nodeBuffer[nodenum].decl);
      printf("  dcls: @%d\n",g.nodeBuffer[nodenum].namespace_decl.dcls);
      printf("  alis: @%d\n",g.nodeBuffer[nodenum].namespace_decl.alis);
      break;
    case gFUNCTION_DECL:
      printf("Type: function_decl\n");
      about_decl(g,g.nodeBuffer[nodenum].decl);
      printf("  args: @%d\n",g.nodeBuffer[nodenum].function_decl.args);
      printf("  body: @%d\n",g.nodeBuffer[nodenum].function_decl.body);
      printf("  undef: %d\n",g.nodeBuffer[nodenum].function_decl.undef);
      printf("  ext_static: %d\n",g.nodeBuffer[nodenum].function_decl.ext_static);
      printf("  member: %d\n",g.nodeBuffer[nodenum].function_decl.member);
      printf("  constr: %d\n",g.nodeBuffer[nodenum].function_decl.constr);
      printf("  decon: %d\n",g.nodeBuffer[nodenum].function_decl.decon);
      printf("  oper: %d\n",g.nodeBuffer[nodenum].function_decl.oper);
      printf("  conver: %d\n",g.nodeBuffer[nodenum].function_decl.conver);
      printf("  ginit: %d\n",g.nodeBuffer[nodenum].function_decl.ginit);
      printf("  gfini: %d\n",g.nodeBuffer[nodenum].function_decl.gfini);
      break;
    case gVAR_DECL:
      printf("Type: var_decl\n");
      about_decl(g,g.nodeBuffer[nodenum].decl);
      printf("  init: @%d\n",g.nodeBuffer[nodenum].var_decl.init);
      printf("  size: @%d\n",g.nodeBuffer[nodenum].var_decl.size);
      printf("  align: %d\n",g.nodeBuffer[nodenum].var_decl.algn);
      printf("  used: %d\n",g.nodeBuffer[nodenum].var_decl.used);
      printf("  reg: %d\n",g.nodeBuffer[nodenum].var_decl.reg);
      break;
    case gPARM_DECL:
      printf("Type: parm_decl\n");
      about_decl(g,g.nodeBuffer[nodenum].decl);
      printf("  argt: @%d\n",g.nodeBuffer[nodenum].parm_decl.argt);
      printf("  size: @%d\n",g.nodeBuffer[nodenum].parm_decl.size);
      printf("  algn: %d\n",g.nodeBuffer[nodenum].parm_decl.algn);
      printf("  used: %d\n",g.nodeBuffer[nodenum].parm_decl.used);
      printf("  reg: %d\n",g.nodeBuffer[nodenum].parm_decl.reg);
      break;
    case gTYPE_DECL:
      printf("Type: type_decl\n");
      about_decl(g,g.nodeBuffer[nodenum].decl);
      break;
    case gLABEL_DECL:
      printf("Type: label_decl\n");
      about_decl(g,g.nodeBuffer[nodenum].decl);
      break;
    case gCONST_DECL:
      printf("Type: const_decl\n");
      about_decl(g,g.nodeBuffer[nodenum].decl);
      printf("  cnst: @%d\n",g.nodeBuffer[nodenum].const_decl.cnst);
      break;
    case gRESULT_DECL:
      printf("Type: result_decl\n");
      about_decl(g,g.nodeBuffer[nodenum].decl);
      printf("  init: @%d\n",g.nodeBuffer[nodenum].result_decl.init);
      printf("  size: @%d\n",g.nodeBuffer[nodenum].result_decl.size);
      printf("  align: %d\n",g.nodeBuffer[nodenum].result_decl.algn);
      break;
    case gFIELD_DECL:
      printf("Type: field_decl\n");
      about_decl(g,g.nodeBuffer[nodenum].decl);
      printf("  init: @%d\n",g.nodeBuffer[nodenum].field_decl.init);
      printf("  size: @%d\n",g.nodeBuffer[nodenum].field_decl.size);
      printf("  align: %d\n",g.nodeBuffer[nodenum].field_decl.algn);
      printf("  bitfield: %d\n",g.nodeBuffer[nodenum].field_decl.bitfield);
      printf("  bpos: @%d\n",g.nodeBuffer[nodenum].field_decl.bpos);
      break;
    case gTEMPLATE_DECL:
      printf("Type: template_decl\n");
      about_decl(g,g.nodeBuffer[nodenum].decl);
      printf("  rslt: @%d\n",g.nodeBuffer[nodenum].template_decl.rslt);
      printf("  inst: @%d\n",g.nodeBuffer[nodenum].template_decl.inst);
      printf("  spcs: @%d\n",g.nodeBuffer[nodenum].template_decl.spcs);
      printf("  parms: @%d\n",g.nodeBuffer[nodenum].template_decl.parms);
      break;
    case gUSING_DECL:
      printf("Type: using_decl\n");
      about_decl(g,g.nodeBuffer[nodenum].decl);
      break;
    case gFUNCTION_TYPE:
      printf("Type: function_type\n");
      about_type(g.nodeBuffer[nodenum].type);
      printf("  retn: @%d\n",g.nodeBuffer[nodenum].function_type.retn);
      printf("  prms: @%d\n",g.nodeBuffer[nodenum].function_type.prms);
      break;
    case gINTEGER_TYPE:
      printf("Type: integer_type\n");
      about_type(g.nodeBuffer[nodenum].type);
      printf("  prec: %d\n",g.nodeBuffer[nodenum].integer_type.prec);
      printf("  unsigned: %d\n",g.nodeBuffer[nodenum].integer_type.unsgnd);
      printf("  min: @%d\n",g.nodeBuffer[nodenum].integer_type.min);
      printf("  max: @%d\n",g.nodeBuffer[nodenum].integer_type.max);
      break;
    case gARRAY_TYPE:
      printf("Type: array_type\n");
      about_type(g.nodeBuffer[nodenum].type);
      printf("  elts: @%d\n",g.nodeBuffer[nodenum].array_type.elts);
      printf("  domn: @%d\n",g.nodeBuffer[nodenum].array_type.domn);
      break;
    case gVOID_TYPE:
      printf("Type: void_type\n");
      about_type(g.nodeBuffer[nodenum].type);
      break;
    case gREAL_TYPE:
      printf("Type: real_type\n");
      about_type(g.nodeBuffer[nodenum].type);
      printf("  prec: %d\n",g.nodeBuffer[nodenum].real_type.prec);
      break;
    case gCOMPLEX_TYPE:
      printf("Type: complex_type\n");
      about_type(g.nodeBuffer[nodenum].type);
      break;
    case gVECTOR_TYPE:
      printf("Type: vector_type\n");
      about_type(g.nodeBuffer[nodenum].type);
      break;
    case gENUMERAL_TYPE:
      printf("Type: enumeral_type\n");
      about_type(g.nodeBuffer[nodenum].type);
      printf("  prec: %d\n",g.nodeBuffer[nodenum].enumeral_type.prec);
      printf("  unsigned: %d\n",g.nodeBuffer[nodenum].enumeral_type.unsgnd);
      printf("  min: @%d\n",g.nodeBuffer[nodenum].enumeral_type.min);
      printf("  max: @%d\n",g.nodeBuffer[nodenum].enumeral_type.max);
      printf("  csts: @%d\n",g.nodeBuffer[nodenum].enumeral_type.csts);
      break;
    case gBOOLEAN_TYPE:
      printf("Type: boolean_type\n");
      about_type(g.nodeBuffer[nodenum].type);
      break;
    case gCHAR_TYPE:
      printf("Type: char_type\n");
      about_type(g.nodeBuffer[nodenum].type);
      break;
    case gPOINTER_TYPE:
      printf("Type: pointer_type\n");
      about_type(g.nodeBuffer[nodenum].type);
      printf("  ptrmem: %d\n",g.nodeBuffer[nodenum].pointer_type.ptrmem);
      printf("  ptd: @%d\n",g.nodeBuffer[nodenum].pointer_type.ptd);
      printf("  cls: @%d\n",g.nodeBuffer[nodenum].pointer_type.cls);
      break;
    case gOFFSET_TYPE:
      printf("Type: offset_type\n");
      about_type(g.nodeBuffer[nodenum].type);
      break;
    case gREFERENCE_TYPE:
      printf("Type: reference_type\n");
      about_type(g.nodeBuffer[nodenum].type);
      printf("  refd: @%d\n",g.nodeBuffer[nodenum].reference_type.refd);
      break;
    case gMETHOD_TYPE:
      printf("Type: method_type\n");
      about_type(g.nodeBuffer[nodenum].type);
      printf("  clas: @%d\n",g.nodeBuffer[nodenum].method_type.clas);
      printf("  retn: @%d\n",g.nodeBuffer[nodenum].method_type.retn);
      printf("  prms: @%d\n",g.nodeBuffer[nodenum].method_type.prms);
      break;
    case gFILE_TYPE:
      printf("Type: file_type\n");
      about_type(g.nodeBuffer[nodenum].type);
      break;
    case gSET_TYPE:
      printf("Type: set_type\n");
      about_type(g.nodeBuffer[nodenum].type);
      break;
    case gRECORD_TYPE:
      printf("Type: record_type\n");
      about_type(g.nodeBuffer[nodenum].type);
      printf("  flds: @%d\n",g.nodeBuffer[nodenum].record_type.flds);
      printf("  fncs: @%d\n",g.nodeBuffer[nodenum].record_type.fncs);
      printf("  ptd: @%d\n",g.nodeBuffer[nodenum].record_type.ptd);
      printf("  cls: @%d\n",g.nodeBuffer[nodenum].record_type.cls);
      printf("  vfld: @%d\n",g.nodeBuffer[nodenum].record_type.vfld);
      printf("  ptrmem: %d\n",g.nodeBuffer[nodenum].record_type.ptrmem);
      printf("  isclass: %d\n",g.nodeBuffer[nodenum].record_type.isClass);
      printf("  binfo: @%d\n",g.nodeBuffer[nodenum].record_type.binf);
      break;
    case gUNION_TYPE:
      printf("Type: union_type\n");
      about_type(g.nodeBuffer[nodenum].type);
      printf("  flds: @%d\n",g.nodeBuffer[nodenum].union_type.flds);
      printf("  fncs: @%d\n",g.nodeBuffer[nodenum].union_type.fncs);
      printf("  ptd: @%d\n",g.nodeBuffer[nodenum].union_type.ptd);
      printf("  cls: @%d\n",g.nodeBuffer[nodenum].union_type.cls);
      printf("  vfld: @%d\n",g.nodeBuffer[nodenum].union_type.vfld);
      printf("  ptrmem: %d\n",g.nodeBuffer[nodenum].union_type.ptrmem);
      printf("  binfo: @%d\n",g.nodeBuffer[nodenum].union_type.binf);
      break;
    case gQUAL_UNION_TYPE:
      printf("Type: qual_union_type\n");
      about_type(g.nodeBuffer[nodenum].type);
      break;
    case gLANG_TYPE:
      printf("Type: lang_type\n");
      about_type(g.nodeBuffer[nodenum].type);
      break;
    case gTEMPLATE_TYPE_PARM:
      printf("Type: tmplate_type_parm\n");
      about_type(g.nodeBuffer[nodenum].type);
      break;
    case gTEMPLATE_PARM_INDEX:
      printf("Type: tmplate_parm_index\n");
      about_common(g.nodeBuffer[nodenum].common);
      break;
    case gTEMPLATE_TEMPLATE_PARM:
      printf("Type: tmplate_template_parm\n");
      about_type(g.nodeBuffer[nodenum].type);
      break;
    case gBOUND_TEMPLATE_TEMPLATE_PARM:
      printf("Type: bound_tmplate_template_parm\n");
      about_type(g.nodeBuffer[nodenum].type);
      break;
    case gTYPENAME_TYPE:
      printf("Type: typename_type\n");
      about_type(g.nodeBuffer[nodenum].type);
      break;
    case gTYPEOF_TYPE:
      printf("Type: typeof_type\n");
      about_type(g.nodeBuffer[nodenum].type);
      break;
    case gBLOCK:
      printf("Type: block\n");
      about_common(g.nodeBuffer[nodenum].common);
      break;
    case gCOMPOUND_STMT:
      printf("Type: compound_stmt\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  line: %d\n", g.nodeBuffer[nodenum].compound_stmt.line);
      printf("  body: @%d\n", g.nodeBuffer[nodenum].compound_stmt.body);
      printf("  next: @%d\n", g.nodeBuffer[nodenum].compound_stmt.next);
      break;
    case gEXPR_STMT:
      printf("Type: expr_stmt\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  line: %d\n", g.nodeBuffer[nodenum].expr_stmt.line);
      printf("  expr: @%d\n", g.nodeBuffer[nodenum].expr_stmt.texpr);
      printf("  next: @%d\n", g.nodeBuffer[nodenum].expr_stmt.next);
      break;
    case gDECL_STMT:
      printf("Type: decl_stmt\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  line: %d\n", g.nodeBuffer[nodenum].decl_stmt.line);
      printf("  decl: @%d\n", g.nodeBuffer[nodenum].decl_stmt.decl);
      printf("  next: @%d\n", g.nodeBuffer[nodenum].decl_stmt.next);
      break;
    case gSCOPE_STMT:
      printf("Type: scope_stmt\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  line: %d\n", g.nodeBuffer[nodenum].scope_stmt.line);
      printf("  begin_end: %d\n", g.nodeBuffer[nodenum].scope_stmt.begin_end);
      printf("  null: %d\n", g.nodeBuffer[nodenum].scope_stmt.null);
      printf("  clnp: %d\n", g.nodeBuffer[nodenum].scope_stmt.clnp);
      printf("  next: @%d\n", g.nodeBuffer[nodenum].scope_stmt.next);
      break;
    case gRETURN_STMT:
      printf("Type: return_stmt\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  line: %d\n", g.nodeBuffer[nodenum].return_stmt.line);
      printf("  expr: @%d\n", g.nodeBuffer[nodenum].return_stmt.ret_expr);
      printf("  next: @%d\n", g.nodeBuffer[nodenum].return_stmt.next);
      break;
    case gSTMT_EXPR:
      printf("Type: stmt_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  elts: @%d\n", g.nodeBuffer[nodenum].stmt_expr.stmt);
      break;
    case gBREAK_STMT:
      printf("Type: break_stmt\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  line: %d\n", g.nodeBuffer[nodenum].break_stmt.line);
      printf("  next: @%d\n", g.nodeBuffer[nodenum].break_stmt.next);
      break;
    case gCONTINUE_STMT:
      printf("Type: continue_stmt\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  line: %d\n", g.nodeBuffer[nodenum].continue_stmt.line);
      printf("  next: @%d\n", g.nodeBuffer[nodenum].continue_stmt.next);
      break;
    case gIF_STMT:
      printf("Type: if_stmt\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  line: %d\n", g.nodeBuffer[nodenum].if_stmt.line);
      printf("  cond: @%d\n", g.nodeBuffer[nodenum].if_stmt.cond);
      printf("  thenc: @%d\n", g.nodeBuffer[nodenum].if_stmt.thenc);
      printf("  elsec: @%d\n", g.nodeBuffer[nodenum].if_stmt.elsec);
      printf("  next: @%d\n", g.nodeBuffer[nodenum].if_stmt.next);
      break;
    case gSWITCH_STMT:
      printf("Type: switch_stmt\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  line: %d\n", g.nodeBuffer[nodenum].switch_stmt.line);
      printf("  cond: @%d\n", g.nodeBuffer[nodenum].switch_stmt.cond);
      printf("  body: @%d\n", g.nodeBuffer[nodenum].switch_stmt.body);
      printf("  next: @%d\n", g.nodeBuffer[nodenum].switch_stmt.next);
      break;
    case gLABEL_STMT:
      printf("Type: label_stmt\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  line: %d\n", g.nodeBuffer[nodenum].label_stmt.line);
      printf("  label: @%d\n", g.nodeBuffer[nodenum].label_stmt.label);
      printf("  next: @%d\n", g.nodeBuffer[nodenum].label_stmt.next);
      break;
    case gTRY_BLOCK:
      printf("Type: try_block\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  line: %d\n", g.nodeBuffer[nodenum].try_block.line);
      printf("  cleanup: %d\n", g.nodeBuffer[nodenum].try_block.cleanup);
      printf("  body: @%d\n", g.nodeBuffer[nodenum].try_block.body);
      printf("  hdlr: @%d\n", g.nodeBuffer[nodenum].try_block.hdlr);
      printf("  next: @%d\n", g.nodeBuffer[nodenum].try_block.next);
      break;
    case gHANDLER:
      printf("Type: handler\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  line: %d\n", g.nodeBuffer[nodenum].handler.line);
      printf("  body: @%d\n", g.nodeBuffer[nodenum].handler.body);
      printf("  next: @%d\n", g.nodeBuffer[nodenum].handler.next);
      break;
    case gSTART_CATCH_STMT:
      printf("Type: start_catch_stmt\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  line: %d\n", g.nodeBuffer[nodenum].start_catch_stmt.line);
      printf("  catch_type: @%d\n", g.nodeBuffer[nodenum].start_catch_stmt.catch_type);
      printf("  next: @%d\n", g.nodeBuffer[nodenum].start_catch_stmt.next);
      break;
    case gCLEANUP_STMT:
      printf("Type: cleanup_stmt\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  line: %d\n", g.nodeBuffer[nodenum].cleanup_stmt.line);
      printf("  decl: @%d\n", g.nodeBuffer[nodenum].cleanup_stmt.decl);
      printf("  expr: @%d\n", g.nodeBuffer[nodenum].cleanup_stmt.clean_expr);
      printf("  next: @%d\n", g.nodeBuffer[nodenum].cleanup_stmt.next);
      break;
    case gCASE_LABEL:
      printf("Type: case_label\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  low: @%d\n", g.nodeBuffer[nodenum].case_label.low);
      printf("  high: @%d\n", g.nodeBuffer[nodenum].case_label.high);
      printf("  next: @%d\n", g.nodeBuffer[nodenum].case_label.next);
      break;
    case gGOTO_STMT:
      printf("Type: goto_stmt\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  line: %d\n", g.nodeBuffer[nodenum].goto_stmt.line);
      printf("  dest: @%d\n", g.nodeBuffer[nodenum].goto_stmt.dest);
      printf("  next: @%d\n", g.nodeBuffer[nodenum].goto_stmt.next);
      break;
    case gFOR_STMT:
      printf("Type: for_stmt\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  line: %d\n", g.nodeBuffer[nodenum].for_stmt.line);
      printf("  init: @%d\n", g.nodeBuffer[nodenum].for_stmt.init);
      printf("  cond: @%d\n", g.nodeBuffer[nodenum].for_stmt.cond);
      printf("  expr: @%d\n", g.nodeBuffer[nodenum].for_stmt.inc_expr);
      printf("  body: @%d\n", g.nodeBuffer[nodenum].for_stmt.body);
      printf("  next: @%d\n", g.nodeBuffer[nodenum].for_stmt.next);
      break;
    case gWHILE_STMT:
      printf("Type: while_stmt\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  line: %d\n", g.nodeBuffer[nodenum].while_stmt.line);
      printf("  cond: @%d\n", g.nodeBuffer[nodenum].while_stmt.cond);
      printf("  body: @%d\n", g.nodeBuffer[nodenum].while_stmt.body);
      printf("  next: @%d\n", g.nodeBuffer[nodenum].while_stmt.next);
      break;
    case gDO_STMT:
      printf("Type: do_stmt\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  line: %d\n", g.nodeBuffer[nodenum].do_stmt.line);
      printf("  cond: @%d\n", g.nodeBuffer[nodenum].do_stmt.cond);
      printf("  body: @%d\n", g.nodeBuffer[nodenum].do_stmt.body);
      printf("  next: @%d\n", g.nodeBuffer[nodenum].do_stmt.next);
      break;
    case gASM_STMT:
      printf("Type: asm_stmt\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  line: %d\n", g.nodeBuffer[nodenum].asm_stmt.line);
      printf("  volitile: %d\n", g.nodeBuffer[nodenum].asm_stmt.vol);
      printf("  strg: @%d\n", g.nodeBuffer[nodenum].asm_stmt.strg);
      printf("  outs: @%d\n", g.nodeBuffer[nodenum].asm_stmt.outs);
      printf("  ins: @%d\n", g.nodeBuffer[nodenum].asm_stmt.ins);
      printf("  clbr: @%d\n", g.nodeBuffer[nodenum].asm_stmt.clbr);
      printf("  next: @%d\n", g.nodeBuffer[nodenum].asm_stmt.next);
      break;
    case gCOMPONENT_REF:
      printf("Type: component_ref\n");
      about_common(g.nodeBuffer[nodenum].common);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].component_ref.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].component_ref.op1);
      break;
    case gBIT_FIELD_REF:
      printf("Type: bit_field_ref\n");
      about_common(g.nodeBuffer[nodenum].common);
      break;
    case gINDIRECT_REF:
      printf("Type: indirect_ref\n");
      about_common(g.nodeBuffer[nodenum].common);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].indirect_ref.op0);
      break;
    case gBUFFER_REF:
      printf("Type: buffer_ref\n");
      about_common(g.nodeBuffer[nodenum].common);
      break;
    case gARRAY_REF:
      printf("Type: array_ref\n");
      about_common(g.nodeBuffer[nodenum].common);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].array_ref.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].array_ref.op1);
      break;
    case gOFFSET_REF:
      printf("Type: offset_ref\n");
      about_common(g.nodeBuffer[nodenum].common);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].offset_ref.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].offset_ref.op1);
      break;
    case gSCOPE_REF:
      printf("Type: scope_ref\n");
      about_common(g.nodeBuffer[nodenum].common);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].scope_ref.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].scope_ref.op1);
      break;
    case gMEMBER_REF:
      printf("Type: member_ref\n");
      about_common(g.nodeBuffer[nodenum].common);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].member_ref.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].member_ref.op1);
      break;
    case gCONSTRUCTOR:
      printf("Type: constructor\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  elts: @%d\n", g.nodeBuffer[nodenum].constructor.elts);
      break;
    case gCOMPOUND_EXPR:
      printf("Type: compound_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].compound_expr.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].compound_expr.op1);
      break;
    case gMODIFY_EXPR:
      printf("Type: modify_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].modify_expr.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].modify_expr.op1);
      break;
    case gINIT_EXPR:
      printf("Type: init_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].init_expr.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].init_expr.op1);
      break;
    case gTHROW_EXPR:
      printf("Type: throw_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].throw_expr.op0);
      break;
    case gTARGET_EXPR:
      printf("Type: target_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  decl: @%d\n", g.nodeBuffer[nodenum].target_expr.decl);
      printf("  init: @%d\n", g.nodeBuffer[nodenum].target_expr.init);
      printf("  clnp: @%d\n", g.nodeBuffer[nodenum].target_expr.clnp);
      break;
    case gCOND_EXPR:
      printf("Type: cond_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].cond_expr.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].cond_expr.op1);
      printf("  op2: @%d\n", g.nodeBuffer[nodenum].cond_expr.op2);
      break;
    case gBIND_EXPR:
      printf("Type: bind_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  expr: @%d\n", g.nodeBuffer[nodenum].bind_expr.expr);
      printf("  vars: @%d\n", g.nodeBuffer[nodenum].bind_expr.vars);
      printf("  body: @%d\n", g.nodeBuffer[nodenum].bind_expr.body);
      break;
    case gCALL_EXPR:
      printf("Type: call_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  fn: @%d\n", g.nodeBuffer[nodenum].call_expr.fn);
      printf("  args: @%d\n", g.nodeBuffer[nodenum].call_expr.args);
      break;
    case gMETHOD_CALL_EXPR:
      printf("Type: call_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      break;
    case gWITH_CLEANUP_EXPR:
      printf("Type: with_cleanup_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      break;
    case gCLEANUP_POINT_EXPR:
      printf("Type: cleanup_point_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      break;
    case gPLACEHOLDER_EXPR:
      printf("Type: placeholder_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      break;
    case gWITH_RECORD_EXPR:
      printf("Type: with_record_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      break;
    case gPLUS_EXPR:
      printf("Type: plus_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].plus_expr.expr2.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].plus_expr.expr2.op1);
      break;
    case gMINUS_EXPR:
      printf("Type: minus_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].minus_expr.expr2.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].minus_expr.expr2.op1);
      break;
    case gMULT_EXPR:
      printf("Type: mult_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].mult_expr.expr2.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].mult_expr.expr2.op1);
      break;
    case gTRUNC_DIV_EXPR:
      printf("Type: trunc_div_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].trunc_div_expr.expr2.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].trunc_div_expr.expr2.op1);
      break;
    case gCEIL_DIV_EXPR:
      printf("Type: ceil_div_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].ceil_div_expr.expr2.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].ceil_div_expr.expr2.op1);
      break;
    case gFLOOR_DIV_EXPR:
      printf("Type: floor_div_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].floor_div_expr.expr2.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].floor_div_expr.expr2.op1);
      break;
    case gROUND_DIV_EXPR:
      printf("Type: round_div_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].round_div_expr.expr2.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].round_div_expr.expr2.op1);
      break;
    case gTRUNC_MOD_EXPR:
      printf("Type: trunc_mod_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].trunc_mod_expr.expr2.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].trunc_mod_expr.expr2.op1);
      break;
    case gCEIL_MOD_EXPR:
      printf("Type: ceil_mod_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].ceil_mod_expr.expr2.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].ceil_mod_expr.expr2.op1);
      break;
    case gFLOOR_MOD_EXPR:
      printf("Type: floor_mod_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].floor_mod_expr.expr2.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].floor_mod_expr.expr2.op1);
      break;
    case gROUND_MOD_EXPR:
      printf("Type: round_mod_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].round_mod_expr.expr2.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].round_mod_expr.expr2.op1);
      break;
    case gRDIV_EXPR:
      printf("Type: rdiv_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].rdiv_expr.expr2.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].rdiv_expr.expr2.op1);
      break;
    case gEXACT_DIV_EXPR:
      printf("Type: exact_div_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].exact_div_expr.expr2.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].exact_div_expr.expr2.op1);
      break;
    case gFIX_TRUNC_EXPR:
      printf("Type: fix_trunc_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].fix_trunc_expr.expr1.op0);
      break;
    case gFIX_CEIL_EXPR:
      printf("Type: fix_ceil_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].fix_ceil_expr.expr1.op0);
      break;
    case gFIX_FLOOR_EXPR:
      printf("Type: fix_floor_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].fix_floor_expr.expr1.op0);
      break;
    case gFIX_ROUND_EXPR:
      printf("Type: fix_round_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].fix_round_expr.expr1.op0);
      break;
    case gFLOAT_EXPR:
      printf("Type: float_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].float_expr.expr1.op0);
      break;
    case gEXPON_EXPR:
      printf("Type: expon_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].expon_expr.expr2.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].expon_expr.expr2.op1);
      break;
    case gNEGATE_EXPR:
      printf("Type: negate_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].negate_expr.expr1.op0);
      break;
    case gMIN_EXPR:
      printf("Type: min_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].min_expr.expr2.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].min_expr.expr2.op1);
      break;
    case gMAX_EXPR:
      printf("Type: max_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].max_expr.expr2.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].max_expr.expr2.op1);
      break;
    case gABS_EXPR:
      printf("Type: abs_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].abs_expr.expr1.op0);
      break;
    case gFFS_EXPR:
      printf("Type: ffs_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].ffs_expr.expr1.op0);
      break;
    case gLSHIFT_EXPR:
      printf("Type: lshift_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].lshift_expr.expr2.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].lshift_expr.expr2.op1);
      break;
    case gRSHIFT_EXPR:
      printf("Type: rshift_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].rshift_expr.expr2.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].rshift_expr.expr2.op1);
      break;
    case gLROTATE_EXPR:
      printf("Type: lrotate_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].lrotate_expr.expr2.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].lrotate_expr.expr2.op1);
      break;
    case gRROTATE_EXPR:
      printf("Type: rrotate_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].rrotate_expr.expr2.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].rrotate_expr.expr2.op1);
      break;
    case gBIT_IOR_EXPR:
      printf("Type: bit_ior_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].bit_ior_expr.expr2.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].bit_ior_expr.expr2.op1);
      break;
    case gBIT_XOR_EXPR:
      printf("Type: bit_xor_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].bit_xor_expr.expr2.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].bit_xor_expr.expr2.op1);
      break;
    case gBIT_AND_EXPR:
      printf("Type: bit_and_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].bit_and_expr.expr2.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].bit_and_expr.expr2.op1);
      break;
    case gBIT_ANDTC_EXPR:
      printf("Type: bit_andtc_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].bit_andtc_expr.expr2.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].bit_andtc_expr.expr2.op1);
      break;
    case gTRUTH_ANDIF_EXPR:
      printf("Type: truth_andif_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].truth_andif_expr.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].truth_andif_expr.op1);
      break;
    case gTRUTH_ORIF_EXPR:
      printf("Type: truth_orif_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].truth_orif_expr.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].truth_orif_expr.op1);
      break;
    case gTRUTH_AND_EXPR:
      printf("Type: truth_and_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].truth_and_expr.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].truth_and_expr.op1);
      break;
    case gTRUTH_OR_EXPR:
      printf("Type: truth_or_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].truth_or_expr.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].truth_or_expr.op1);
      break;
    case gTRUTH_XOR_EXPR:
      printf("Type: truth_xor_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].truth_xor_expr.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].truth_xor_expr.op1);
      break;
    case gTRUTH_NOT_EXPR:
      printf("Type: truth_not_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].truth_not_expr.op0);
      break;
    case gLT_EXPR:
      printf("Type: lt_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].lt_expr.expr_ineq.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].lt_expr.expr_ineq.op1);
      break;
    case gLE_EXPR:
      printf("Type: le_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].le_expr.expr_ineq.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].le_expr.expr_ineq.op1);
      break;
    case gGT_EXPR:
      printf("Type: gt_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].gt_expr.expr_ineq.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].gt_expr.expr_ineq.op1);
      break;
    case gGE_EXPR:
      printf("Type: ge_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].ge_expr.expr_ineq.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].ge_expr.expr_ineq.op1);
      break;
    case gEQ_EXPR:
      printf("Type: eq_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].eq_expr.expr_ineq.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].eq_expr.expr_ineq.op1);
      break;
    case gNE_EXPR:
      printf("Type: ne_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].ne_expr.expr_ineq.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].ne_expr.expr_ineq.op1);
      break;
    case gUNORDERED_EXPR:
      printf("Type: unordered_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].unordered_expr.expr_ineq.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].unordered_expr.expr_ineq.op1);
      break;
    case gORDERED_EXPR:
      printf("Type: ordered_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].ordered_expr.expr_ineq.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].ordered_expr.expr_ineq.op1);
      break;
    case gUNLT_EXPR:
      printf("Type: unlt_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].unlt_expr.expr_ineq.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].unlt_expr.expr_ineq.op1);
      break;
    case gUNLE_EXPR:
      printf("Type: unle_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].unle_expr.expr_ineq.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].unle_expr.expr_ineq.op1);
      break;
    case gUNGT_EXPR:
      printf("Type: ungt_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].ungt_expr.expr_ineq.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].ungt_expr.expr_ineq.op1);
      break;
    case gUNGE_EXPR:
      printf("Type: unge_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].unge_expr.expr_ineq.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].unge_expr.expr_ineq.op1);
      break;
    case gUNEQ_EXPR:
      printf("Type: uneq_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].uneq_expr.expr_ineq.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].uneq_expr.expr_ineq.op1);
      break;
    case gIN_EXPR:
      printf("Type: in_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].in_expr.expr2.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].in_expr.expr2.op1);
      break;
    case gSET_LE_EXPR:
      printf("Type: set_le_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].set_le_expr.expr_ineq.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].set_le_expr.expr_ineq.op1);
      break;
    case gCARD_EXPR:
      printf("Type: card_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].card_expr.expr1.op0);
      break;
    case gRANGE_EXPR:
      printf("Type: range_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].range_expr.expr2.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].range_expr.expr2.op1);
      break;
    case gCONVERT_EXPR:
      printf("Type: convert_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].convert_expr.expr1.op0);
      break;
    case gNOP_EXPR:
      printf("Type: nop_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].nop_expr.expr1.op0);
      break;
    case gNON_LVALUE_EXPR:
      printf("Type: non_lvalue_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].non_lvalue_expr.expr1.op0);
      break;
    case gSAVE_EXPR:
      printf("Type: save_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].save_expr.op0);
      break;
    case gUNSAVE_EXPR:
      printf("Type: unsave_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      break;
    case gRTL_EXPR:
      printf("Type: rtl_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      break;
    case gADDR_EXPR:
      printf("Type: addr_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].addr_expr.op0);
      break;
    case gREFERENCE_EXPR:
      printf("Type: reference_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      break;
    case gENTRY_VALUE_EXPR:
      printf("Type: entry_value_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      break;
    case gCOMPLEX_EXPR:
      printf("Type: complex_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].complex_expr.expr2.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].complex_expr.expr2.op1);
      break;
    case gCONJ_EXPR:
      printf("Type: conj_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].conj_expr.expr1.op0);
      break;
    case gREALPART_EXPR:
      printf("Type: realpart_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].realpart_expr.expr1.op0);
      break;
    case gPREDECREMENT_EXPR:
      printf("Type: predecrement_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].predecrement_expr.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].predecrement_expr.op1);
      break;
    case gPREINCREMENT_EXPR:
      printf("Type: preincrement_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].preincrement_expr.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].preincrement_expr.op1);
      break;
    case gPOSTDECREMENT_EXPR:
      printf("Type: postdecrement_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].postdecrement_expr.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].postdecrement_expr.op1);
      break;
    case gPOSTINCREMENT_EXPR:
      printf("Type: postincrement_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].postincrement_expr.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].postincrement_expr.op1);
      break;
    case gVA_ARG_EXPR:
      printf("Type: va_arg_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].va_arg_expr.op0);
      break;
    case gTRY_CATCH_EXPR:
      printf("Type: try_catch_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].try_catch_expr.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].try_catch_expr.op1);
      break;
    case gTRY_FINALLY_EXPR:
      printf("Type: try_finally_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].try_finally_expr.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].try_finally_expr.op1);
      break;
    case gGOTO_SUBROUTINE_EXPR:
      printf("Type: goto_subroutine_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].goto_subroutine_expr.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].goto_subroutine_expr.op1);
      break;
    case gPOPDHC_EXPR:
      printf("Type: popdhc_expr\n");
      about_common(g.nodeBuffer[nodenum].common);
      break;
    case gPOPDCC_EXPR:
      printf("Type: popdcc_expr\n");
      about_common(g.nodeBuffer[nodenum].common);
      break;
    case gLABEL_EXPR:
      printf("Type: label_expr\n");
      about_common(g.nodeBuffer[nodenum].common);
      break;
    case gGOTO_EXPR:
      printf("Type: goto_expr\n");
      about_common(g.nodeBuffer[nodenum].common);
      break;
    case gRETURN_EXPR:
      printf("Type: return_expr\n");
      about_common(g.nodeBuffer[nodenum].common);
      break;
    case gEXIT_EXPR:
      printf("Type: exit_expr\n");
      about_common(g.nodeBuffer[nodenum].common);
      printf("  cond: @%d\n", g.nodeBuffer[nodenum].exit_expr.cond);
      break;
    case gLOOP_EXPR:
      printf("Type: loop_expr\n");
      about_common(g.nodeBuffer[nodenum].common);
      printf("  cond: @%d\n", g.nodeBuffer[nodenum].loop_expr.body);
      break;
    case gLABELED_BLOCK_EXPR:
      printf("Type: labeled_block_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      break;
    case gEXIT_BLOCK_EXPR:
      printf("Type: exit_block_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      break;
    case gEXPR_WITH_FILE_LOCATION:
      printf("Type: expr_with_file_location\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  expr2: @%d\n", g.nodeBuffer[nodenum].expr_with_file_location.expr2);
      break;
    case gSWITCH_EXPR:
      printf("Type: switch_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].switch_expr.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].switch_expr.op1);
      break;
    case gSIZEOF_EXPR:
      printf("Type: sizeof_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].sizeof_expr.op0);
      break;
    case gARROW_EXPR:
      printf("Type: arrow_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].arrow_expr.op0);
      break;
    case gALIGNOF_EXPR:
      printf("Type: alignof_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].alignof_expr.op0);
      break;
    case gNEW_EXPR:
      printf("Type: new_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].new_expr.op0);
      break;
    case gVEC_NEW_EXPR:
      printf("Type: vec_new_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].vec_new_expr.op0);
      break;
    case gDELETE_EXPR:
      printf("Type: delete_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].delete_expr.op0);
      break;
    case gVEC_DELETE_EXPR:
      printf("Type: vec_delete_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].vec_delete_expr.op0);
      break;
    case gTYPE_EXPR:
      printf("Type: type_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].type_expr.op0);
      break;
    case gAGGR_INIT_EXPR:
      printf("Type: aggr_init_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  ctor: %d\n", g.nodeBuffer[nodenum].aggr_init_expr.ctor);
      printf("  fn: @%d\n", g.nodeBuffer[nodenum].aggr_init_expr.fn);
      printf("  args: @%d\n", g.nodeBuffer[nodenum].aggr_init_expr.args);
      printf("  decl: @%d\n", g.nodeBuffer[nodenum].aggr_init_expr.decl);
      break;
    case gEMPTY_CLASS_EXPR:
      printf("Type: empty_class_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      break;
    case gTEMPLATE_ID_EXPR:
      printf("Type: template_id_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].template_id_expr.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].template_id_expr.op1);
      break;
    case gLOOKUP_EXPR:
      printf("Type: lookup_expr\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].lookup_expr.op0);
      break;
    case gFUNCTION_NAME:
      printf("Type: function_name\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      break;
    case gSUBOBJECT:
      printf("Type: subobject\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  line: %d\n", g.nodeBuffer[nodenum].subobject.line);
      printf("  clnp: @%d\n", g.nodeBuffer[nodenum].subobject.clnp);
      printf("  next: @%d\n", g.nodeBuffer[nodenum].subobject.next);
      break;
    case gCTOR_STMT:
      printf("Type: ctor_stmt\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  line: %d\n", g.nodeBuffer[nodenum].ctor_stmt.line);
      printf("  beg_end: %d\n", g.nodeBuffer[nodenum].ctor_stmt.beg_end);
      printf("  next: @%d\n", g.nodeBuffer[nodenum].ctor_stmt.next);
      break;
    case gCTOR_INITIALIZER:
      printf("Type: ctor_initializer\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].ctor_initializer.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].ctor_initializer.op1);
      break;
    case gRETURN_INIT:
      printf("Type: return_init\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].return_init.op0);
      printf("  op1: @%d\n", g.nodeBuffer[nodenum].return_init.op1);
      break;
    case gTAG_DEFN:
      printf("Type: tag_defn\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      break;
    case gIDENTITY_CONV:
      printf("Type: identity_conv\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].identity_conv.op0);
      break;
    case gLVALUE_CONV:
      printf("Type: lvalue_conv\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].lvalue_conv.op0);
      break;
    case gQUAL_CONV:
      printf("Type: qual_conv\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].qual_conv.op0);
      break;
    case gSTD_CONV:
      printf("Type: std_conv\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].std_conv.op0);
      break;
    case gPTR_CONV:
      printf("Type: ptr_conv\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].ptr_conv.op0);
      break;
    case gPMEM_CONV:
      printf("Type: pmem_conv\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].pmem_conv.op0);
      break;
    case gBASE_CONV:
      printf("Type: base_conv\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].base_conv.op0);
      break;
    case gREF_BIND:
      printf("Type: ref_bind\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].ref_bind.op0);
      break;
    case gUSER_CONV:
      printf("Type: user_conv\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].user_conv.op0);
      break;
    case gRVALUE_CONV:
      printf("Type: rvalue_conv\n");
      about_expr(g.nodeBuffer[nodenum].expr);
      printf("  op0: @%d\n", g.nodeBuffer[nodenum].rvalue_conv.op0);
      break;
    case gOP_IDENTIFIER:
      printf("Type: op_identifier\n");
      about_common(g.nodeBuffer[nodenum].common);
      break;
    case gIDENTIFIER_NODE:
      printf("Type: identifier_node\n");
      about_common(g.nodeBuffer[nodenum].common);
      printf("  lngt: %d\n", g.nodeBuffer[nodenum].identifier_node.lngt);
      printf("  strg: %s(%d)\n", &(g.stringBuffer[g.nodeBuffer[nodenum].identifier_node.strg]),g.nodeBuffer[nodenum].identifier_node.strg);
      printf("  opt_type_un: %d\n", g.nodeBuffer[nodenum].identifier_node.op_type_un);
      printf("  tynm: %d\n", g.nodeBuffer[nodenum].identifier_node.tynm);
      break;
    case gINTEGER_CST:
      printf("Type: integer_cst\n");
      about_cst(g.nodeBuffer[nodenum].gcc_cst);
      printf("  high: %d\n", g.nodeBuffer[nodenum].integer_cst.high);
      printf("  low: %d\n", g.nodeBuffer[nodenum].integer_cst.low);
      break;
    case gREAL_CST:
      printf("Type: real_cst\n");
      about_cst(g.nodeBuffer[nodenum].gcc_cst);
      printf("  val: %s\n", &(g.stringBuffer[g.nodeBuffer[nodenum].real_cst.value]));
      break;
    case gPTRMEM_CST:
      printf("Type: ptrmem_cst\n");
      about_cst(g.nodeBuffer[nodenum].gcc_cst);
      printf("  clas: @%d\n", g.nodeBuffer[nodenum].ptrmem_cst.clas);
      printf("  mbr: @%d\n", g.nodeBuffer[nodenum].ptrmem_cst.mbr);
      break;
    case gCOMPLEX_CST:
      printf("Type: complex_cst\n");
      about_cst(g.nodeBuffer[nodenum].gcc_cst);
      break;
    case gSTRING_CST:
      printf("Type: string_cst\n");
      about_cst(g.nodeBuffer[nodenum].gcc_cst);
      printf("  lngt: %d\n", g.nodeBuffer[nodenum].string_cst.lngt);
      printf("  strg: %s\n", &(g.stringBuffer[g.nodeBuffer[nodenum].string_cst.strg]));
      break;
    case gTREE_LIST:
      printf("Type: tree_list\n");
      about_common(g.nodeBuffer[nodenum].common);
      printf("  purpose: @%d\n",g.nodeBuffer[nodenum].tree_list.purpose);
      printf("  value: @%d\n",g.nodeBuffer[nodenum].tree_list.value);
      printf("  chan: @%d\n",g.nodeBuffer[nodenum].tree_list.chan);
      break;
    case gTREE_VEC:
      printf("Type: tree_vec\n");
      about_common(g.nodeBuffer[nodenum].common);
      printf("  binfo: %d\n",g.nodeBuffer[nodenum].tree_vec.binfo);
      printf("  inherit: @%d\n",g.nodeBuffer[nodenum].tree_vec.inherit);
      printf("  type: @%d\n",g.nodeBuffer[nodenum].tree_vec.type);
      printf("  base: @%d\n",g.nodeBuffer[nodenum].tree_vec.base);
      printf("  pub: %d\n",g.nodeBuffer[nodenum].tree_vec.pub);
      printf("  prot: %d\n",g.nodeBuffer[nodenum].tree_vec.prot);
      printf("  priv: %d\n",g.nodeBuffer[nodenum].tree_vec.priv);
      printf("  virt: %d\n",g.nodeBuffer[nodenum].tree_vec.virt);
      break;
    case gERROR_MARK:
      printf("Type: namespace_decl\n");
      about_common(g.nodeBuffer[nodenum].common);
      break;
    case gDEFAULT_ARG:
      printf("Type: default_arg\n");
      about_common(g.nodeBuffer[nodenum].common);
      break;
    case gCPLUS_BINDING:
      printf("Type: cplus_binding\n");
      about_common(g.nodeBuffer[nodenum].common);
      break;
    case gOVERLOAD:
      printf("Type: overload\n");
      about_common(g.nodeBuffer[nodenum].common);
      printf("  crnt: @%d\n",g.nodeBuffer[nodenum].overload.crnt);
      printf("  chan: @%d\n",g.nodeBuffer[nodenum].overload.chan);
      break;
    case gWRAPPER:
      printf("Type: wrapper\n");
      about_common(g.nodeBuffer[nodenum].common);
      break;
    case NODE_DELTED:
      printf("Type: deleted node\n");
      break;
    case NODE_NONE:
      printf("Type: unassigned node (NODE_NONE)\n");
      break;
      /* datrix types */
    case dC_ASG_ND:
      printf("Type: dc_asg_nd\n");
      datrix_asg_nd(g, g.nodeBuffer[nodenum].dc_asg_nd);
      break;
    case dC_IDENTIFIER:
      printf("Type: dc_identifier\n");
      datrix_identifier(g, g.nodeBuffer[nodenum].dc_identifier);
      break;
    case dC_TYPE:
      printf("Type: dc_type\n");
      datrix_type(g, g.nodeBuffer[nodenum].dc_type);
      break;
    case dC_BUILT_IN_TYPE:
      printf("Type: dc_built_in_type\n");
      datrix_type(g, g.nodeBuffer[nodenum].dc_type);
      break;
    case dC_AGGR_TYPE:
      printf("Type: dc_aggr_type\n");
      datrix_type(g, g.nodeBuffer[nodenum].dc_type);
      switch(g.nodeBuffer[nodenum].dc_aggr_type.aggr_type){
	case aggr_class:
	  printf(" aggr_type: class\n");
	  break;
	case aggr_record:
	  printf(" aggr_type: record\n");
	  break;
	case aggr_struct:
	  printf(" aggr_type: struct\n");
	  break;
	case aggr_union:
	  printf(" aggr_type: union\n");
	  break;
	case aggr_area:
	  printf(" aggr_type: area\n");
	  break;
	case aggr_interface:
	  printf(" aggr_type: interface\n");
	  break;
	default:
	  fprintf(stderr,"Unknown datrix aggregate type %d\n",
	      g.nodeBuffer[nodenum].dc_aggr_type.aggr_type);
	  exit(1);
      }
      printf(" members: @%d\n",g.nodeBuffer[nodenum].dc_aggr_type.members);
      printf(" inheritance: @%d\n",g.nodeBuffer[nodenum].dc_aggr_type.inheritance);
      break;
    case dC_ENUM_TYPE:
      printf("Type: dc_enum_type\n");
      datrix_type(g, g.nodeBuffer[nodenum].dc_type);
      printf(" values: @%d\n",g.nodeBuffer[nodenum].dc_enum_type.values);
      break;
    case dC_TEMPL_PARM_TYPE:
      printf("Type: dc_templ_parm_type\n");
      datrix_type(g, g.nodeBuffer[nodenum].dc_type);
      break;
    case dC_GENERIC_TYPE:
      printf("Type: dc_generic_type\n");
      datrix_type(g, g.nodeBuffer[nodenum].dc_type);
      printf(" data_functs: @%d\n",g.nodeBuffer[nodenum].dc_generic_type.data_functs);
      printf(" formals: @%d\n",g.nodeBuffer[nodenum].dc_generic_type.formals);
      break;
    case dC_GENERATED_TYPE:
      printf("Type: dc_generated_type\n");
      datrix_type(g, g.nodeBuffer[nodenum].dc_type);
      printf("  actuals: @%d\n",g.nodeBuffer[nodenum].dc_generated_type.actuals);
      break;
    case dC_ALIAS_TYPE:
      printf("Type: dc_alias_type\n");
      datrix_type(g, g.nodeBuffer[nodenum].dc_type);
      printf(" original: @%d\n",g.nodeBuffer[nodenum].dc_alias_type.original);
      break;
    case dC_FORWARD_TYPE:
      printf("Type: dc_forward_type\n");
      datrix_type(g, g.nodeBuffer[nodenum].dc_type);
      break;
    case dC_ARRAY_TYPE:
      printf("Type: dc_array_type\n");
      datrix_type(g, g.nodeBuffer[nodenum].dc_type);
      break;
   case dC_PTR_TYPE:
      printf("Type: dc_ptr_type\n");
      datrix_type(g, g.nodeBuffer[nodenum].dc_type);
      break;
    case dC_REF_TYPE:
      printf("Type: dc_ref_type\n");
      datrix_type(g, g.nodeBuffer[nodenum].dc_type);
      break;
    case dC_FCT_TYPE:
      printf("Type: dc_fct_type\n");
      datrix_type(g, g.nodeBuffer[nodenum].dc_type);
      printf(" formals: @%d\n",g.nodeBuffer[nodenum].dc_fct_type.formalParams);
      break;
    case dC_SCOPE:
      printf("Type: dc_scope\n");
      datrix_scope(g, g.nodeBuffer[nodenum].dc_scope);
      break;
    case dC_SCOPE_GLB:
      printf("Type: dc_scope_glb\n");
      datrix_scope(g, g.nodeBuffer[nodenum].dc_scope);
      break;
    case dC_SCOPE_COMPIL:
      printf("Type: dc_scope_compil\n");
      datrix_scope(g, g.nodeBuffer[nodenum].dc_scope);
      break;
    case dC_SCOPE_FILE:
      printf("Type: dc_scope_file\n");
      datrix_scope(g, g.nodeBuffer[nodenum].dc_scope);
      break;
    case dC_PACKAGE:
      printf("Type: dc_package\n");
      datrix_scope(g, g.nodeBuffer[nodenum].dc_scope);
      break;
    case dC_NAME_SPACE:
      printf("Type: dc_name_space\n");
      datrix_scope(g, g.nodeBuffer[nodenum].dc_scope);
      break;
    case dC_BLOCK:
      printf("Type: dc_block\n");
      datrix_scope(g, g.nodeBuffer[nodenum].dc_scope);
      break;
    case dC_STATIC_BLOCK:
      printf("Type: dc_static_block\n");
      datrix_scope(g, g.nodeBuffer[nodenum].dc_scope);
      break;
    case dC_TRY_BLOCK:
      printf("Type: dc_try_block\n");
      datrix_scope(g, g.nodeBuffer[nodenum].dc_scope);
      break;
    case dC_CATCH_BLOCK:
      printf("Type: dc_catch_block\n");
      datrix_scope(g, g.nodeBuffer[nodenum].dc_scope);
      break;
    case dC_FINALLY_BLOCK:
      printf("Type: dc_finally_block\n");
      datrix_scope(g, g.nodeBuffer[nodenum].dc_scope);
      break;
    case dC_SYNCHRONIZED_BLOCK:
      printf("Type: dc_synchronized_block\n");
      datrix_scope(g, g.nodeBuffer[nodenum].dc_scope);
      break;
    case dC_OBJECT:
      printf("Type: dc_object\n");
      datrix_object(g, g.nodeBuffer[nodenum].dc_object);
      break;
    case dC_FORMAL_FCT_PARAM:
      printf("Type: dc_formal_fct_param\n");
      datrix_object(g, g.nodeBuffer[nodenum].dc_object);
      break;
    case dC_FORMAL_CATCH_PARAM:
      printf("Type: dc_formal_catch_param\n");
      datrix_object(g, g.nodeBuffer[nodenum].dc_object);
      break;
    case dC_FUNCTION:
      printf("Type: dc_function\n");
      datrix_identifier(g, g.nodeBuffer[nodenum].dc_identifier);
      printf("  static: %d\n",g.nodeBuffer[nodenum].dc_function.static_flag);
      printf("  extern: %d\n",g.nodeBuffer[nodenum].dc_function.extern_flag);
      printf("  const: %d\n",g.nodeBuffer[nodenum].dc_function.const_flag);
      switch(g.nodeBuffer[nodenum].dc_function.virtual_code){
	case 0: 
	  printf("  virtual: non-virtual\n");
	  break;
	case 1: 
	  printf("  virtual: virtual\n");
	  break;
	case 2: 
	  printf("  virtual: pure-virtual\n");
	  break;
	case 3:
	  fprintf(stderr,"unknown virual flag in function\n");
	  exit(1);
      }
      printf(" formal_fct: @%d\n",g.nodeBuffer[nodenum].dc_function.formal_fct);
      printf(" exception: @%d\n",g.nodeBuffer[nodenum].dc_function.exception);
      printf(" block: @%d\n",g.nodeBuffer[nodenum].dc_function.block);
      break;
    case dC_LABEL:
      printf("Type: dc_label\n");
      datrix_label(g, g.nodeBuffer[nodenum].dc_label);
      break;
    case dC_CASE_LABEL:
      printf("Type: dc_case_label\n");
      datrix_label(g, g.nodeBuffer[nodenum].dc_label);
      printf(" value: @%d\n",g.nodeBuffer[nodenum].dc_case_label.value);
      break;
    case dC_DEFAULT_LABEL:
      printf("Type: dc_default_label\n");
      datrix_label(g, g.nodeBuffer[nodenum].dc_label);
      break;
    case dC_ENUMERATOR:
      printf("Type: dc_enumerator\n");
      datrix_identifier(g, g.nodeBuffer[nodenum].dc_identifier);
      printf(" value: @%d\n",g.nodeBuffer[nodenum].dc_enumerator.value);
      break;
    case dC_EXPRESSION:
      printf("Type: dc_expression\n");
      datrix_expression(g, g.nodeBuffer[nodenum].dc_expression);
      break;
    case dC_EMPTY_EXPR:
      printf("Type: dc_empty_expr\n");
      datrix_expression(g, g.nodeBuffer[nodenum].dc_expression);
      break;
    case dC_LITERAL:
      printf("Type: dc_literal\n");
      datrix_expression(g, g.nodeBuffer[nodenum].dc_expression);
      printf(" value: %s\n",&(g.stringBuffer[g.nodeBuffer[nodenum].dc_literal.value]));
      printf(" type: @%d\n",g.nodeBuffer[nodenum].dc_literal.type);
      break;
    case dC_SYNTAX_ERROR:
      printf("Type: dc_syntax_error\n");
      datrix_expression(g, g.nodeBuffer[nodenum].dc_expression);
      break;
    case dC_NAME_REF:
      printf("Type: dc_name_ref\n");
      datrix_expression(g, g.nodeBuffer[nodenum].dc_expression);
      printf(" name: %s\n",&(g.stringBuffer[g.nodeBuffer[nodenum].dc_name_ref.name]));
      break;
    case dC_OPERATOR:
      printf("Type: dc_operator\n");
      datrix_operator(g, g.nodeBuffer[nodenum].dc_operator);
      break;
    case dC_UNARY_OPERATOR:
      printf("Type: dc_unary_operator\n");
      datrix_operator(g, g.nodeBuffer[nodenum].dc_operator);
      break;
    case dC_BINARY_OPERATOR:
      printf("Type: dc_binary_operator\n");
      datrix_operator(g, g.nodeBuffer[nodenum].dc_operator);
      break;
    case dC_QUESTION_OPERATOR:
      printf("Type: dc_question_operator\n");
      datrix_operator(g, g.nodeBuffer[nodenum].dc_operator);
      break;
    case dC_INIT_LIST:
      printf("Type: dc_init_list\n");
      datrix_expression(g, g.nodeBuffer[nodenum].dc_expression);
      printf(" init: @%d\n",g.nodeBuffer[nodenum].dc_init_list.init);
      break;
    case dC_ACTUAL_FCT_PARAM_LIST:
      printf("Type: dc_actual_fct_param_list\n");
      datrix_expression(g, g.nodeBuffer[nodenum].dc_expression);
      printf(" actuals: @%d\n",g.nodeBuffer[nodenum].dc_actual_fct_param_list.actuals);
      break;
    case dC_ACTUAL_TEMPL_PARAM_LIST:
      printf("Type: dc_actual_templ_param_list\n");
      datrix_expression(g, g.nodeBuffer[nodenum].dc_expression);
      printf(" actuals: @%d\n",g.nodeBuffer[nodenum].dc_actual_templ_param_list.actuals);
      break;
    case dC_CREATION_EXPR:
      printf("Type: dc_creation_expr\n");
      datrix_expression(g, g.nodeBuffer[nodenum].dc_expression);
      printf(" init: @%d\n",g.nodeBuffer[nodenum].dc_creation_expr.init);
      break;
    case dC_DESTROY_EXPR:
      printf("Type: dc_destroy_expr\n");
      datrix_expression(g, g.nodeBuffer[nodenum].dc_expression);
      printf(" what: @%d\n",g.nodeBuffer[nodenum].dc_destroy_expr.what);
      printf(" array: @%d\n",g.nodeBuffer[nodenum].dc_destroy_expr.array);
      break;
    case dC_CNTRL_STMT:
      printf("Type: dc_cntrl_stmt\n");
      datrix_ctrl_stmt(g, g.nodeBuffer[nodenum].dc_ctrl_stmt);
      break;
    case dC_SELECTION:
      printf("Type: dc_selection\n");
      datrix_ctrl_stmt(g, g.nodeBuffer[nodenum].dc_ctrl_stmt);
      break;
    case dC_IF:
      printf("Type: dc_if\n");
      datrix_ctrl_stmt(g, g.nodeBuffer[nodenum].dc_ctrl_stmt);
      printf(" cond: @%d\n",g.nodeBuffer[nodenum].dc_if.cond);
      printf(" true: @%d\n",g.nodeBuffer[nodenum].dc_if.true_block);
      printf(" false: @%d\n",g.nodeBuffer[nodenum].dc_if.false_block);
      break;
    case dC_SWITCH:
      printf("Type: dc_switch\n");
      datrix_ctrl_stmt(g, g.nodeBuffer[nodenum].dc_ctrl_stmt);
      printf(" cond: @%d\n",g.nodeBuffer[nodenum].dc_switch.cond);
      printf(" body: @%d\n",g.nodeBuffer[nodenum].dc_switch.body);
      break;
    case dC_JUMP:
      printf("Type: dc_jump\n");
      datrix_jump(g, g.nodeBuffer[nodenum].dc_jump);
      break;
    case dC_GOTO:
      printf("Type: dc_goto\n");
      datrix_jump(g, g.nodeBuffer[nodenum].dc_jump);
      printf(" dest: @%d\n",g.nodeBuffer[nodenum].dc_goto.dest);
      break;
    case dC_RETURN:
      printf("Type: dc_return\n");
      datrix_jump(g, g.nodeBuffer[nodenum].dc_jump);
      printf(" value: @%d\n",g.nodeBuffer[nodenum].dc_return.value);
      break;
    case dC_BREAK:
      printf("Type: dc_break\n");
      datrix_jump(g, g.nodeBuffer[nodenum].dc_jump);
      printf(" dest: @%d\n",g.nodeBuffer[nodenum].dc_break.dest);
      break;
    case dC_CONTINUE:
      printf("Type: dc_continue\n");
      datrix_jump(g, g.nodeBuffer[nodenum].dc_jump);
      printf(" dest: @%d\n",g.nodeBuffer[nodenum].dc_continue.dest);
      break;
    case dC_LOOP:
      printf("Type: dc_loop\n");
      datrix_loop(g, g.nodeBuffer[nodenum].dc_loop);
      break;
    case dC_DO_WHILE_LOOP:
      printf("Type: dc_do_while_loop\n");
      datrix_loop(g, g.nodeBuffer[nodenum].dc_loop);
      printf(" cond: @%d\n",g.nodeBuffer[nodenum].dc_do_while_loop.cond);
      printf(" block: @%d\n",g.nodeBuffer[nodenum].dc_do_while_loop.block);
      break;
    case dC_WHILE_DO_LOOP:
      printf("Type: dc_while_do_loop\n");
      datrix_loop(g, g.nodeBuffer[nodenum].dc_loop);
      printf(" cond: @%d\n",g.nodeBuffer[nodenum].dc_while_do_loop.cond);
      printf(" block: @%d\n",g.nodeBuffer[nodenum].dc_while_do_loop.block);
      break;
    case dC_FOR_LOOP:
      printf("Type: dc_for_loop\n");
      datrix_loop(g, g.nodeBuffer[nodenum].dc_loop);
      printf(" cond: @%d\n",g.nodeBuffer[nodenum].dc_for_loop.cond);
      printf(" block: @%d\n",g.nodeBuffer[nodenum].dc_for_loop.block);
      printf(" init: @%d\n",g.nodeBuffer[nodenum].dc_for_loop.init);
      printf(" incr: @%d\n",g.nodeBuffer[nodenum].dc_for_loop.incr);
      break;
    case dC_SQL_STMT:
      printf("Type: dc_sql_stmt\n");
      datrix_asg_nd(g, g.nodeBuffer[nodenum].dc_asg_nd);
      break;
    case dC_COMMENT:
      printf("Type: dc_comment\n");
      datrix_comment(g, g.nodeBuffer[nodenum].dc_comment);
      break;
    case dC_JAVA_DOC_COMMENT:
      printf("Type: dc_java_doc_comment\n");
      datrix_comment(g, g.nodeBuffer[nodenum].dc_comment);
      break;
    case dC_THROW_SPEC:
      printf("Type: dc_throw_spec\n");
      datrix_asg_nd(g, g.nodeBuffer[nodenum].dc_asg_nd);
      printf(" except: @%d\n",g.nodeBuffer[nodenum].dc_throw_spec.except);
      break;
    case dC_ASM_BLOCK:
      printf("Type: dc_asm_block\n");
      datrix_asg_nd(g, g.nodeBuffer[nodenum].dc_asg_nd);
      break;
    case dC_ARC:
      printf("Type: Arc @%d @%d\n",
	  g.nodeBuffer[nodenum].dc_arc.src,
	  g.nodeBuffer[nodenum].dc_arc.dest);
      datrix_arc(g.nodeBuffer[nodenum].dc_arc);
      break;
    case dC_ARC_REFERENCE:
      printf("Type: ArcReference @%d @%d\n",
	  g.nodeBuffer[nodenum].dc_arc.src,
	  g.nodeBuffer[nodenum].dc_arc.dest);
      datrix_arc_reference(g.nodeBuffer[nodenum].dc_arc_reference);
      break;
    case dC_INSTANCE:
      printf("Type: Instance @%d @%d\n",
	  g.nodeBuffer[nodenum].dc_arc.src,
	  g.nodeBuffer[nodenum].dc_arc.dest);
      datrix_arc_reference(g.nodeBuffer[nodenum].dc_arc_reference);
      break;
    case dC_INHERITANCE:
      printf("Type: Inheritance @%d @%d\n",
	  g.nodeBuffer[nodenum].dc_arc.src,
	  g.nodeBuffer[nodenum].dc_arc.dest);
      datrix_arc_reference(g.nodeBuffer[nodenum].dc_arc_reference);
      printf("  virt: %d\n",g.nodeBuffer[nodenum].dc_inheritance.virt);
      switch (g.nodeBuffer[nodenum].dc_inheritance.visb){
	case visb_unk:
	  printf("  visb: unkown\n");
	  break;
	case visb_pri:
	  printf("  visb: private\n");
	  break;
	case visb_pro:
	  printf("  visb: protected\n");
	  break;
	case visb_pub:
	  printf("  visb: public\n");
	  break;
	default:
	  fprintf(stderr,"Illegal visibility\n");
	  exit(1);
      }
      break;
    case dC_IMPLEMENTS:
      printf("Type: Implements @%d @%d\n",
	  g.nodeBuffer[nodenum].dc_arc.src,
	  g.nodeBuffer[nodenum].dc_arc.dest);
      datrix_arc_reference(g.nodeBuffer[nodenum].dc_arc_reference);
      break;
    case dC_ARC_FRIEND:
      printf("Type: ArcFriend @%d @%d\n",
	  g.nodeBuffer[nodenum].dc_arc.src,
	  g.nodeBuffer[nodenum].dc_arc.dest);
      datrix_arc_reference(g.nodeBuffer[nodenum].dc_arc_reference);
      break;
    case dC_ARC_DEF_LOC:
      printf("Type: ArcDefLoc @%d @%d\n",
	  g.nodeBuffer[nodenum].dc_arc.src,
	  g.nodeBuffer[nodenum].dc_arc.dest);
      datrix_arc_reference(g.nodeBuffer[nodenum].dc_arc_reference);
      break;
    case dC_REFERS_TO:
      printf("Type: RefersTo @%d @%d\n",
	  g.nodeBuffer[nodenum].dc_arc.src,
	  g.nodeBuffer[nodenum].dc_arc.dest);
      datrix_arc_reference(g.nodeBuffer[nodenum].dc_arc_reference);
      break;
    case dC_ARC_SYNTAX:
      printf("Type: ArcSyntax @%d @%d\n",
	  g.nodeBuffer[nodenum].dc_arc.src,
	  g.nodeBuffer[nodenum].dc_arc.dest);
      datrix_arc_syntax(g.nodeBuffer[nodenum].dc_arc_syntax);
      break;
    case dC_ARC_SON:
      printf("Type: ArcSon @%d @%d\n",
	  g.nodeBuffer[nodenum].dc_arc.src,
	  g.nodeBuffer[nodenum].dc_arc.dest);
      datrix_arc_syntax(g.nodeBuffer[nodenum].dc_arc_syntax);
      break;
    case dC_ARC_TRUE:
      printf("Type: ArcTrue @%d @%d\n",
	  g.nodeBuffer[nodenum].dc_arc.src,
	  g.nodeBuffer[nodenum].dc_arc.dest);
      datrix_arc_syntax(g.nodeBuffer[nodenum].dc_arc_syntax);
      break;
    case dC_ARC_FALSE:
      printf("Type: ArcFalse @%d @%d\n",
	  g.nodeBuffer[nodenum].dc_arc.src,
	  g.nodeBuffer[nodenum].dc_arc.dest);
      datrix_arc_syntax(g.nodeBuffer[nodenum].dc_arc_syntax);
      break;
    case dC_ARC_OPD:
      printf("Type: ArcOpd @%d @%d\n",
	  g.nodeBuffer[nodenum].dc_arc.src,
	  g.nodeBuffer[nodenum].dc_arc.dest);
      datrix_arc_syntax(g.nodeBuffer[nodenum].dc_arc_syntax);
      break;
    case dC_IMPORT:
      printf("Type: Import @%d @%d\n",
	  g.nodeBuffer[nodenum].dc_arc.src,
	  g.nodeBuffer[nodenum].dc_arc.dest);
      datrix_arc_syntax(g.nodeBuffer[nodenum].dc_arc_syntax);
      break;
    case dC_ARC_COND:
      printf("Type: ArcCond @%d @%d\n",
	  g.nodeBuffer[nodenum].dc_arc.src,
	  g.nodeBuffer[nodenum].dc_arc.dest);
      datrix_arc_syntax(g.nodeBuffer[nodenum].dc_arc_syntax);
      break;
    case dC_ARC_FOR_INIT:
      printf("Type: ArcForInit @%d @%d\n",
	  g.nodeBuffer[nodenum].dc_arc.src,
	  g.nodeBuffer[nodenum].dc_arc.dest);
      datrix_arc_syntax(g.nodeBuffer[nodenum].dc_arc_syntax);
      break;
    case dC_ARC_FOR_INCR:
      printf("Type: ArcForIncr @%d @%d\n",
	  g.nodeBuffer[nodenum].dc_arc.src,
	  g.nodeBuffer[nodenum].dc_arc.dest);
      datrix_arc_syntax(g.nodeBuffer[nodenum].dc_arc_syntax);
      break;
    case dC_ARC_INIT_VAL:
      printf("Type: ArcInitVal @%d @%d\n",
	  g.nodeBuffer[nodenum].dc_arc.src,
	  g.nodeBuffer[nodenum].dc_arc.dest);
      datrix_arc_syntax(g.nodeBuffer[nodenum].dc_arc_syntax);
      break;
    case dC_ARC_BIT_DIM:
      printf("Type: ArcBitDim @%d @%d\n",
	  g.nodeBuffer[nodenum].dc_arc.src,
	  g.nodeBuffer[nodenum].dc_arc.dest);
      datrix_arc_syntax(g.nodeBuffer[nodenum].dc_arc_syntax);
      break;
    case dC_ARC_ARRAY_DIM:
      printf("Type: ArcArrayDim @%d @%d\n",
	  g.nodeBuffer[nodenum].dc_arc.src,
	  g.nodeBuffer[nodenum].dc_arc.dest);
      datrix_arc_syntax(g.nodeBuffer[nodenum].dc_arc_syntax);
      break;
    default: {
      int i;
      unsigned char *x;
      printf("unknown node type: %d\n" ,g.nodeBuffer[nodenum].common.type);
      printf("node data:\n");
      x = (unsigned char *)(&(g.nodeBuffer[nodenum]));
      i = 0;
      while (i < sizeof(gd_graph_node)){
	printf(" %2x",*x);
	x++;
	i++;
	if (i % 16 == 0) {
	  printf("\n");
	}
      }
      printf("\n");
    }
  }

  return 0;

}

void datrix_asg_nd(gd_graph g, t_dc_asg_nd nd){
  /* for now nothing*/
  about_common(nd.common);
  printf("asg_nd\n");
  if (nd.file >= 0 && nd.file < g.curStringPos){
    printf(" file: %s\n",&(g.stringBuffer[nd.file]));
  }
  printf(" line: %d\n",nd.line);
}

void datrix_identifier(gd_graph g, t_dc_identifier ident){
  datrix_asg_nd(g, ident.dc_asg_nd);
  printf("identifier\n");
  if (ident.name != -1){
    printf(" name: %s(%d)\n",&(g.stringBuffer[ident.name]),ident.name);
  }
  switch (ident.visb){
    case visb_unk:
      printf(" visb: unkown\n");
      break;
    case visb_pri:
      printf(" visb: private\n");
      break;
    case visb_pro:
      printf(" visb: protected\n");
      break;
    case visb_pub:
      printf(" visb: public\n");
      break;
    default:
      fprintf(stderr,"Illegal visibility\n");
      exit(1);
  }
  printf(" instance: @%d\n",ident.instance);
  printf(" defLoc: @%d\n",ident.defLoc);
  printf(" arcFriend: @%d\n",ident.arcFriend);
  printf(" array_dim: @%d\n",ident.array_dim);
}

void datrix_loop(gd_graph g, t_dc_loop loop){
  datrix_ctrl_stmt(g, loop.ctrl_stmt);
}

void datrix_ctrl_stmt(gd_graph g, t_dc_ctrl_stmt cntrl){
  datrix_asg_nd(g, cntrl.dc_asg_nd);
}

void datrix_comment(gd_graph g, t_dc_comment comment){
  datrix_asg_nd(g, comment.dc_asg_nd);
}

void datrix_jump(gd_graph g, t_dc_jump jump) {
  datrix_ctrl_stmt(g,jump.ctrl_stmt);
}

void datrix_expression(gd_graph g, t_dc_expression expr){
  datrix_asg_nd(g, expr.dc_asg_nd);
  printf("expression\n");
  printf(" refers: @%d\n",expr.refers);
}

void datrix_operator(gd_graph g, t_dc_operator operator){
  datrix_expression(g, operator.expression);
  printf("operator\n");
  if (operator.op < 0 || operator.op > dc_range){
    fprintf(stderr,"unknown operator %d\n",operator.op);
  }
  printf(" op",dc_op_string[operator.op]);
  printf(" ops: @%d\n",operator.ops);
}

void datrix_type(gd_graph g, t_dc_type type){
  datrix_identifier(g, type.dc_identifier);
  printf(" implements; @%d\n",type.implements);
}

void datrix_object(gd_graph g, t_dc_object object){
  datrix_identifier(g, object.dc_identifier);
  printf("object\n");
  printf(" static: %d\n",object.static_flag);
  printf(" extern: %d\n",object.extern_flag);
  printf(" const: %d\n",object.const_flag);
  printf(" bitfield_dim: @%d\n",object.bitfield_dim);
  printf(" initial_val: @%d\n",object.initial_val);
}

void datrix_label(gd_graph g, t_dc_label label){
  datrix_identifier(g, label.dc_identifier);
}

void datrix_scope(gd_graph g, t_dc_scope scope){
  datrix_identifier(g, scope.dc_identifier);
  printf("scope\n");
  printf(" contents: @%d\n",scope.contents);
}

void datrix_arc(t_dc_arc  dc_arc){
  about_common(dc_arc.common);
  printf("arc\n");
  if (dc_arc.ordinal != 0){
    printf(" ordinal %d\n", dc_arc.ordinal);
  }
  printf(" next: @%d\n", dc_arc.next);
}

void datrix_arc_reference(t_dc_arc_reference  dc_arc_reference){
  datrix_arc(dc_arc_reference.arc);
}

void datrix_arc_syntax(t_dc_arc_syntax  dc_arc_syntax){
  datrix_arc(dc_arc_syntax.arc);
}


void about_decl(gd_graph g, t_gcc_decl decl) {
  about_common(decl.common);
  printf("decl\n");
  printf(" name: @%d\n",decl.name);
  printf(" mngl: @%d\n",decl.mngl);
  printf(" type: @%d\n",decl.type);
  printf(" chan: @%d\n",decl.chan);
  printf(" scpe: @%d\n",decl.scpe);
  if (decl.srcl == -1) {
    printf(" srcf: <none>\n");
    printf(" srcl: <none>\n");
  } else{
    if (decl.srcf >= g.curStringPos){
      fprintf(stderr,"string index is out of bounds\n");
      exit(1);
    }
    printf(" srcf: %s\n",&(g.stringBuffer[decl.srcf]));
    printf(" srcl: %d\n",decl.srcl);
  }
  switch (decl.lang) {
    case src_lang_c:
      printf(" src_lang: c\n");
      break;
    case src_lang_cp:
      printf(" src_lang: c++\n");
      break;
    case src_lang_java:
      printf(" src_lang: java\n");
      break;
    default:
  }
  printf(" artificial: %d\n",decl.artificial);
}

void about_common(gd_graph_common comm){
  printf("common");
  printf(" Type_number: %d\n",comm.type);
  printf(" unsigned_flag: %d\n", comm.unsigned_flag);
  printf(" constant_flag: %d\n", comm.constant_flag);
  printf(" static_flag: %d\n", comm.static_flag);
  printf(" public_flag: %d\n", comm.public_flag);
  printf(" private_flag: %d\n", comm.private_flag);
  printf(" protected_flag: %d\n", comm.protected_flag);
  printf(" other_node: @%d\n",comm.other_node);
}

void about_type(t_gcc_type type) {
  about_common(type.common);
  printf("type\n");
  printf(" name: @%d\n",type.name);
  printf(" size: @%d\n",type.size);
  printf(" algn: %d\n",type.algn);
  printf(" unql: @%d\n",type.unql);
  printf(" qual: %c%c%c\n",
      	type.qual_const?'c':' ',
	type.qual_volatile?'v':' ',
	type.qual_restrict?'r':' ');
}

void about_cst(t_gcc_const cst) {
  about_common(cst.common);
  printf("cst\n");
  printf(" type: @%d\n",cst.type);
}

void about_expr(t_gcc_expr expr) {
  about_common(expr.common);
  printf("expr\n");
  printf(" type: @%d\n",expr.type);
}
